@@ -4,17 +4,20 @@ license: GPL
 """
 #============================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/business/gmPathLab.py,v $
-# $Id: gmPathLab.py,v 1.52 2006/07/19 20:25:00 ncq Exp $
-__version__ = "$Revision: 1.52 $"
+# $Id: gmPathLab.py,v 1.54 2007/01/09 12:56:18 ncq Exp $
+__version__ = "$Revision: 1.54 $"
 __author__ = "K.Hilbert <Karsten.Hilbert@gmx.net>"
 
 import types, sys
 
-from Gnumed.pycommon import gmLog, gmPG, gmExceptions
+from Gnumed.pycommon import gmLog, gmExceptions
 from Gnumed.business import gmClinItem
 
 _log = gmLog.gmDefLog
 _log.Log(gmLog.lInfo, __version__)
+
+# FIXME: use psyopg2 dbapi extension of named cursors - they are *server* side !
+
 #============================================================
 # FIXME: TODO
 class cTestResult(gmClinItem.cClinItem):
@@ -343,7 +346,7 @@ def create_test_type(lab=None, code=None, unit=None, name=None):
 		# yes but ambigous
 		if name != db_lname:
 			_log.Log(gmLog.lErr, 'test type found for [%s:%s] but long name mismatch: expected [%s], in DB [%s]' % (lab, code, name, db_lname))
-			me = '$RCSfile: gmPathLab.py,v $ $Revision: 1.52 $'
+			me = '$RCSfile: gmPathLab.py,v $ $Revision: 1.54 $'
 			to = 'user'
 			prob = _('The test type already exists but the long name is different. '
 					'The test facility may have changed the descriptive name of this test.')
@@ -430,7 +433,7 @@ def create_lab_request(lab=None, req_id=None, pat_id=None, encounter_id=None, ep
 		# yes but ambigous
 		if pat_id != db_pat[0]:
 			_log.Log(gmLog.lErr, 'lab request found for [%s:%s] but patient mismatch: expected [%s], in DB [%s]' % (lab, req_id, pat_id, db_pat))
-			me = '$RCSfile: gmPathLab.py,v $ $Revision: 1.52 $'
+			me = '$RCSfile: gmPathLab.py,v $ $Revision: 1.54 $'
 			to = 'user'
 			prob = _('The lab request already exists but belongs to a different patient.')
 			sol = _('Verify which patient this lab request really belongs to.')
@@ -686,6 +689,13 @@ if __name__ == '__main__':
 	gmPG.ConnectionPool().StopListeners()
 #============================================================
 # $Log: gmPathLab.py,v $
+# Revision 1.54  2007/01/09 12:56:18  ncq
+# - comment
+#
+# Revision 1.53  2006/10/25 07:17:40  ncq
+# - no more gmPG
+# - no more cClinItem
+#
 # Revision 1.52  2006/07/19 20:25:00  ncq
 # - gmPyCompat.py is history
 #
