@@ -2,20 +2,19 @@
 """
 #============================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/business/gmVaccination.py,v $
-# $Id: gmVaccination.py,v 1.31 2006/07/19 20:25:00 ncq Exp $
-__version__ = "$Revision: 1.31 $"
+# $Id: gmVaccination.py,v 1.33 2006/11/24 14:15:36 ncq Exp $
+__version__ = "$Revision: 1.33 $"
 __author__ = "K.Hilbert <Karsten.Hilbert@gmx.net>"
 __license__ = "GPL"
 
 import types, copy
 
-from Gnumed.pycommon import gmLog, gmExceptions, gmI18N, gmPG
-from Gnumed.business import gmClinItem
+from Gnumed.pycommon import gmLog, gmExceptions, gmI18N, gmBusinessDBObject
 
 _log = gmLog.gmDefLog
 _log.Log(gmLog.lInfo, __version__)
 #============================================================
-class cVaccination(gmClinItem.cClinItem):
+class cVaccination(gmBusinessDBObject.cBusinessDBObject):
 	"""Represents one vaccination event.
 	"""
 	_cmd_fetch_payload = """
@@ -152,7 +151,7 @@ class cMissingBooster(gmClinItem.cClinItem):
 class cScheduledVaccination(gmClinItem.cClinItem):
 	"""Represents one vaccination scheduled following a course.
 	"""
-	_cmd_fetch_payload = """select * from clin.v_vaccs_scheduled4pat where pk_vacc_def=%s"""
+	_cmd_fetch_payload = u"select * from clin.v_vaccs_scheduled4pat where pk_vacc_def=%s"
 	_cmds_lock_rows_for_update = []
 	_cmds_store_payload = ["""select 1"""]
 	_updatable_fields = []
@@ -545,6 +544,13 @@ if __name__ == '__main__':
 #	test_due_booster()
 #============================================================
 # $Log: gmVaccination.py,v $
+# Revision 1.33  2006/11/24 14:15:36  ncq
+# - u'' one query
+#
+# Revision 1.32  2006/10/25 07:17:40  ncq
+# - no more gmPG
+# - no more cClinItem
+#
 # Revision 1.31  2006/07/19 20:25:00  ncq
 # - gmPyCompat.py is history
 #
