@@ -5,13 +5,13 @@ This maps XDT fields in various ways.
 """
 #==============================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/business/gmXdtMappings.py,v $
-# $Id: gmXdtMappings.py,v 1.28 2006/07/13 21:00:32 ncq Exp $
-__version__ = "$Revision: 1.28 $"
+# $Id: gmXdtMappings.py,v 1.37 2007/01/21 15:13:46 ncq Exp $
+__version__ = "$Revision: 1.37 $"
 __author__ = "S.Hilbert, K.Hilbert"
 __license__ = "GPL"
 
 try:
-	_('dummy-no-need-to-translate')
+	_('dummy-no-need-to-translate-but-make-epydoc-happy')
 except NameError:
 	_ = lambda x:x
 #==============================================================
@@ -51,37 +51,28 @@ xdt_id_map = {
 	'0130':'Telfonnummer des SB',
 	'0131':'Telefaxnummer des SB',
 	'0132':'Release-Stand der Software',
-	#Arztnummer 
-	'0201':'Arztnummer',
-	#Praxistyp 
-	'0202':'Praxistyp',
-	#Arztname
-	'0203':'Arztname',
-	#Arztgruppe verbal 
-	'0204':'Arztgruppe verbal',
-	#street
-	'0205':'Strasse der Praxisadresse',
-	#postcode and city
-	'0206':'PLZ Ort',
-	#Arzt und Leistungskennzeichen
-	'0207':'Arzt mit Leistungskennzeichen',
-	#phone
-	'0208':'Telefonnummer',
-	#fax
-	'0209':'Telefaxnummer',
-	#modem
-	'0210':'Modemnummer',
-	'0211':'Arztname fr Leistungsdifferenzierung',
-	'0213':'Leistungskennzeichen',
-	'0214':'Erluterung zum Leistungskennzeichen',
-	'0215':'PLZ der Praxisadresse',
-	'0216':'Ort der Praxisadresse',
-	#number of doctors
-	'0225':'Anzahl der rzte',
-	#name of first free category
+
+	'0201': 'Arztnummer',
+	'0202': 'Praxistyp',
+	'0203': 'Arztname',
+	'0204': 'Arztgruppe verbal',
+	'0205': 'Strasse der Praxisadresse',
+	'0206': 'PLZ Ort',
+	'0207': 'Arzt mit Leistungskennzeichen',
+	'0208': 'Telefonnummer',
+	'0209': 'Telefaxnummer',
+	'0210': ' Modemnummer',
+	'0211': 'Arztname fr Leistungsdifferenzierung',
+	'0213': 'Leistungskennzeichen',
+	'0214': 'Erluterung zum Leistungskennzeichen',
+	'0215': 'PLZ der Praxisadresse',
+	'0216': 'Ort der Praxisadresse',
+	'0218': 'E-Mail der Praxis/des Arztes',
+	'0225': 'Anzahl der rzte',
+
 	'0250':'Name erste freie Kategorie',
-	#content of first free category
 	'0251':'Inhalt erste freie Kategorie',
+
 	'0915':'PZN Medikament auf Kassenrezept',
 	'0917':'Packungsgrsse Medikament auf Kassenrezept',
 	'0918':'Packungsgrsse Medikament auf Privatrezept',
@@ -149,8 +140,8 @@ xdt_id_map = {
 	'3101':'Name des Patienten',
 	#Vorname des Patienten
 	'3102':'Vorname des Patienten',
-	#Geburstdatum des Patienten
-	'3103':'Geburstdatum des Patienten', 
+	#Geburtsdatum des Patienten
+	'3103':'Geburtsdatum des Patienten', 
 	#Titel des Patienten 
 	'3104':'Titel des Patienten',
 	#Versichertennummer des Patienten
@@ -584,8 +575,24 @@ xdt_id_map = {
 	'8000':'Satzidentifikation >>===============',
 	# Satzlnge
 	'8100': 'Satzlnge',
+
 	# LDT
+	'8301': 'UNBEKANNT Datum',       ## nicht in GDT 2.1 Specs (KS)
+	'8302': 'UNBEKANNT Datum',       ## nicht in GDT 2.1 Specs (KS)
+	'8303': 'UNBEKANNT Uhrzeit',     ## nicht in GDT 2.1 Specs (KS)
+
 	'8310': 'Anforderungsnummer', 
+
+	'8311': 'UNBEKANNT Integer lang',## nicht in GDT 2.1 Specs (KS)
+	'8312': 'UNBEKANNT Integer kurz',## nicht in GDT 2.1 Specs (KS)
+	'8315': 'GDT-ID Empfnger',
+	'8316': 'GDT-ID Sender',
+
+	'8320': 'Labor Bezeichnung',     ## nicht in GDT 2.1 Specs (KS)
+	'8321': 'Labor Strasse',         ## nicht in GDT 2.1 Specs (KS)
+	'8322': 'Labor PLZ',             ## nicht in GDT 2.1 Specs (KS)
+	'8323': 'Labor Ort',             ## nicht in GDT 2.1 Specs (KS)
+
 	# Befundstatus -- nur bei header 6200/8202 --
 	'8401': 'Befundstatus',
 	#Gerte- bzw. Verfahrensspezifisches Kennfeld -- nur bei header 6200 --
@@ -595,6 +602,8 @@ xdt_id_map = {
 	# LDT
 	'8404': 'Kosten in Doppelpfennigen',
 	'8406': 'Kosten in Cent',
+	'8407': 'UNBEKANNT Flag 1|2',    ## nicht in GDT 2.1 Specs (KS)
+
 	#Test-Ident -- nur bei header 6200 --
 	'8410':'Test-Ident',
 	#Testbezeichnung -- nur bei header 6200 --
@@ -607,6 +616,7 @@ xdt_id_map = {
 	'8421':'Einheit',
 	#Grenzwert Indikator -- nur bei header 6200 --
 	'8422':'Grenzwert Indikator',
+	'8428':'Probematerial-Ident',
 	#Probematerial-Nummer -- nur bei header 6200 --
 	'8429':'Probenmaterial-Nummer',
 	#Probenmaterial-Bezeichnung -- nur bei header 6200 --
@@ -635,6 +645,10 @@ xdt_id_map = {
 	'8447':'Resistenz-Interpretation',
 	#Normalwert-Text -- nur bei header 6200 --
 	'8460':'Normalwert-Text',
+
+	'8461':'Normalwert untere Grenze',
+	'8462':'Normalwert obere Grenze',
+
 	#Anmerkung -- nur bei header 6200 --
 	'8470':'Anmerkung',
 	#Ergebnis-Text -- nur bei header 6200 --
@@ -659,10 +673,12 @@ xdt_id_map = {
 	#number of media for this data package     
 	'9203':'Anzahl Datentrger im Paket',
 	'9204':'Abrechnungsquartal',
+	'9206': 'Zeichensatz (encoding)',
 	#ADT-version
-	'9210':'Version ADT-Satzbeschreibung',
-	'9212':'Version der Satzbeschreibung',
-	'9213':'Version BDT',
+	'9210': 'Version ADT-Satzbeschreibung',
+	'9212': 'Version der Satzbeschreibung',
+	'9213': 'Version BDT',
+	'9218': 'Version GDT',
 	'9233':'GO-Stammdatei-Version',
 	#way data is archived
 	'9600':'Archivierungsart',
@@ -690,6 +706,7 @@ xdt_packet_type_map = {
 	'0199': "========<< Fall: unstrukturiert >>========",
 	'6100': "========<< Patientenstamm >>========",
 	'6200': "========<< Behandlungsdaten >>========",
+	'6302': "========>> GDT: Anforderung einer neuen Untersuchung >>========",
 	'adt0': "========<< ADT-Datenpaket-Header >>========",
 	'adt9': "========<< ADT-Datenpaket-Abschluss >>========",
 	'con0': "========<< Container-Header >>========",
@@ -719,10 +736,13 @@ name_xdtID_map = {
 # sort of GNUmed compatible
 map_gender_xdt2gm = {
 	'1': 'm',
-	'2': 'f',
 	'm': 'm',
-	'f': 'f',
+	'M': 'm',
 	'4': 'm',
+	'2': 'f',
+	'f': 'f',
+	'W': 'f',
+	'w': 'f',
 	'5': 'f'
 }
 
@@ -739,11 +759,29 @@ map_8407_2str = {
 
 # xDT character code mapping : 9106
 xdt_character_code_map = {
-	'1':'7-bit-Code ASCII',
-	'2':'8-bit-Code ASCII',
-	##'2':'IBM-Code',
-	'3':'ISO 8859-1 Code'
+	'1': 'ASCII (DIN 66003/ISO 646)',
+	'2': 'cp437 (8 Bit)',
+	'3': 'ISO 8859-1/cp1252'
 }
+
+_charset_fields = [
+	'9106',			# LDT
+	'9206'			# GDT
+]
+
+_map_field2charset = {
+	'9106': {
+		'1': 'ascii',
+		'2': 'cp437',
+		'3': 'iso8859-1'
+	},
+	'9206': {
+		'1': 'ascii',
+		'2': 'cp437',
+		'3': 'iso8859-1'
+	}
+}
+
 # Archivierungsart : 9600
 xdt_Archivierungsart_map = {
 	'1':'Speicherung Gesamtbestand',
@@ -1353,6 +1391,33 @@ def xdt_8date2iso(date=None):
 	return '%s-%s-%s' % (date[-4:], date[2:4], date[:2])
 #==============================================================
 # $Log: gmXdtMappings.py,v $
+# Revision 1.37  2007/01/21 15:13:46  ncq
+# - add a few more codes
+#
+# Revision 1.36  2007/01/06 23:04:06  ncq
+# - missing ","
+#
+# Revision 1.35  2007/01/04 22:50:35  ncq
+# - add a mapping
+#
+# Revision 1.34  2006/12/27 16:39:57  ncq
+# - a few more gender codes as used by Medistar
+#
+# Revision 1.33  2006/12/17 15:00:31  ncq
+# - fixed typo
+#
+# Revision 1.32  2006/10/31 15:59:11  ncq
+# - somewhat improved mappings
+#
+# Revision 1.31  2006/10/31 12:01:17  ncq
+# - some xDT mappings provided by Kai Schmidt
+#
+# Revision 1.30  2006/10/30 16:41:12  ncq
+# - add _charset_fields
+#
+# Revision 1.29  2006/10/08 10:48:28  ncq
+# - teach xdt reader to derive encoding from gdt 6301 record
+#
 # Revision 1.28  2006/07/13 21:00:32  ncq
 # - cleanup gender mappings
 # - streamline cXdtPatient and improve test harness
