@@ -1,7 +1,7 @@
 #!/bin/python
 
 # $Source: /sources/gnumed/gnumed/check-prerequisites.py,v $
-# $Revision: 1.10 $
+# $Revision: 1.13 $
 
 import sys
 
@@ -17,14 +17,14 @@ except ImportError:
 	print "INFO :", "\nINFO : ".join(sys.path)
 	sys.exit(-1)
 
-print "=> checking for Python module 'pyPgSQL' ..."
+print "=> checking for Python module 'psycopg2' ..."
 try:
-	import pyPgSQL.PgSQL
+	import psycopg2
 	print "=> found"
 except ImportError:
-	print "ERROR: pyPgSQL not installed"
+	print "ERROR: psycopg2 not installed"
 	print "ERROR: this is needed to access PostgreSQL"
-	print "ERROR: pyPgSQL is available from http://pypgsql.sourceforge.net"
+	print "ERROR: psycopg2 is available from http://www.initd.org/"
 	print "INFO : sys.path is set as follows:"
 	print "INFO :", "\nINFO : ".join(sys.path)
 	sys.exit(-1)
@@ -115,6 +115,15 @@ sys.exit(0)
 
 #=================================================================
 # $Log: check-prerequisites.py,v $
+# Revision 1.13  2006/12/18 16:17:42  ncq
+# - do not check for pyPgSQL anymore
+#
+# Revision 1.12  2006/11/05 18:00:06  ncq
+# - drop check for pyPgSQL
+#
+# Revision 1.11  2006/09/01 15:31:13  ncq
+# - add check for psycopg2
+#
 # Revision 1.10  2006/08/09 14:05:28  ncq
 # - more unified output
 # - better wxversion/wxPython detection
