@@ -10,18 +10,18 @@ TODO:
 """
 #============================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/exporters/gmPatientExporter.py,v $
-# $Id: gmPatientExporter.py,v 1.81.2.1 2006/09/01 09:23:49 ncq Exp $
-__version__ = "$Revision: 1.81.2.1 $"
+# $Id: gmPatientExporter.py,v 1.96 2007/01/18 22:03:58 ncq Exp $
+__version__ = "$Revision: 1.96 $"
 __author__ = "Carlos Moro"
 __license__ = 'GPL'
 
-import os.path, sys, traceback, string, types, time
+import os.path, sys, traceback, string, types, time, codecs
 
 import mx.DateTime.Parser as mxParser
 import mx.DateTime as mxDT
 
-from Gnumed.pycommon import gmLog, gmPG, gmI18N, gmCLI, gmCfg, gmExceptions, gmNull
-from Gnumed.business import gmClinicalRecord, gmPerson, gmAllergy, gmVaccination, gmPathLab, gmMedDoc, gmDemographicRecord
+from Gnumed.pycommon import gmLog, gmI18N, gmCLI, gmCfg, gmExceptions, gmNull, gmPG2, gmTools
+from Gnumed.business import gmClinicalRecord, gmPerson, gmAllergy, gmMedDoc, gmDemographicRecord
 
 _log = gmLog.gmDefLog
 _log.Log(gmLog.lInfo, __version__)
@@ -51,7 +51,7 @@ class cEmrExport:
         self.__target = fileout
         self.__patient = patient
         self.lab_new_encounter = True
-	self.__filtered_items = []
+        self.__filtered_items = []
     #--------------------------------------------------------
     def set_constraints(self, constraints = None):
         """Sets exporter constraints.
@@ -121,7 +121,7 @@ class cEmrExport:
         
         #patient_dob = mxParser.DateFromString(self.__patient.get_identity().getDOB(aFormat = 'YYYY-MM-DD'), formats= ['iso']) 
         patient_dob = self.__patient.get_identity()['dob']
-        date_length = len(patient_dob.Format('%Y-%m-%d')) + 2 # (YYYY-mm-dd)
+        date_length = len(patient_dob.strftime('%Y-%m-%d')) + 2 # (YYYY-mm-dd)
 
         # dictionary of pairs indication : scheduled vaccination
         vaccinations4regimes = {}
@@ -154,7 +154,7 @@ class cEmrExport:
             vaccinations[a_vacc_regime['indication']] = emr.get_vaccinations(indications=[a_vacc_regime['indication']]) # given shots 4 indication
                 
         # patient dob in top of vaccination chart 
-        txt = '\nDOB: %s' %(patient_dob.Format('%Y-%m-%d')) + '\n'         
+        txt = '\nDOB: %s' %(patient_dob.strftime('%Y-%m-%d')) + '\n'         
 
         # vacc chart table headers
         # top ---- header line
@@ -192,7 +192,7 @@ class cEmrExport:
                 elif row_index < seq_no: # vaccination scheduled
                     try:
                         vacc_date = vaccinations[indication][row_index]['date'] # vaccination given                           
-                        vacc_date_str = vacc_date.Format('%Y-%m-%d')                        
+                        vacc_date_str = vacc_date.strftime('%Y-%m-%d')                        
                         txt +=    vacc_date_str + (column_widths[col_index] - len(vacc_date_str)) * ' ' + '|'                           
                         prev_displayed_date[col_index] = vacc_date                                                  
                     except:
@@ -200,7 +200,7 @@ class cEmrExport:
                             due_date = prev_displayed_date[col_index] + vaccinations4regimes[indication][row_index]['age_due_min'] # FIXME 'age_due_min' not properly retrieved
                         else: # due any other than first shot
                             due_date = prev_displayed_date[col_index] + vaccinations4regimes[indication][row_index]['min_interval']
-                        txt += '('+ due_date.Format('%Y-%m-%d') + ')' + (column_widths[col_index] - date_length) * ' ' + '|'
+                        txt += '('+ due_date.strftime('%Y-%m-%d') + ')' + (column_widths[col_index] - date_length) * ' ' + '|'
                         prev_displayed_date[col_index] = due_date
                 else: # not scheduled vaccination at that position
                     txt += column_widths[col_index] * ' ' + '|'
@@ -240,21 +240,21 @@ class cEmrExport:
             else:
                 indication = vacc_regimes[cont]['indication']
                 if len(vaccinations[indication]) > vacc_regimes[cont]['shots']: # boosters given
-                    all_vreg_boosters[cont] = vaccinations[indication][len(vaccinations[indication])-1]['date'].Format('%Y-%m-%d') # show last given booster date
+                    all_vreg_boosters[cont] = vaccinations[indication][len(vaccinations[indication])-1]['date'].strftime('%Y-%m-%d') # show last given booster date
                     scheduled_booster = vaccinations4regimes[indication][len(vaccinations4regimes[indication])-1]
                     booster_date = vaccinations[indication][len(vaccinations[indication])-1]['date'] + scheduled_booster['min_interval']                                        
                     if booster_date < mxDT.today():
-                        all_next_boosters[cont] = '<(' + booster_date.Format('%Y-%m-%d') + ')>' # next booster is due
+                        all_next_boosters[cont] = '<(' + booster_date.strftime('%Y-%m-%d') + ')>' # next booster is due
                     else:
-                        all_next_boosters[cont] = booster_date.Format('%Y-%m-%d')
+                        all_next_boosters[cont] = booster_date.strftime('%Y-%m-%d')
                 elif len(vaccinations[indication]) == vacc_regimes[cont]['shots']: # just finished vaccinations, begin boosters
                     all_vreg_boosters[cont] = column_widths[cont+1] * ' '
                     scheduled_booster = vaccinations4regimes[indication][len(vaccinations4regimes[indication])-1]
                     booster_date = vaccinations[indication][len(vaccinations[indication])-1]['date'] + scheduled_booster['min_interval']                    
                     if booster_date < mxDT.today():
-                        all_next_boosters[cont] = '<(' + booster_date.Format('%Y-%m-%d') + ')>' # next booster is due
+                        all_next_boosters[cont] = '<(' + booster_date.strftime('%Y-%m-%d') + ')>' # next booster is due
                     else:
-                        all_next_boosters[cont] = booster_date.Format('%Y-%m-%d')
+                        all_next_boosters[cont] = booster_date.strftime('%Y-%m-%d')
                 else:
                     all_vreg_boosters[cont] = column_widths[cont+1] * ' '  # unfinished schedule
                     all_next_boosters[cont] = column_widths[cont+1] * ' '
@@ -326,9 +326,10 @@ class cEmrExport:
         """
         txt = ''
         for a_field in field_list:
-            txt += offset*' ' + a_field + ': ' + str(item[a_field]) + '\n'
+            if type(a_field) is not types.UnicodeType:
+                a_field = unicode(a_field, encoding='latin1', errors='replace')
+            txt += u'%s%s%s' % ((offset * u' '), a_field, gmTools.coalesce(item[a_field], u'', template = u': %s\n'))
         return txt
-        
     #--------------------------------------------------------
     def get_allergy_output(self, allergy, left_margin = 0):
         """
@@ -375,9 +376,9 @@ class cEmrExport:
             txt += self.get_allergy_output(item, left_margin)
 #        elif isinstance(item, gmVaccination.cVaccination):
  #           txt += self.get_vaccination_output(item, left_margin)
-        elif isinstance(item, gmPathLab.cLabResult):
-            txt += self.get_lab_result_output(item, left_margin)
-            self.lab_new_encounter = False
+#        elif isinstance(item, gmPathLab.cLabResult):
+ #           txt += self.get_lab_result_output(item, left_margin)
+  #          self.lab_new_encounter = False
         return txt
     #--------------------------------------------------------
     def __fetch_filtered_items(self):
@@ -404,12 +405,12 @@ class cEmrExport:
        # except:
         #        _log.Error("vaccination error? outside regime")
 
-        filtered_items.extend(emr.get_lab_results(
-            since=self.__constraints['since'],
-            until=self.__constraints['until'],
-            encounters=self.__constraints['encounters'],
-            episodes=self.__constraints['episodes'],
-            issues=self.__constraints['issues']))
+#        filtered_items.extend(emr.get_lab_results(
+ #           since=self.__constraints['since'],
+  #          until=self.__constraints['until'],
+   #         encounters=self.__constraints['encounters'],
+    #        episodes=self.__constraints['episodes'],
+     #       issues=self.__constraints['issues']))
         self.__filtered_items = filtered_items
         return True
     #--------------------------------------------------------
@@ -420,7 +421,7 @@ class cEmrExport:
             left_margin - Number of spaces on the left margin
         """
         txt = left_margin*' ' + _('Allergy') + ': ' + allergy['descriptor'] + ', ' + \
-            allergy['reaction'] + '\n'
+            gmTools.coalesce(allergy['reaction'], _('unknown reaction')) + '\n'
         return txt
     #--------------------------------------------------------
     def get_vaccination_summary(self, vaccination, left_margin = 0):
@@ -443,7 +444,7 @@ class cEmrExport:
         if self.lab_new_encounter:
             txt += (left_margin+3)*' ' + _('Lab') + ': '  + \
                 lab_result['unified_name'] + '-> ' + lab_result['unified_val'] + \
-                ' ' + lab_result['val_unit']+ '\n' + '(' + lab_result['req_when'].Format('%Y-%m-%d') + ')'
+                ' ' + lab_result['val_unit']+ '\n' + '(' + lab_result['req_when'].strftime('%Y-%m-%d') + ')'
         return txt
     #--------------------------------------------------------
     def get_item_summary(self, item, left_margin = 0):
@@ -457,26 +458,24 @@ class cEmrExport:
             txt += self.get_allergy_summary(item, left_margin)
 #        elif isinstance(item, gmVaccination.cVaccination):
  #           txt += self.get_vaccination_summary(item, left_margin)
-        elif isinstance(item, gmPathLab.cLabResult) and \
-	    True: 
-            #(item['relevant'] == True or item['abnormal'] == True):
-            txt += self.get_lab_result_summary(item, left_margin)
-            self.lab_new_encounter = False
+#        elif isinstance(item, gmPathLab.cLabResult) and \
+#	    True: 
+ #           #(item['relevant'] == True or item['abnormal'] == True):
+  #          txt += self.get_lab_result_summary(item, left_margin)
+   #         self.lab_new_encounter = False
         return txt
     #--------------------------------------------------------             
     def refresh_historical_tree(self, emr_tree):
-    	"""
-	checks a emr_tree constructed with this.get_historical_tree() 
-	and sees if any new items need to be inserted.
-	"""
-	#TODO , caching eliminates tree update time, so don't really need this
-	
-    	self._traverse_health_issues( emr_tree, self._update_health_issue_branch)
+        """
+        checks a emr_tree constructed with this.get_historical_tree() 
+        and sees if any new items need to be inserted.
+        """
+        #TODO , caching eliminates tree update time, so don't really need this
+        self._traverse_health_issues( emr_tree, self._update_health_issue_branch)
 	
     #--------------------------------------------------------             
     def get_historical_tree( self, emr_tree):
-    	self._traverse_health_issues( emr_tree, self._add_health_issue_branch)
-	
+        self._traverse_health_issues( emr_tree, self._add_health_issue_branch)
     #--------------------------------------------------------             
     def _traverse_health_issues(self, emr_tree, health_issue_action):
         """
@@ -531,7 +530,7 @@ class cEmrExport:
     #--------------------------------------------------------             
     def _add_encounters_to_tree( self, encounters, emr_tree, episode_node):
                for an_encounter in encounters:
-                    label = '%s:%s' % (an_encounter['l10n_type'], an_encounter['started'].Format('%Y-%m-%d'))
+                    label = '%s:%s' % (an_encounter['l10n_type'], an_encounter['started'].strftime('%Y-%m-%d'))
                     encounter_node = emr_tree.AppendItem(episode_node, label)
 		    #import threading
 		    #def added():
@@ -688,18 +687,17 @@ class cEmrExport:
         txt += _('%s%s episode(s) between %s and %s\n') % (
             left_margin * ' ',
             str(no_epis),
-            first_encounter['started'].Format('%m/%Y'),
-            last_encounter['last_affirmed'].Format('%m/%Y')
+            first_encounter['started'].strftime('%m/%Y'),
+            last_encounter['last_affirmed'].strftime('%m/%Y')
         )
-        txt += _('%sLast seen: %s\n\n') % (left_margin * ' ', last_encounter['last_affirmed'].Format('%Y-%m-%d %H:%M'))
+        txt += _('%sLast seen: %s\n\n') % (left_margin * ' ', last_encounter['last_affirmed'].strftime('%Y-%m-%d %H:%M'))
         # FIXME: list each episode with range of time
         return txt
     #--------------------------------------------------------
     def get_episode_summary (self, episode, left_margin = 0):
-        """Dumps episode specific data
-        """
+        """Dumps episode specific data"""
         emr = self.__patient.get_emr()
-        encs = emr.get_encounters(episodes=[episode['pk_episode']], issues=[episode['pk_health_issue']])
+        encs = emr.get_encounters(episodes = [episode['pk_episode']])
         if encs is None:
             txt = left_margin * ' ' + _('Error retrieving encounters for episode\n%s') % str(episode)
             return txt
@@ -719,8 +717,8 @@ class cEmrExport:
             '%sLast worked on: %s\n'
         ) % (
             left_margin * ' ', episode['description'], status,
-            left_margin * ' ', no_encs, first_encounter['started'].Format('%m/%Y'), last_encounter['last_affirmed'].Format('%m/%Y'),
-            left_margin * ' ', last_encounter['last_affirmed'].Format('%Y-%m-%d %H:%M')
+            left_margin * ' ', no_encs, first_encounter['started'].strftime('%m/%Y'), last_encounter['last_affirmed'].strftime('%m/%Y'),
+            left_margin * ' ', last_encounter['last_affirmed'].strftime('%Y-%m-%d %H:%M')
         )
         return txt
     #--------------------------------------------------------
@@ -731,8 +729,8 @@ class cEmrExport:
         emr = self.__patient.get_emr()
         # general
         txt = (' ' * left_margin) + '%s - %s: %s' % (
-            encounter['started'].Format('%Y-%m-%d  %H:%M'),
-            encounter['last_affirmed'].Format('%H:%M'),
+            encounter['started'].strftime('%Y-%m-%d  %H:%M'),
+            encounter['last_affirmed'].strftime('%H:%M'),
             encounter['l10n_type']
         )
         if (encounter['assessment_of_encounter'] is not None) and (len(encounter['assessment_of_encounter']) > 0):
@@ -763,12 +761,20 @@ class cEmrExport:
                 continue
             txt += (' ' * left_margin) + soap_cat_labels[soap_cat] + ':\n'
             for soap_entry in soap_cat_narratives:
-                txt += (
-                    (' ' * (left_margin+3)) +
-                    soap_entry['date'].Format(_('%H:%M %.8s: ')) % soap_entry['provider'] +
-                    soap_entry['narrative'].replace('\n', eol_w_margin) +
-                    '\n'
+                txt += gmTools.wrap (
+                    '%s %.8s: %s\n' % (
+                        soap_entry['date'].strftime('%H:%M'),
+                        soap_entry['provider'],
+                        soap_entry['narrative']
+                    ), 75
                 )
+
+#                txt += (
+ #                   (' ' * (left_margin+3)) +
+  #                  soap_entry['date'].strftime('%H:%M %.8s: ') % soap_entry['provider'] +
+   #                 soap_entry['narrative'].replace('\n', eol_w_margin) +
+    #                '\n'
+     #           )
 		#FIXME: add diagnoses
 
         # items
@@ -805,9 +811,17 @@ class cEmrExport:
             episodes = emr.get_episodes(id_list=self.__constraints['episodes'], issues = [a_health_issue['pk']])
             for an_episode in episodes:
                self.__target.write('\n' + 6*' ' + 'Episode: ' + an_episode['description'] + '\n')
-               encounters = emr.get_encounters(since=self.__constraints['since'],
-                until=self.__constraints['until'], id_list=self.__constraints['encounters'],
-                episodes=[an_episode['pk_episode']], issues=[a_health_issue['pk']])
+               if a_health_issue['pk'] is None:
+                  issues = None
+               else:
+                  issues = [a_health_issue['pk']]
+               encounters = emr.get_encounters (
+                  since = self.__constraints['since'],
+                  until = self.__constraints['until'],
+                  id_list = self.__constraints['encounters'],
+                  episodes = [an_episode['pk_episode']],
+                  issues = issues
+               )
                for an_encounter in encounters:
                     # title
                     self.lab_new_encounter = True
@@ -815,8 +829,8 @@ class cEmrExport:
                         '\n            %s %s: %s - %s (%s)\n' % (
                             _('Encounter'),
                             an_encounter['l10n_type'],
-                            an_encounter['started'].Format('%A, %Y-%m-%d %H:%M'),
-                            an_encounter['last_affirmed'].Format('%m-%d %H:%M'),
+                            an_encounter['started'].strftime('%A, %Y-%m-%d %H:%M'),
+                            an_encounter['last_affirmed'].strftime('%m-%d %H:%M'),
                             an_encounter['assessment_of_encounter']
                         )
                     )
@@ -864,7 +878,7 @@ class cEmrExport:
         docs = doc_folder.get_documents()
         for doc in docs:
             self.__target.write('\n\n    (%s) %s - %s "%s"' % (
-                doc['date'].Format('%Y-%m-%d'),
+                doc['date'].strftime('%Y-%m-%d'),
                 doc['ext_ref'],
                 doc['l10n_type'],
                 doc['comment'])
@@ -902,13 +916,8 @@ class cEmrExport:
             cont += 1
         self.__target.write('    Gender: %s\n' % ident['gender'])
         self.__target.write('    Title: %s\n' % ident['title'])
-        self.__target.write('    Dob: %s\n' % ident['dob'].Format('%Y-%m-%d'))
+        self.__target.write('    Dob: %s\n' % ident['dob'].strftime('%Y-%m-%d'))
         self.__target.write('    Medical age: %s\n' % gmPerson.dob2medical_age(ident['dob']))
-        #addr_types = ident['addresses'].keys()
-        #for addr_t in addr_types:
-        #    addr_lst = ident['addresses'][addr_t]
-        #    for address in addr_lst:
-        #        self.__target.write('    Address (' + addr_t + '): ' + address + '\n')
     #--------------------------------------------------------
     def dump_constraints(self):
         """
@@ -917,11 +926,11 @@ class cEmrExport:
         self.__first_constraint = True
         if not self.__constraints['since'] is None:
             self.dump_constraints_header()
-            self.__target.write('\nSince: %s' % self.__constraints['since'].Format('%Y-%m-%d'))
+            self.__target.write('\nSince: %s' % self.__constraints['since'].strftime('%Y-%m-%d'))
 
         if not self.__constraints['until'] is None:
             self.dump_constraints_header()
-            self.__target.write('\nUntil: %s' % self.__constraints['until'].Format('%Y-%m-%d'))
+            self.__target.write('\nUntil: %s' % self.__constraints['until'].strftime('%Y-%m-%d'))
 
         if not self.__constraints['encounters'] is None:
             self.dump_constraints_header()
@@ -952,6 +961,8 @@ class cEmrExport:
 #============================================================
 class cEMRJournalExporter:
 	"""Exports patient EMR into a simple chronological journal.
+
+	Note that this export will emit u'' strings only.
 	"""
 	def __init__(self, patient=None):
 		if patient is None:
@@ -973,17 +984,17 @@ class cEMRJournalExporter:
 	#--------------------------------------------------------
 	def export_to_file(self, filename=None):
 		if not self.__pat.is_connected():
-			return (False, 'no active patient')
+			return (False, u'no active patient')
 		if filename is None:
 			ident = self.__pat.get_identity()
-			path = os.path.abspath(os.path.expanduser('~/gnumed/export'))
-			filename = '%s-%s-%s-%s.txt' % (
+			path = os.path.expanduser(os.path.join('~', 'gnumed', 'export'))
+			filename = u'%s-%s-%s-%s.txt' % (
 				os.path.join(path, _('emr-journal')),
-				ident['lastnames'].replace(' ', '-'),
-				ident['firstnames'].replace(' ', '_'),
-				ident['dob'].Format('%Y-%m-%d')
+				ident['lastnames'].replace(u' ', u'-'),
+				ident['firstnames'].replace(u' ', u'_'),
+				ident['dob'].strftime('%Y-%m-%d')
 			)
-		f = open(filename, 'wb')
+		f = codecs.open(filename = filename, mode = 'w+b', encoding = 'utf8')
 		status = self.__export(target = f)
 		f.close()
 		return (status, filename)
@@ -998,55 +1009,48 @@ class cEMRJournalExporter:
 			return False
 		ident = self.__pat.get_identity()
 		# write header
-		txt = _('Chronological EMR Journal\n').encode('iso8859-15')
+		txt = _('Chronological EMR Journal\n')
 		target.write(txt)
-		target.write('=' * (len(txt)-1))
+		target.write(u'=' * (len(txt)-1))
 		target.write('\n')
-		target.write(_('Patient: %s (%s), No: %s\n').encode('iso8859-15') % (ident['description'], ident['gender'], self.__pat['ID']))
-		target.write(_('Born   : %s, age: %s\n\n').encode('iso8859-15') % (ident['dob'].Format('%Y-%m-%d'), ident.get_medical_age()))
-		target.write('.-%10.10s---%9.9s-------%72.72s\n' % ('-' * 10, '-' * 9, '-' * self.__part_len))
-		target.write('| %10.10s | %9.9s |   | %s\n' % (_('Date'), _('Doc'), _('Narrative')))
-		target.write('|-%10.10s---%9.9s-------%72.72s\n' % ('-' * 10, '-' * 9, '-' * self.__part_len))
+		target.write(_('Patient: %s (%s), No: %s\n') % (ident['description'], ident['gender'], self.__pat['ID']))
+		target.write(_('Born   : %s, age: %s\n\n') % (ident['dob'].strftime('%Y-%m-%d'), ident.get_medical_age()))
+		target.write(u'.-%10.10s---%9.9s-------%72.72s\n' % (u'-' * 10, u'-' * 9, u'-' * self.__part_len))
+		target.write(u'| %10.10s | %9.9s |   | %s\n' % (_('Date'), _('Doc'), _('Narrative')))
+		target.write(u'|-%10.10s---%9.9s-------%72.72s\n' % (u'-' * 10, u'-' * 9, u'-' * self.__part_len))
 		# get data
-		cmd = """
+		cmd = u"""
 select
 	to_char(vemrj.clin_when, 'YYYY-MM-DD') as date,
 	vemrj.*,
 	(select rank from clin.soap_cat_ranks where soap_cat=vemrj.soap_cat) as scr
 from clin.v_emr_journal vemrj
 where pk_patient=%s order by date, pk_episode, scr"""
-		rows, idx = gmPG.run_ro_query (
-			'clinical',
-			cmd,
-			True,
-			self.__pat['ID']
-		)
-		if rows is None:
-			_log.Log(gmLog.lErr, 'cannot export EMR as journal')
-			return False
+		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': [self.__pat['ID']]}], get_col_idx = True)
 		# write data
-		prev_date = ''
-		prev_doc = ''
+		prev_date = u''
+		prev_doc = u''
 		for row in rows:
 			# narrative
-			if row[idx['narrative']] is not None:
-				txt = row[idx['narrative']].replace('\n', ' ').replace('\r', ' ')
+			if row[idx['narrative']] is None:
+				continue
+			txt = row[idx['narrative']].replace(u'\n', u' ').replace(u'\r', u' ')
 			no_parts = (len(txt) / self.__part_len) + 1
 			# doc
 			curr_doc = row[idx['modified_by']]
 			if curr_doc != prev_doc:
 				prev_doc = curr_doc
 			else:
-				curr_doc = ''
+				curr_doc = u''
 			# date
 			curr_date = row[idx['date']]
 			if curr_date != prev_date:
 				prev_date = curr_date
 				curr_doc = row[idx['modified_by']]
 			else:
-				curr_date = ''
+				curr_date = u''
 			# display first part
-			target.write('| %10.10s | %9.9s | %s | %s\n' % (
+			target.write(u'| %10.10s | %9.9s | %s | %s\n' % (
 				curr_date,
 				curr_doc,
 				self.__tx_soap[row[idx['soap_cat']]],
@@ -1060,10 +1064,10 @@ where pk_patient=%s order by date, pk_episode, scr"""
 				soap_cat = self.__tx_soap[row[idx['soap_cat']]]
 				msg = txt[(part * self.__part_len):((part+1) * self.__part_len)]
 				line = template % (u'', u'', soap_cat, msg)
-				target.write(line.encode('iso8859-15'))
+				target.write(line)
 		# write footer
-		target.write('`-%10.10s---%9.9s-------%72.72s\n\n'.encode('iso8859-15') % ('-' * 10, '-' * 9, '-' * self.__part_len))
-		target.write(_('Exported: %s\n').encode('iso8859-15') % time.asctime())
+		target.write(u'`-%10.10s---%9.9s-------%72.72s\n\n' % (u'-' * 10, u'-' * 9, u'-' * self.__part_len))
+		target.write(_('Exported: %s\n') % time.asctime())
 		return True
 #============================================================
 class cMedistarSOAPExporter:
@@ -1095,9 +1099,9 @@ class cMedistarSOAPExporter:
 				time.strftime('%Y-%m-%d',time.localtime()),
 				ident['lastnames'].replace(' ', '-'),
 				ident['firstnames'].replace(' ', '_'),
-				ident['dob'].Format('%Y-%m-%d')
+				ident['dob'].strftime('%Y-%m-%d')
 			)
-		f = open(filename, 'wb')
+		f = codecs.open(filename = filename, mode = 'w+b', encoding = 'utf8')
 		status = self.__export(target = f)
 		f.close()
 		return (status, filename)
@@ -1114,44 +1118,16 @@ class cMedistarSOAPExporter:
 		if encounter is None:
 			encounter = emr.get_active_encounter()
 		# get data
-		cmd = "select narrative from clin.v_emr_journal where pk_patient=%s and pk_encounter=%s and soap_cat=%s"
+		cmd = u"select narrative from clin.v_emr_journal where pk_patient=%s and pk_encounter=%s and soap_cat=%s"
 		for soap_cat in 'soap':
-			rows = gmPG.run_ro_query (
-				'clinical',
-				cmd,
-				False,
-				self.__pat['ID'],
-				encounter['pk_encounter'],
-				soap_cat
-			)
-			if rows is None:
-				_log.Log(gmLog.lErr, 'cannot export SOAP from EMR')
-				return False
+			rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': (self.__pat['ID'], encounter['pk_encounter'], soap_cat)}])
 			target.write('*MD%s*\n' % self.__tx_soap[soap_cat])
 			for row in rows:
-				target.write('%s\n' % wrap(row[0], 64))
+				text = row[0]
+				if text is not None:
+					target.write('%s\n' % gmTools.wrap(text, 64))
 		return True
 #============================================================
-def wrap(text, width):
-	"""
-	A word-wrap function that preserves existing line breaks
-	and most spaces in the text. Expects that existing line
-	breaks are posix newlines (\n).
-	"""
-	return reduce (
-		lambda line, word, width=width: '%s%s%s' % (
-			line,
-			' \n'[(
-				len(line)
-				- line.rfind('\n')
-				- 1
-				+ len(word.split('\n',1)[0])
-				>= width
-			)],
-			word),
-		text.split(' ')
-	)
-#============================================================
 # main
 #------------------------------------------------------------
 def usage():
@@ -1252,9 +1228,6 @@ if __name__ == "__main__":
     if gmCLI.has_arg('--help'):
         usage()
 
-    gmPG.set_default_client_encoding({'wire': 'utf8', 'string': 'utf8'})
-    # make sure we have a connection
-    pool = gmPG.ConnectionPool()
     # run main loop
     try:
         run()
@@ -1268,8 +1241,52 @@ if __name__ == "__main__":
         _log.LogException('unhandled exception caught', sys.exc_info(), verbose=1)
 #============================================================
 # $Log: gmPatientExporter.py,v $
-# Revision 1.81.2.1  2006/09/01 09:23:49  ncq
-# - robustify re encoding issues
+# Revision 1.96  2007/01/18 22:03:58  ncq
+# - a bit of cleanup
+#
+# Revision 1.95  2007/01/15 20:20:03  ncq
+# - move wrap() to gmTools
+#
+# Revision 1.94  2007/01/13 22:17:40  ncq
+# - wrap narrative to 75 characters per line
+#
+# Revision 1.93  2006/12/13 00:31:24  ncq
+# - export into unicode files
+# - fix use of get_encounters()
+#
+# Revision 1.92  2006/11/26 15:44:34  ncq
+# - strftime() does not accept u''
+#
+# Revision 1.91  2006/11/24 14:16:20  ncq
+# - unicode-robustify dump_item_fields()
+#
+# Revision 1.90  2006/11/19 11:05:38  ncq
+# - cleanup
+#
+# Revision 1.89  2006/11/09 17:48:05  ncq
+# - ever more careful handling of NULLs
+#
+# Revision 1.88  2006/11/07 00:25:19  ncq
+# - make journal exporter emit strictly u''
+#
+# Revision 1.87  2006/11/05 17:54:17  ncq
+# - don't use issue pk in get_encounters()
+# - gmPG -> gmPG2
+#
+# Revision 1.86  2006/11/05 17:02:54  ncq
+# - comment out lab results access, not in use yet
+#
+# Revision 1.85  2006/10/25 07:46:44  ncq
+# - Format() -> strftime() since datetime.datetime does not have .Format()
+#
+# Revision 1.84  2006/10/25 07:18:12  ncq
+# - no more gmPG
+#
+# Revision 1.83  2006/10/23 13:21:50  ncq
+# - vaccs/path lab not yet converted to gmPG2
+#
+# Revision 1.82  2006/09/03 14:46:26  ncq
+# - robustify regarding encoding issues
 # - improve test suite
 #
 # Revision 1.81  2006/07/19 20:25:48  ncq
