@@ -1,4 +1,4 @@
-"""GNUMed client log handling.
+"""GNUmed client log handling.
 
 All error logging, user notification and otherwise unhandled 
 exception handling should go through classes or functions of 
@@ -53,7 +53,7 @@ Usage:
 @license: GPL
 """
 # $Source: /sources/gnumed/gnumed/gnumed/client/pycommon/gmLog.py,v $
-__version__ = "$Revision: 1.17.2.1 $"
+__version__ = "$Revision: 1.26 $"
 __author__ = "Karsten Hilbert <Karsten.Hilbert@gmx.net>"
 #-------------------------------------------
 # don't use gmCLI in here since that would give a circular reference
@@ -64,9 +64,9 @@ import sys, time, traceback, os.path, os, string, stat
 # safely import SYSLOG, currently POSIX only
 try:
 	import syslog
-	_use_syslog = (1 == 1)
+	_use_syslog = True
 except ImportError:
-	_use_syslog = (1 == 0)
+	_use_syslog = False
 	if os.name == 'posix':
 		print "Although we are on a POSIX compliant platform the module SYSLOG cannot be imported !"
 		print "You should download and install this module !"
@@ -191,7 +191,6 @@ class cLogger:
 		- for a list of log levels see top of file
 		- messages above the currently active level of logging/verbosity are dropped silently
 		- if Rawness == lCooked non-printables < 32 (space) will be mapped to their name in ASCII
-		- FIXME: this should be a Unicode mapping
 		"""
 		# are we in for work ?
 		if self.__targets is not None:
@@ -231,7 +230,10 @@ class cLogger:
 				level1 = lWarn
 				level2 = lData
 			# split the tuple
-			exc_type, exc_val, exc_traceback = sys.exc_info()
+			if exception is None:
+				exc_type, exc_val, exc_traceback = sys.exc_info()
+			else:
+				exc_type, exc_val, exc_traceback = exception
 			# FIXME: I wonder if the following back-and-forth reversing is necessary
 			# trace back to root caller
 			tb = exc_traceback
@@ -392,7 +394,7 @@ class cLogTargetFile(cLogTarget):
 	# the actual file handle
 	__handle = None
 	#---------------------------
-	def __init__ (self, aLogLevel = lErr, aFileName = "", aMode = "ab"):
+	def __init__ (self, aLogLevel = lErr, aFileName = '', aMode = 'ab'):
 		# do our extra work
 		self.__handle = open (aFileName, aMode)
 		if self.__handle is None:
@@ -402,7 +404,7 @@ class cLogTargetFile(cLogTarget):
 			cLogTarget.__init__(self, aLogLevel)
 			self.ID = os.path.abspath (aFileName) # the file name canonicalized
 
-		self.writeMsg (lInfo, "instantiated log file " + aFileName + " with ID " + str(self.ID))
+		self.writeMsg (lInfo, "instantiated log file [%s] with ID [%s] " % (aFileName, self.ID))
 	#---------------------------
 	def dummy(self):
 		for module in sys.modules.values():
@@ -411,9 +413,15 @@ class cLogTargetFile(cLogTarget):
 		cLogTarget.close(self)
 		self.__handle.close()
 	#---------------------------
-	def dump2stderr (self, aTimeStamp, aPrefix, aLocation, aMsg):
+	def dump2stderr(self, aTimeStamp, aPrefix, aLocation, aMsg):
 		try:
-			self.__handle.write(aTimeStamp + aPrefix + aLocation + aMsg)
+			msg = []
+			for tmp in [aTimeStamp, aPrefix, aLocation, aMsg]:
+				if type(tmp) == type(u''):
+					msg.append(tmp.encode('latin1'))			# FIXME: should be locale encoding, not latin1
+				if type(tmp) == type(''):
+					msg.append(unicode(tmp, errors='replace').replace(u'\ufffd', '?').encode('latin1'))
+			self.__handle.write(' '.join(msg))
 		except:
 			print "*** cannot write to log file [%s] ***" % self.ID
 #---------------------------------------------------------------
@@ -426,15 +434,27 @@ class cLogTargetConsole(cLogTarget):
 		self.writeMsg (lData, "instantiated console logging with ID " + str(self.ID))
 	#---------------------------
 	def dump2stdout (self, aTimeStamp, aPrefix, aLocation, aMsg):
-		try: 
-			sys.stdout.write(aPrefix + aLocation + aMsg)
-		except IOError:
+		try:
+			msg = []
+			for tmp in [aPrefix, aLocation, aMsg]:
+				if type(tmp) == type(u''):
+					msg.append(tmp.encode('latin1'))
+				if type(tmp) == type(''):
+					msg.append(unicode(tmp, errors='replace').replace(u'\ufffd', '?').encode('latin1'))
+			sys.stdout.write(' '.join(msg))
+		except:
 			print aPrefix + aLocation + aMsg
 	#---------------------------
 	def dump2stderr (self, aTimeStamp, aPrefix, aLocation, aMsg):
 		try:
-			sys.stderr.write(aPrefix + aLocation + aMsg)
-		except IOError:
+			msg = []
+			for tmp in [aPrefix, aLocation, aMsg]:
+				if type(tmp) == type(u''):
+					msg.append(tmp.encode('latin1'))
+				if type(tmp) == type(''):
+					msg.append(unicode(tmp, errors='replace').replace(u'\ufffd', '?').encode('latin1'))
+			sys.stderr.write(' '.join(msg))
+		except:
 			print aPrefix + aLocation + aMsg
 #---------------------------------------------------------------
 class cLogTargetSyslog(cLogTarget):
@@ -824,8 +844,32 @@ myLogger = gmLog.cLogger(aTarget = your-log-target)
 # __is_subclass__
 #===============================================================
 # $Log: gmLog.py,v $
-# Revision 1.17.2.1  2006/08/28 10:21:22  ncq
-# - be slightly smarter about unicode data being written to the log file
+# Revision 1.26  2006/11/15 00:38:58  ncq
+# - only use sys.exc_info() if exception is not None
+#
+# Revision 1.25  2006/11/05 14:24:53  ncq
+# - fix improper indentation
+#
+# Revision 1.24  2006/11/04 22:25:10  ncq
+# - try to catch more encoding errors in console handler
+#
+# Revision 1.23  2006/11/04 20:04:47  ncq
+# - be even more careful about console output
+#
+# Revision 1.22  2006/11/04 19:59:35  ncq
+# - remove superfluous " "
+#
+# Revision 1.21  2006/11/01 23:20:09  ncq
+# - be even *more* careful about unicode-to-console
+#
+# Revision 1.20  2006/11/01 12:21:39  ncq
+# - apply unicode handling to console logger, too
+#
+# Revision 1.19  2006/10/24 13:17:30  ncq
+# - much improved string handling, should catch most Unicode decode output errors now
+#
+# Revision 1.18  2006/09/01 14:42:16  ncq
+# - handle unicode output to log file slightly smarter
 #
 # Revision 1.17  2006/06/20 14:28:23  ncq
 # - give people an email address they can actually send the log to ;-)
