@@ -14,10 +14,12 @@ def resultset_functional_batchgenerator(cursor, size=100):
 """
 # =======================================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/pycommon/gmPG.py,v $
-__version__ = "$Revision: 1.78.2.2 $"
+__version__ = "$Revision: 1.84 $"
 __author__  = "H.Herb <hherb@gnumed.net>, I.Haywood <i.haywood@ugrad.unimelb.edu.au>, K.Hilbert <Karsten.Hilbert@gmx.net>"
 __license__ = 'GPL (details at http://www.gnu.org)'
 
+print "gmPG phased out, please replace with gmPG2"
+
 # python standard modules
 import string, copy, os, sys, time, locale
 
@@ -122,8 +124,7 @@ where
 		and
 	cols.table_name = %s
 order by
-	cols.ordinal_position
-"""
+	cols.ordinal_position"""
 
 query_table_attributes = """select
 	cols.column_name
@@ -152,8 +153,7 @@ where
 				relnamespace = (select oid from pg_namespace where nspname = %(schema)s) and
 				relname = %(table)s
 		)
-	)
-;"""
+	)"""
 
 
 # a handy return to dbapi simplicity
@@ -505,12 +505,8 @@ class ConnectionPool:
 		if wire_encoding is None:
 			raise ValueError, '<wire_encoding> cannot be None'
 
-#		if string_encoding.lower() == 'utf8':
-#			unicode_results = 1
-#		else:
-#			unicode_results = 0
 		try:
-#			conn = dbapi.connect(dsn=dsn, client_encoding=(string_encoding, 'strict'), unicode_results=unicode_results)
+			# FIXME: eventually use UTF or UTF8 for READONLY connections _only_
 			conn = dbapi.connect(dsn=dsn, client_encoding=(string_encoding, 'strict'), unicode_results=1)
 		except StandardError:
 			_log.LogException("database connection failed: DSN = [%s], host:port = [%s]" % (dsn, hostport), sys.exc_info(), verbose = 1)
@@ -1364,7 +1360,7 @@ def get_current_user():
 	return result[0][0]
 #---------------------------------------------------
 def add_housekeeping_todo(
-	reporter='$RCSfile: gmPG.py,v $ $Revision: 1.78.2.2 $',
+	reporter='$RCSfile: gmPG.py,v $ $Revision: 1.84 $',
 	receiver='DEFAULT',
 	problem='lazy programmer',
 	solution='lazy programmer',
@@ -1427,7 +1423,7 @@ def __request_login_params_tui():
 	print "\nPlease enter the required login parameters:"
 	try:
 		host = __prompted_input("host ['' = non-TCP/IP]: ", '')
-		database = __prompted_input("database [gnumed_v2]: ", 'gnumed_v2')
+		database = __prompted_input("database [gnumed_v3]: ", 'gnumed_v3')
 		user = __prompted_input("user name: ", '')
 		password = getpass.getpass("password (not shown): ")
 		port = __prompted_input("port [5432]: ", 5432)
@@ -1601,11 +1597,23 @@ if __name__ == "__main__":
 
 #==================================================================
 # $Log: gmPG.py,v $
-# Revision 1.78.2.2  2006/09/01 09:24:35  ncq
-# - always return unicode results, this will point us to encoding errors early
+# Revision 1.84  2006/12/06 16:04:48  ncq
+# - cleanup
 #
-# Revision 1.78.2.1  2006/08/28 14:30:18  ncq
-# - do use unicode_results in connect() when encoding is utf8
+# Revision 1.83  2006/10/23 13:22:26  ncq
+# - cleanup only
+#
+# Revision 1.82  2006/10/08 09:24:02  ncq
+# - add deprecation warning
+#
+# Revision 1.81  2006/09/21 19:47:40  ncq
+# - change default to "gnumed_v3"
+#
+# Revision 1.80  2006/09/01 14:42:54  ncq
+# - *always* return unicode from database
+#
+# Revision 1.79  2006/08/28 14:32:40  ncq
+# - read data from database in unicode encoding, hence use unicode_results in connect()
 #
 # Revision 1.78  2006/08/01 22:02:42  ncq
 # - update v2 hash
