@@ -2,16 +2,21 @@
 # GNUmed SANE/TWAIN scanner classes
 #==================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/pycommon/gmScanBackend.py,v $
-# $Id: gmScanBackend.py,v 1.16.2.5 2006/08/28 22:04:11 ncq Exp $
-__version__ = "$Revision: 1.16.2.5 $"
+# $Id: gmScanBackend.py,v 1.34 2007/01/19 14:06:17 ncq Exp $
+__version__ = "$Revision: 1.34 $"
 __license__ = "GPL"
-__author__ = """Sebastian Hilbert <Sebastian.Hilbert@gmx.net>,
-Karsten Hilbert <Karsten.Hilbert@gmx.net>"""
+__author__ = """Sebastian Hilbert <Sebastian.Hilbert@gmx.net>, Karsten Hilbert <Karsten.Hilbert@gmx.net>"""
 
 #==================================================
-import sys, os.path, os, Image, string, time, shutil, tempfile
+# stdlib
+import sys, os.path, os, Image, string, time, shutil, tempfile, codecs, glob, locale
+
+
+# GNUmed
+if __name__ == '__main__':
+	sys.path.insert(0, '../../')
+from Gnumed.pycommon import gmLog, gmExceptions, gmShellAPI
 
-from Gnumed.pycommon import gmLog, gmExceptions
 
 _log = gmLog.gmDefLog
 _log.Log(gmLog.lInfo, __version__)
@@ -19,129 +24,24 @@ _log.Log(gmLog.lInfo, __version__)
 _twain_module = None
 _sane_module = None
 
+use_XSane = True
 #=======================================================
 class cTwainScanner:
 
 	def __init__(self, calling_window=None):
-		msg = 'cannot instantiate TWAIN driver class'
 		if not _twain_import_module():
-			raise gmExceptions.ConstructorError, msg
-
-		self.__setup_event_handler()
+			raise gmExceptions.ConstructorError, 'cannot instantiate TWAIN driver class: cannot import TWAIN module'
 
 		self.__calling_window = calling_window
-
 		self.__src_manager = None
-#		if not self.__init_src_manager():
-#			raise gmExceptions.ConstructorError, msg
-
 		self.__scanner = None
-#		if not self.__init_scanner():
-#			raise gmExceptions.ConstructorError, msg
-	#---------------------------------------------------
-	def __setup_event_handler(self):
-		# FIXME: this means we cannot use more than one TWAIN source at once
-		self.__twain_event_handler = {
-			_twain_module.MSG_XFERREADY: self._twain_handle_transfer,
-			_twain_module.MSG_CLOSEDSREQ: self._twain_close_datasource,
-			_twain_module.MSG_CLOSEDSOK: self._twain_save_state,
-			_twain_module.MSG_DEVICEEVENT: self._twain_handle_src_event
-		}
-	#---------------------------------------------------
-	def __init_src_manager(self):
-		# open scanner manager
-		if self.__src_manager is None:
-			# TWAIN talks to us via MS-Windows message queues so we
-			# need to pass it a handle to ourselves
-			self.__src_manager = _twain_module.SourceManager(self.__calling_window.GetHandle())
-			if not self.__src_manager:
-				_log.Log(gmLog.lErr, "cannot get a handle for the TWAIN source manager")
-				return False
-			# TWAIN will notify us when the image is scanned
-			self.__src_manager.SetCallback(self._twain_event_callback)
-			_log.Log(gmLog.lData, "TWAIN source manager config: %s" % str(self.__src_manager.GetIdentity()))
-			self.__scanner = None
-		return True
-	#---------------------------------------------------
-	def __init_scanner(self):
-		if not self.__init_src_manager():
-			return False
-		if self.__scanner is None:
-			# FIXME: set source by string
-			self.__scanner = self.__src_manager.OpenSource()
-			if not self.__scanner:
-				_log.Log(gmLog.lErr, "cannot open scanner via TWAIN source manager")
-				return False
-			_log.Log(gmLog.lInfo, "TWAIN data source: %s" % self.__scanner.GetSourceName())
-			_log.Log(gmLog.lData, "TWAIN data source config: %s" % str(self.__scanner.GetIdentity()))
-		return True
-	#---------------------------------------------------
-	def close(self):
-#		self.__scanner.CancelAllPendingXfers()
-#		self.__scanner.destroy()
-#		self.__scanner = None
-#		self.__src_manager.destroy()
-#		self.__src_manager = None
-		return
-	#---------------------------------------------------
-	# TWAIN callback handling
-	#---------------------------------------------------
-	def _twain_event_callback(self, twain_event):
-		_log.Log(gmLog.lData, 'notification of TWAIN event <%s>' % str(twain_event))
-		self.__twain_event_handler[twain_event]()
-		return
-	#---------------------------------------------------
-	def _twain_close_datasource(self):
-		_log.Log(gmLog.lInfo, "being asked to close data source")
-#		self.__scanner.CancelAllPendingXfers()
-#		self.__scanner.HideUI()
-#		self.__scanner.destroy()
-		self.__scanner = None
-		return True
-	#---------------------------------------------------
-	def _twain_save_state():
-		_log.Log(gmLog.lInfo, "being asked to save application state")
-		return True
-	#---------------------------------------------------
-	def _twain_handle_src_event():
-		_log.Log(gmLog.lInfo, "being asked to handle device specific event")
-		return True
-	#---------------------------------------------------
-	def _twain_handle_transfer(self):
-		_log.Log(gmLog.lData, 'receiving image from TWAIN source')
-		_log.Log(gmLog.lData, 'image info: %s' % self.__scanner.GetImageInfo())
+		self.__done_transferring_image = False
 
-		# get from source
-		try:
-			(external_data_handle, more_images_pending) = self.__scanner.XferImageNatively()
-		except:
-			_log.LogException('XferImageNatively() failed, unable to get global heap image handle', sys.exc_info(), verbose=1)
-			# free external image memory
-			_twain_module.GlobalHandleFree(external_data_handle)
-			# hide the scanner user interface again
-			self.__scanner.HideUI()
-			return False
-
-		_log.Log(gmLog.lData, '%s pending images' % more_images_pending)
-
-		try:
-			# convert DIB to standard bitmap file
-			_twain_module.DIBToBMFile(external_data_handle, self.__filename)
-		except:
-			_log.LogException('DIBToBMFile() failed, unable to convert image in global heap handle into file [%s]' % self.__filename, sys.exc_info(), verbose=1)
-			# free external image memory
-			_twain_module.GlobalHandleFree(external_data_handle)
-			# hide the scanner user interface again
-			self.__scanner.HideUI()
-			return False
-		# free external image memory
-		_twain_module.GlobalHandleFree(external_data_handle)
-		# hide the scanner user interface again
-		self.__scanner.HideUI()
-
-		return True
+		self.__register_event_handlers()
 	#---------------------------------------------------
-	def acquire_page_into_file(self, delay=None, filename=None, tmpdir=None):
+	# external API
+	#---------------------------------------------------
+	def acquire_pages_into_files(self, delay=None, filename=None, tmpdir=None):
 		if filename is None:
 			if tmpdir is None:
 				(handle, filename) = tempfile.mkstemp(suffix='.bmp', prefix='gmScannedObj-')
@@ -157,52 +57,145 @@ class cTwainScanner:
 		if not self.__init_scanner():
 			return False
 
-		self.__scanner.RequestAcquire()
-		return self.__filename
+		self.__done_transferring_image = False
+		self.__scanner.RequestAcquire(True)
+
+		return [self.__filename]
 	#---------------------------------------------------
-#	def dummy(self):
-#
-#		# make tmp file name
-#		tempfile.tempdir = self.scan_tmp_dir
-#		tempfile.template = 'obj-'
-#		bmp_name = tempfile.mktemp('.bmp')
-#		fname = bmp_name
-#
-#		# convert to JPEG ?
-#		do_jpeg = _cfg.get("scanning", "convert to JPEG")
-#		if do_jpeg in ["yes", "on"]:
-#			jpg_name = tempfile.mktemp('.jpg')
-#			fname = jpg_name
-#			# get JPEG quality factor
-#			quality_value = _cfg.get("scanning", "JPEG quality level")
-#			if quality_value is None:
-#				_log.Log(gmLog.lWarn, "JPEG quality level not specified in config file, using default level of 75")
-#				quality_value = 75
-#			else:
-#				if quality_value.isdigit():
-#					quality_value = int(quality_value, 10)
-#				else:
-#					_log.Log(gmLog.lWarn, "JPEG quality level [%s] not a number, using default level of 75" % quality_value)
-#					quality_value = 75
-#			# do we want progression ?
-#			progression_flag = _cfg.get("scanning", "progressive JPEG")
-#			_log.Log(gmLog.lData, "JPEG conversion: quality level: %s, progression: %s" % (quality_value, progression_flag))
-#			kwds = {}
-#			kwds['quality'] = quality_value
-#			if progression_flag in ["yes", "on"]:
-#				kwds['optimize'] = 1
-#				kwds['progressive'] = 1
-#			# actually convert to JPEG
-#			try:
-#				Image.open(bmp_name).convert('RGB').save(jpg_name, **kwds)
-#			except:
-#				_log.LogException("optimized JPEG write failed, turning off optimization", sys.exc_info(), fatal=0)
-#				Image.open(bmp_name).convert('RGB').save(jpg_name)
-#			# remove bitmap (except Windows can't do that sometimes :-(
-#			try:
-#				os.remove(bmp_name)
-#			except:
-#				_log.LogException("Can't remove bitmap.", sys.exc_info(), fatal=0)
+	def image_transfer_done(self):
+		return self.__done_transferring_image
+	#---------------------------------------------------
+	def close(self):
+		# close() is called after acquire_pages*() so if we destroy the source
+		# before TWAIN is done we hang it, an RequestAcquire() only *requests*
+		# a scan, we would have to wait for process_xfer to finisch before
+		# destroying the source, and even then it might destroy state in the
+		# non-Python TWAIN subsystem
+		#**********************************
+		# if we do this TWAIN does not work
+		#**********************************
+#		if self.__scanner is not None:
+#			self.__scanner.destroy()
+
+#		if self.__src_manager is not None:
+#			self.__src_manager.destroy()
+
+#		del self.__scanner
+#		del self.__src_manager
+		return
+	#---------------------------------------------------
+	# internal helpers
+	#---------------------------------------------------
+	def __init_scanner(self):
+		if self.__scanner is not None:
+			return True
+
+		self.__init_src_manager()
+		if self.__src_manager is None:
+			return False
+
+		# TWAIN will notify us when the image is scanned
+		self.__src_manager.SetCallback(self._twain_event_callback)
+
+		# no arg == show "select source" dialog
+		self.__scanner = self.__src_manager.OpenSource()
+		if self.__scanner is None:
+			_log.Log(gmLog.lErr, "user canceled scan source selection dialog")
+			return False
+
+		_log.Log(gmLog.lInfo, "TWAIN data source: %s" % self.__scanner.GetSourceName())
+		_log.Log(gmLog.lData, "TWAIN data source config: %s" % str(self.__scanner.GetIdentity()))
+
+		return True
+	#---------------------------------------------------
+	def __init_src_manager(self):
+		# open scanner manager
+		if self.__src_manager is not None:
+			return
+
+		# clean up scanner driver since we will initialize the source manager
+#		if self.__scanner is not None:
+#			self.__scanner.destroy()			# this probably should not be done here
+#			del self.__scanner					# try to sneak this back in later
+#			self.__scanner = None				# this really should work
+
+		# TWAIN talks to us via MS-Windows message queues
+		# so we need to pass it a handle to ourselves,
+		# the following fails with "attempt to create Pseudo Window failed",
+		# I assume because the TWAIN vendors want to sabotage rebranding their GUI
+#		self.__src_manager = _twain_module.SourceManager(self.__calling_window.GetHandle(), ProductName = 'GNUmed - The EMR that never sleeps.')
+		self.__src_manager = _twain_module.SourceManager(self.__calling_window.GetHandle())
+
+		_log.Log(gmLog.lInfo, "TWAIN source manager config: %s" % str(self.__src_manager.GetIdentity()))
+	#---------------------------------------------------
+	# TWAIN callback handling
+	#---------------------------------------------------
+	def __register_event_handlers(self):
+		self.__twain_event_handlers = {
+			_twain_module.MSG_XFERREADY: self._twain_handle_transfer_in_memory,
+			_twain_module.MSG_CLOSEDSREQ: self._twain_close_datasource,
+			_twain_module.MSG_CLOSEDSOK: self._twain_save_state,
+			_twain_module.MSG_DEVICEEVENT: self._twain_handle_src_event
+		}
+	#---------------------------------------------------
+	def _twain_event_callback(self, twain_event):
+		_log.Log(gmLog.lData, 'notification of TWAIN event <%s>' % str(twain_event))
+		self.__twain_event_handlers[twain_event]()
+		self.__scanner = None
+		return
+	#---------------------------------------------------
+	def _twain_close_datasource(self):
+		_log.Log(gmLog.lInfo, "being asked to close data source")
+	#---------------------------------------------------
+	def _twain_save_state(self):
+		_log.Log(gmLog.lInfo, "being asked to save application state")
+	#---------------------------------------------------
+	def _twain_handle_src_event(self):
+		_log.Log(gmLog.lInfo, "being asked to handle device specific event")
+	#---------------------------------------------------
+	def _twain_handle_transfer_in_memory(self):
+
+		# FIXME: handle more images
+
+		_log.Log(gmLog.lData, 'receiving image from TWAIN source')
+		_log.Log(gmLog.lData, 'image info: %s' % self.__scanner.GetImageInfo())
+		_log.Log(gmLog.lData, 'image layout: %s' % str(self.__scanner.GetImageLayout()))
+
+		# get image from source
+		(external_data_handle, more_images_pending) = self.__scanner.XferImageNatively()
+		try:
+			# convert DIB to standard bitmap file (always .bmp)
+			_twain_module.DIBToBMFile(external_data_handle, self.__filename)
+		finally:
+			_twain_module.GlobalHandleFree(external_data_handle)
+		_log.Log(gmLog.lData, '%s pending images' % more_images_pending)
+
+		# hide the scanner user interface again
+#		self.__scanner.HideUI()						# needed ?
+#		self.__scanner = None		# not sure why this should be needed, simple_wx does it, though
+
+		self.__done_transferring_image = True
+	#---------------------------------------------------
+	def _twain_handle_transfer_by_file(self):
+
+		# the docs say this is not required to be implemented
+		# therefor we can't use it by default :-(
+		# UNTESTED !!!!
+
+		_log.Log(gmLog.lData, 'receiving image from TWAIN source')
+		_log.Log(gmLog.lData, 'image info: %s' % self.__scanner.GetImageInfo())
+		_log.Log(gmLog.lData, 'image layout: %s' % self.__scanner.GetImageLayout())
+
+		self.__scanner.SetXferFileName(self.__filename)	# FIXME: allow format
+
+		more_images_pending = self.__scanner.XferImageByFile()
+		_log.Log(gmLog.lData, '%s pending images' % more_images_pending)
+
+		# hide the scanner user interface again
+		self.__scanner.HideUI()
+#		self.__scanner = None
+
+		return 
 #=======================================================
 class cSaneScanner:
 
@@ -245,7 +238,7 @@ class cSaneScanner:
 	def close(self):
 		self.__scanner.close()
 	#---------------------------------------------------
-	def acquire_page_into_file(self, delay=None, filename=None, tmpdir=None):
+	def acquire_pages_into_files(self, delay=None, filename=None, tmpdir=None):
 		if filename is None:
 			if tmpdir is None:
 				(handle, filename) = tempfile.mkstemp(suffix='.bmp', prefix='gmScannedObj-')
@@ -269,7 +262,11 @@ class cSaneScanner:
 		except:
 			_log.LogException('Unable to get image from scanner into [%s] !' % filename, sys.exc_info(), verbose=1)
 			return False
-		return filename
+
+		return [filename]
+	#---------------------------------------------------
+	def image_transfer_done(self):
+		return True
 	#---------------------------------------------------
 #	def dummy(self):
 #		pass
@@ -291,6 +288,92 @@ class cSaneScanner:
 #		#self.__scannerbr_x = 412.0
 #		#self.__scannerbr_y = 583.0
 #==================================================
+class cXSaneScanner:
+
+	_filetype = '.png'					# FIXME: configurable, TIFF ?
+	_xsanerc = os.path.expanduser(os.path.join('~',".sane","xsane","xsane.rc"))
+	_xsanerc_backup = os.path.expanduser(os.path.join('~',".sane","xsane","xsane.rc.gnumed.bak"))
+
+	#----------------------------------------------
+	def __init__(self):
+		# while not strictly necessary it is good to fail early
+		# this will tell us fairly safely whether XSane is properly installed
+		if not os.access(cXSaneScanner._xsanerc, os.W_OK):
+			raise IOError('XSane not properly installed for this user, no write access for [%s]' % cXSaneScanner._xsanerc)
+	#----------------------------------------------
+	def close(self):
+		pass
+	#----------------------------------------------
+	def acquire_pages_into_files(self, delay=None, filename=None, tmpdir=None):
+		"""Call XSane.
+
+		<filename name part must have format name-xxx.ext>
+		"""
+		if filename is None:
+			if tmpdir is None:
+				(handle, filename) = tempfile.mkstemp(suffix=cXSaneScanner._filetype, prefix='gmScannedObj-')
+			else:
+				(handle, filename) = tempfile.mkstemp(suffix=cXSaneScanner._filetype, prefix='gmScannedObj-', dir=tmpdir)
+		else:
+			tmp, ext = os.path.splitext(filename)
+			if ext != cXSaneScanner._filetype:
+				filename = filename + cXSaneScanner._filetype
+
+		filename = os.path.abspath(os.path.expanduser(filename))
+		path, name = os.path.split(filename)
+
+		self.__prepare_xsanerc(tmpdir=path)
+
+		gmShellAPI.run_command_in_shell (
+			command = 'xsane --no-mode-selection --save --force-filename "%s"' % filename, 
+			blocking = True
+		)
+
+		self.__restore_xsanerc()
+
+		return glob.glob(filename.replace('###', '*'))
+	#---------------------------------------------------
+	def image_transfer_done(self):
+		return True
+	#----------------------------------------------
+	# internal API
+	#----------------------------------------------
+	def __prepare_xsanerc(self, tmpdir=None):
+
+		shutil.copy2(cXSaneScanner._xsanerc, cXSaneScanner._xsanerc_backup)
+
+		# our closest bet, might contain umalauts
+		enc = locale.getlocale()[1]
+		fread = codecs.open(cXSaneScanner._xsanerc_backup, mode = "rU", encoding = enc)
+		fwrite = codecs.open(cXSaneScanner._xsanerc, mode = "w", encoding = enc)
+
+		val_dict = {
+			'filetype': cXSaneScanner._filetype,
+			'tmp-path': tmpdir,
+			'working-directory': tmpdir
+		}
+
+		for idx, line in enumerate(fread):
+			line = line.replace(fread.newlines, '')
+
+			if idx % 2 == 0:			# even lines are keys
+				key = line.strip('"')
+			else: 						# odd lines are corresponding values
+				try:
+					value = val_dict[key]
+				except KeyError:
+					value = line
+				fwrite.write('"%s"%s' % (key, fread.newlines))
+				fwrite.write('%s%s' % (value, fread.newlines))
+
+		fwrite.close()
+		fread.close()
+
+		return True
+	#----------------------------------------------
+	def __restore_xsanerc(self):
+		shutil.copy2(cXSaneScanner._xsanerc_backup, cXSaneScanner._xsanerc)
+#==================================================
 def _twain_import_module():
 	global _twain_module
 	if _twain_module is None:
@@ -326,68 +409,123 @@ def get_devices():
 		# TWAIN does not support get_devices():
 		# devices can only be selected from within TWAIN itself
 		return None
+	if use_XSane:
+		return None
 	if _sane_import_module():
 		return _sane_module.get_devices()
 	return False
 #-----------------------------------------------------
-def acquire_page_into_file(device=None, delay=None, filename=None, tmpdir=None, calling_window=None):
+def acquire_pages_into_files(device=None, delay=None, filename=None, tmpdir=None, calling_window=None):
 	try:
 		scanner = cTwainScanner(calling_window=calling_window)
 	except gmExceptions.ConstructorError:
-		try:
-			scanner = cSaneScanner(device=device)
-		except gmExceptions.ConstructorError:
-			_log.Log(gmLog.lErr, 'Cannot load any scanner driver (SANE or TWAIN).')
-			return None
+		if use_XSane:
+			_log.Log(gmLog.lData, 'using XSane')
+			try:
+				scanner = cXSaneScanner()
+			except IOError:
+				_log.LogException('Cannot load any scanner driver (XSANE or TWAIN).', verbose=False)
+				return None
+		else:
+			_log.Log(gmLog.lData, 'using SANE directly')
+			try:
+				scanner = cSaneScanner(device=device)
+			except gmExceptions.ConstructorError:
+				_log.LogException('Cannot load any scanner driver (SANE or TWAIN).', verbose=False)
+				return None
 
 	_log.Log(gmLog.lData, 'requested filename: [%s]' % filename)
-	fname = scanner.acquire_page_into_file(filename=filename, delay=delay, tmpdir=tmpdir)
+	fnames = scanner.acquire_pages_into_files(filename=filename, delay=delay, tmpdir=tmpdir)
 	scanner.close()
-	_log.Log(gmLog.lData, 'acquired page into file: [%s]' % fname)
+	_log.Log(gmLog.lData, 'acquired pages into files: %s' % str(fnames))
 
-	return fname
+	return fnames
 #==================================================
 # main
 #==================================================
 if __name__ == '__main__':
 	_log.SetAllLogLevels(gmLog.lData)
 
-	from Gnumed.pycommon import gmI18N
+#	from Gnumed.pycommon import gmI18N
 
 	print "devices:"
 	print get_devices()
 
-	print "getting bitmap #1"
-	if not acquire_page_into_file(device='test:0', filename='x1-test0-1', delay=5):
-		print "error, cannot acquire page"
+	setups = [
+		{'dev': 'test:0', 'file': 'x1-test0-1-0001'},
+		{'dev': 'test:1', 'file': 'x2-test1-1-0001.bmp'},
+		{'dev': 'test:0', 'file': 'x3-test0-2-0001.bmp-ccc'}
+	]
+
+	idx = 1
+	for setup in setups:
+		print "scanning page #%s from device [%s]" % (idx, setup['dev'])
+		idx += 1
+		fnames = acquire_pages_into_files(device = setup['dev'], filename = setup['file'], delay = (idx*5))
+		if fnames is False:
+			print "error, cannot acquire page"
+		else:
+			print " image files:", fnames
 
-	print "getting bitmap #2"
-	if not acquire_page_into_file(device='test:1', filename='x2-test1-1.bmp', delay=10):
-		print "error, cannot acquire page"
 
-	print "getting bitmap #3"
-	if not acquire_page_into_file(device='test:0', filename='x3-test0-2.bmp-ccc', delay=15):
-		print "error, cannot acquire page"
-	
 #==================================================
 # $Log: gmScanBackend.py,v $
-# Revision 1.16.2.5  2006/08/28 22:04:11  ncq
-# - eventually fix TWAIN scanning
+# Revision 1.34  2007/01/19 14:06:17  ncq
+# - do not attempt to handle several scans at once
 #
-# Revision 1.16.2.4  2006/08/28 21:02:23  ncq
-# - absolutize scan file path names on return
+# Revision 1.33  2007/01/19 13:37:39  ncq
+# - cannot wait on semaphore as it blocks the TWAIN GUI
+#
+# Revision 1.32  2007/01/19 13:29:35  ncq
+# - try semaphore on TWAIN scanning to detect finish
+#
+# Revision 1.31  2007/01/19 12:43:39  ncq
+# - properly return False if initing scanner was cancelled
+#
+# Revision 1.30  2007/01/19 10:55:56  ncq
+# - clean up
+#
+# Revision 1.29  2007/01/18 19:33:09  ncq
+# - fix typo
+#
+# Revision 1.28  2007/01/18 18:43:07  ncq
+# - added print "" for debugging
+#
+# Revision 1.27  2007/01/18 17:58:34  ncq
+# - explicitely wait for TWAIN memory scan transfer to finish
+#
+# Revision 1.26  2007/01/18 17:14:54  ncq
+# - closer to twain example code
+#
+# Revision 1.25  2007/01/18 14:41:29  ncq
+# - use ShowModal in RequestAcquire()
+#
+# Revision 1.24  2007/01/18 13:27:16  ncq
+# - try to comply more closely with TWAIN sample wx app
+#
+# Revision 1.23  2007/01/18 13:03:25  ncq
+# - no ProductName
+#
+# Revision 1.22  2007/01/18 12:34:01  ncq
+# - must init self.__scanner/self.__src_manager
+#
+# Revision 1.21  2007/01/18 12:08:56  ncq
+# - try to once again fix/improve TWAIN scanning
+#
+# Revision 1.20  2006/12/27 16:42:53  ncq
+# - cleanup
+# - add XSane interface in cXSaneScanner as worked out by Kai Schmidt
+# - acquire_pages_into_files() now returns a list
+# - fix test suite
+#
+# Revision 1.19  2006/09/02 21:11:59  ncq
+# - improved test suite
+#
+# Revision 1.18  2006/08/29 18:41:58  ncq
 # - improve test suite
 #
-# Revision 1.16.2.3  2006/08/28 17:25:46  ncq
-# - make TWAIN class entirely self-contained
-# - close source/manager according to documentation
-#
-# Revision 1.16.2.2  2006/08/28 16:53:01  ncq
-# - try to fix close logic in TWAIN
-# - keep global scanner singleton if module level acquire_page_into_file() is used
-#
-# Revision 1.16.2.1  2006/08/28 11:36:40  ncq
-# - fix improper/pre-mature closing of TWAIN scanner device
+# Revision 1.17  2006/08/29 18:33:02  ncq
+# - forward port TWAIN fixes from 0.2 branch
 #
 # Revision 1.16  2006/05/14 20:42:20  ncq
 # - properly handle get_devices()
