@@ -2,8 +2,8 @@
 """
 #================================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/wxpython/gmEMRBrowser.py,v $
-# $Id: gmEMRBrowser.py,v 1.55 2006/06/26 13:03:22 ncq Exp $
-__version__ = "$Revision: 1.55 $"
+# $Id: gmEMRBrowser.py,v 1.66 2007/01/16 18:00:59 ncq Exp $
+__version__ = "$Revision: 1.66 $"
 __author__ = "cfmoro1976@yahoo.es, sjtan@swiftdsl.com.au, Karsten.Hilbert@gmx.net"
 __license__ = "GPL"
 
@@ -11,14 +11,10 @@ __license__ = "GPL"
 import sys, types, os.path, StringIO
 
 # 3rd party
-try:
-	import wxversion
-	import wx
-except ImportError:
-	from wxPython import wx
+import wx
 
 # GNUmed libs
-from Gnumed.pycommon import gmLog, gmI18N, gmPG, gmDispatcher, gmSignals
+from Gnumed.pycommon import gmLog, gmI18N, gmDispatcher, gmSignals
 from Gnumed.exporters import gmPatientExporter
 from Gnumed.business import gmEMRStructItems, gmPerson, gmSOAPimporter
 from Gnumed.wxpython import gmRegetMixin, gmGuiHelpers, gmEMRStructWidgets, gmSOAPWidgets, gmEditArea
@@ -28,9 +24,7 @@ _log = gmLog.gmDefLog
 _log.Log(gmLog.lInfo, __version__)
 
 # module level constants
-dialog_CANCELLED = -1
 dialog_OK = -2
-MAX_EXPANSION_HISTORY = 2
 
 #============================================================
 def export_emr_to_ascii(parent=None):
@@ -42,7 +36,7 @@ def export_emr_to_ascii(parent=None):
 	# sanity checks
 	pat = gmPerson.gmCurrentPatient()
 	if not pat.is_connected():
-		gmGuiHelpers.gm_beep_statustext(_('Cannot export EMR. No active patient.'), gmLog.lErr)
+		gmGuiHelpers.gm_statustext(_('Cannot export EMR. No active patient.'), gmLog.lErr)
 		return False
 	if parent is None:
 		_log.Log(gmLog.lErr, 'cannot dump emr in gui mode without parent widget')
@@ -75,11 +69,8 @@ def export_emr_to_ascii(parent=None):
 		exporter.dump_med_docs()
 		output_file.close()
 		gmGuiHelpers.gm_show_info('EMR successfully exported to file: %s' % fname, _('emr_dump'), gmLog.lInfo)
-
-
-
 #============================================================
-class cEMRTree(wx.TreeCtrl):
+class cEMRTree(wx.TreeCtrl, gmGuiHelpers.cTreeExpansionHistoryMixin):
 	"""This wx.TreeCtrl derivative displays a tree view of the medical record."""
 
 	#--------------------------------------------------------
@@ -89,6 +80,8 @@ class cEMRTree(wx.TreeCtrl):
 		kwds['style'] = wx.TR_HAS_BUTTONS | wx.NO_BORDER
 		wx.TreeCtrl.__init__(self, parent, id, *args, **kwds)
 
+		gmGuiHelpers.cTreeExpansionHistoryMixin.__init__(self)
+
 		try:
 			self.__narr_display = kwds['narr_display']
 			del kwds['narr_display']
@@ -98,18 +91,15 @@ class cEMRTree(wx.TreeCtrl):
 		self.__exporter = gmPatientExporter.cEmrExport(patient = self.__pat)
 
 		self.__make_popup_menus()
-
-#		self.expansion_history = ExpansionHistory(self, self.__emr_tree)
-
 		self.__register_events()
 	#--------------------------------------------------------
 	# external API
 	#--------------------------------------------------------
 	def refresh(self):
 		if not self.__pat.is_connected():
-			gmGuiHelpers.gm_beep_statustext (
-				_('Cannot load documents. No active patient.'),
-				gmLog.lErr
+			gmGuiHelpers.gm_statustext (
+				_('Cannot load clinical narrative. No active patient.'),
+				gmLog.lWarn
 			)
 			return False
 
@@ -134,9 +124,8 @@ class cEMRTree(wx.TreeCtrl):
 		# FIXME: error handling
 
 		wx.BeginBusyCursor()
-		# remember expansion for previous patient
-		#self.expansion_history.remember_expansion()
-		# clean old tree
+
+		self.snapshot_expansion()
 		self.DeleteAllItems()
 
 		# init new tree
@@ -159,7 +148,8 @@ class cEMRTree(wx.TreeCtrl):
 			self.__narr_display.Clear()
 			self.__narr_display.WriteText('%s\n%s\n\n' % (label, underline))
 			self.__narr_display.WriteText(self.__exporter.get_summary_info(0))
-#		self.expansion_history.restore_expansion()
+
+		self.restore_expansion()
 
 		wx.EndBusyCursor()
 		return True
@@ -169,38 +159,36 @@ class cEMRTree(wx.TreeCtrl):
 
 		# - episodes
 		self.__epi_context_popup = wx.Menu()
-		menu_id = wx.NewId()
-		self.__epi_context_popup.AppendItem(wx.MenuItem(self.__epi_context_popup, menu_id, _('rename episode')))
-		wx.EVT_MENU(self.__epi_context_popup, menu_id, self.__rename_episode)
 
 		menu_id = wx.NewId()
-		self.__epi_context_popup.AppendItem(wx.MenuItem(self.__epi_context_popup, menu_id, _('move episode to another health issue')))
-		wx.EVT_MENU(self.__epi_context_popup, menu_id, self.__relink_episode2issue)
-		# close episode
+		self.__epi_context_popup.AppendItem(wx.MenuItem(self.__epi_context_popup, menu_id, _('Edit episode details')))
+		wx.EVT_MENU(self.__epi_context_popup, menu_id, self.__edit_episode)
 		# delete episode
 		# attach all encounters to another episode
 
 		# - encounters
 		self.__enc_context_popup = wx.Menu()
 		menu_id = wx.NewId()
+		# - move data
 		self.__enc_context_popup.AppendItem(wx.MenuItem(self.__enc_context_popup, menu_id, _('move encounter data to another episode')))
 		wx.EVT_MENU(self.__enc_context_popup, menu_id, self.__relink_encounter_data2episode)
-		# attach encounter to another patient
+		# - edit encounter details
+		self.__enc_context_popup.AppendItem(wx.MenuItem(self.__enc_context_popup, menu_id, _('edit consultation details')))
+		wx.EVT_MENU(self.__enc_context_popup, menu_id, self.__edit_consultation_details)
 		# delete encounter
 		# attach all progress notes to another encounter
 
 		# - health issues
 		self.__issue_context_popup = wx.Menu()
 		menu_id = wx.NewId()
-		self.__issue_context_popup.AppendItem(wx.MenuItem(self.__issue_context_popup, menu_id, _('rename health issue')))
-		wx.EVT_MENU(self.__issue_context_popup, menu_id, self.__rename_issue)
+		self.__issue_context_popup.AppendItem(wx.MenuItem(self.__issue_context_popup, menu_id, _('edit health issue')))
+		wx.EVT_MENU(self.__issue_context_popup, menu_id, self.__edit_issue)
 		
 		# - root node
 		self.__root_context_popup = wx.Menu()
 		menu_id = wx.NewId()
 		self.__root_context_popup.AppendItem(wx.MenuItem(self.__root_context_popup, menu_id, _('create health issue')))
 		wx.EVT_MENU(self.__root_context_popup, menu_id, self.__create_issue)
-		# print " add new episode to issue"
 		# print " attach issue to another patient"
 		# print " move all episodes to another issue"
 	#--------------------------------------------------------
@@ -209,94 +197,12 @@ class cEMRTree(wx.TreeCtrl):
 		self.__epi_context_popup.SetTitle(_('Episode %s') % self.__curr_node_data['description'])
 		self.PopupMenu(self.__epi_context_popup, pos)
 	#--------------------------------------------------------
-	def __rename_episode(self, event):
-		dlg = wx.TextEntryDialog (
-			parent = self,
-			message = _('Old: "%s"\nPlease type the new description:\n') % self.__curr_node_data['description'],
-			caption = _('Renaming episode ...'),
-			defaultValue = ''
-		)
+	def __edit_episode(self, event):
+		node_data = self.GetPyData(self.__curr_node)
+		dlg = gmEMRStructWidgets.cEpisodeEditAreaDlg(parent=self, episode=node_data)
 		result = dlg.ShowModal()
-		if result == wx.ID_CANCEL:
-			return
-		new_name = dlg.GetValue().strip()
-		if new_name == '':
-			return
-		if new_name == self.__curr_node_data['description']:
-			return
-		if self.__curr_node_data.rename(new_name):
-			# avoid collapsing view
-			self.SetItemText(self.__curr_node, new_name)
-			return
-		gmGuiHelpers.gm_show_error(
-			_('Cannot rename episode from\n\n [%s]\n\nto\n\n [%s].') % (self.__curr_node_data['description'], new_name),
-			_('Error renaming episode ...'),
-			gmLog.lErr
-		)
-		return
-	#--------------------------------------------------------
-	def __relink_episode2issue(self, event):
-		if self.__curr_node_data['health_issue'] is None:
-			msg = _(
-				'\nThe episode\n'
-				'  [%(epi)s]\n'
-				'currently does not belong to a health issue.\n\n'
-				'Please select the health issue you want to move it to:\n'
-			) % {'epi': self.__curr_node_data['description']}
-		else:
-			msg = _(
-				'\nThe episode\n'
-				'  [%(epi)s]\n'
-				'currently belongs to the health issue\n'
-				'  [%(issue)s]\n\n'
-				'Please select the health issue you want to move it to:\n'
-			) % {'epi': self.__curr_node_data['description'], 'issue': self.__curr_node_data['health_issue']}
-
-		dlg = gmEMRStructWidgets.cIssueSelectionDlg(self, -1, message=msg)
-		result = dlg.ShowModal()
-		if result == wx.ID_CANCEL:
-			dlg.Destroy()
-			return
-
-		pk_issue = dlg._PhWheel_issue.GetData()
-		dlg.Destroy()
-
-		if self.__curr_node_data['pk_health_issue'] == pk_issue:
-			return
-
-		issue = gmEMRStructItems.cHealthIssue(aPK_obj=pk_issue)
-		issue.close_expired_episode()
-		if issue.has_open_episode() and self.__curr_node_data['episode_open']:
-			if not gmGuiHelpers.gm_show_question (
-				_('The episode [%(epi)s] is open. However, there\n'
-				  'already exists an open episode for health issue\n\n'
-				  ' [%(issue)s]\n\n'
-				  'Do you want to close the episode [%(epi)s]\n'
-				  'before moving it to the new health issue ?\n'
-				) % {'epi': self.__curr_node_data['description'], 'issue': issue['description']},
-				_('Moving episode')
-			):
-				return
-			self.__curr_node_data['episode_open'] = False
-
-		self.__curr_node_data['pk_health_issue'] = pk_issue
-		success, data = self.__curr_node_data.save_payload()
-		if not success:
-			gmGuiHelpers.gm_show_error (
-				_('Cannot move episode\n [%(epi)s]\n into health issue\n [%(issue)s].') % {
-					'epi': self.__curr_node_data['description'],
-					'issue': issue['description']
-				},
-				_('Moving episode')
-			)
-			return
-
-		self.__populate_tree()
-		return
-	#--------------------------------------------------------
-	def __close_episode(self, event):
-		print "closing episode"
-		print self.__curr_node_data
+		if result == wx.ID_OK:
+			self.__populate_tree()
 	#--------------------------------------------------------
 	def __delete_episode(self, event):
 		print "deleting episode"
@@ -310,6 +216,12 @@ class cEMRTree(wx.TreeCtrl):
 	def __handle_encounter_context(self, pos=wx.DefaultPosition):
 		self.PopupMenu(self.__enc_context_popup, pos)
 	#--------------------------------------------------------
+	def __edit_consultation_details(self, event):
+		node_data = self.GetPyData(self.__curr_node)
+		dlg = gmEMRStructWidgets.cEncounterEditAreaDlg(parent=self, encounter=node_data)
+		dlg.ShowModal()
+		self.__populate_tree()
+	#--------------------------------------------------------
 	def __relink_encounter_data2episode(self, event):
 
 		node_parent = self.GetItemParent(self.__curr_node)
@@ -339,8 +251,8 @@ class cEMRTree(wx.TreeCtrl):
 			else:
 				print "No target episode node found"
 			self.__curr_node_data.transfer_clinical_data (
-				from_episode = owning_episode,
-				to_episode = target_episode_node
+				source_episode = owning_episode,
+				target_episode = target_episode_node
 			)
 		
 		# FIXME: GNUmed internal signal
@@ -348,7 +260,7 @@ class cEMRTree(wx.TreeCtrl):
 	#--------------------------------------------------------
 	def __find_node(self, root, data_object):
 		nodes = []
-		id , cookie = self.GetFirstChild( root)
+		id , cookie = self.GetFirstChild(root)
 
 		#print "DEBUG id , cookie", id, cookie
 		#print "DEBUG id dict is ", id.__dict__.keys()
@@ -383,30 +295,12 @@ class cEMRTree(wx.TreeCtrl):
 #		self.__issue_context_popup.SetTitle(_('Episode %s') % episode['description'])
 		self.PopupMenu(self.__issue_context_popup, pos)
 	#--------------------------------------------------------
-	def __rename_issue(self, event):
-		dlg = wx.TextEntryDialog (
-			parent = self,
-			message = _('Old: "%s"\nPlease type the new description:\n') % self.__curr_node_data['description'],
-			caption = _('Renaming health issue ...'),
-			defaultValue = ''
-		)
-		result = dlg.ShowModal()
-		if result == wx.ID_CANCEL:
-			return
-		new_name = dlg.GetValue().strip()
-		if new_name == '':
-			return
-		if new_name == self.__curr_node_data['description']:
-			return
-		if self.__curr_node_data.rename(new_name):
-			#DEBUG why doesn't a refresh occur ?
-			self.SetItemText(self.__curr_node, new_name)
-			return
-		gmGuiHelpers.gm_show_error (
-			_('Cannot rename health issue from\n\n [%s]\n\nto\n\n [%s].') % (self.__curr_node_data['description'], new_name),
-			_('Error renaming health issue ...'),
-			gmLog.lErr
-		)
+	def __edit_issue(self, event):
+		ea = gmEMRStructWidgets.cHealthIssueEditAreaDlg(parent=self, id=-1, issue=self.__curr_node_data)
+		ea.ShowModal()
+		self.SetItemText(self.__curr_node, self.__curr_node_data['description'])
+		self.Refresh()
+		#self.Update()
 		return
 	#--------------------------------------------------------
 	# root
@@ -414,10 +308,13 @@ class cEMRTree(wx.TreeCtrl):
 		self.PopupMenu(self.__root_context_popup, pos)		
 	#--------------------------------------------------------
 	def __create_issue(self, event):
+
+		return True
+
 		# FIXME: refactor this code, present in three places
 		pat = gmPerson.gmCurrentPatient()
 		if not pat.is_connected():
-			gmGuiHelpers.gm_beep_statustext(_('Cannot add health issue. No active patient.'), gmLog.lErr)
+			gmGuiHelpers.gm_statustext(_('Cannot add health issue. No active patient.'), gmLog.lErr)
 			return False
 		ea = gmEMRStructWidgets.cHealthIssueEditArea (
 			self,
@@ -489,6 +386,34 @@ class cEMRTree(wx.TreeCtrl):
 		else:
 			print "error: unknown node type, no popup menu"
 		event.Skip()
+	#--------------------------------------------------------
+	def OnCompareItems (self, node1=None, node2=None):
+		"""Used in sorting items.
+
+		-1: 1 < 2
+		 0: 1 = 2
+		 1: 1 > 2
+		"""
+		item1 = self.GetPyData(node1)
+		item2 = self.GetPyData(node2)
+
+		if isinstance(item1, gmEMRStructItems.cEpisode):
+			start1 = item1.get_access_range()[0]
+			start2 = item2.get_access_range()[0]
+			if start1 == start2:
+				return 0
+			if start1 < start2:
+				return -1
+			return 1
+
+		if isinstance(item1, gmEMRStructItems.cEncounter):
+			if item1['started'] == item2['started']:
+				return 0
+			if item1['started'] < item2['started']:
+				return -1
+			return 1
+
+		return 0
 #================================================================
 class cScrolledEMRTreePnl(wxgScrolledEMRTreePnl.wxgScrolledEMRTreePnl, gmRegetMixin.cRegetOnPaintMixin):
 	"""A scrollable panel holding an EMR tree.
@@ -533,10 +458,11 @@ class cSplittedEMRTreeBrowserPnl(wxgSplittedEMRTreeBrowserPnl.wxgSplittedEMRTree
 		self._splitter_browser.SetSashPosition(self._splitter_browser.GetSizeTuple()[0]/3, True)
 		return True
 #================================================================
-class cEMRJournalPanel(wx.Panel, gmRegetMixin.cRegetOnPaintMixin):
+#class cEMRJournalPanel(wx.Panel, gmRegetMixin.cRegetOnPaintMixin):
+class cEMRJournalPanel(wx.Panel):
 	def __init__(self, *args, **kwargs):
 		wx.Panel.__init__(self, *args, **kwargs)
-		gmRegetMixin.cRegetOnPaintMixin.__init__(self)
+#		gmRegetMixin.cRegetOnPaintMixin.__init__(self)
 
 		self.__do_layout()
 
@@ -567,20 +493,13 @@ class cEMRJournalPanel(wx.Panel, gmRegetMixin.cRegetOnPaintMixin):
 		return 1
 	#--------------------------------------------------------
 	def _on_post_patient_selection(self):
-		self._schedule_data_reget()
+		# FIXME: check for visibility
+		self.__journal.SetValue(u'')
+		return True
 	#--------------------------------------------------------
-	# reget mixin API
+	# notebook plugin API
 	#--------------------------------------------------------
-	def _populate_with_data(self):
-		"""Fills UI with data.
-		"""
-#		self.__reset_ui_content()
-		if self.refresh_journal():
-			return True
-		return False
-	#--------------------------------------------------------
-	def refresh_journal(self):
-
+	def repopulate_ui(self):
 		# get data from backend
 		txt = StringIO.StringIO()
 		exporter = gmPatientExporter.cEMRJournalExporter()
@@ -597,91 +516,15 @@ class cEMRJournalPanel(wx.Panel, gmRegetMixin.cRegetOnPaintMixin):
 		self.__journal.SetValue(txt.getvalue())
 		txt.close()
 		return True
-
-
-#================================================================
-
-class ExpansionHistory:
-	def __init__(self, emr_browser, tree):
-		self._browser =emr_browser
-		self._tree = tree
-
-		
-		self._expansion_history = {} # for remembering expansion states of previously visited histories
-		self.__last_pat_desc = None
-		self.__selected_node = None
-		
-	def remember_expansion(self):
-		ix = self.__last_pat_desc
-		if ix is None:
-			return
-		#print "\nDEBUG Remembering with index",ix , " type of ix is ", type(ix)
-		#print
-		l = []
-		
-		#if cache full resize 
-		if not self._expansion_history.has_key(ix) and  len( self._expansion_history) > MAX_EXPANSION_HISTORY:
-				i = MAX_EXPANSION_HISTORY / 4
-				for k in self._expansion_history.keys():
-					del (self._expansion_history[k])
-					i -= 1
-					if i < 1:
-						break
-				
-		self._expansion_history[ix] = l
-		self._record_expansion(self._tree, self._tree.GetRootItem(), l)
-		
-
-	def _record_expansion(self,tree,  root, l):
-		if not root or not root.IsOk():
-			return
-		id, cookie = tree.GetFirstChild( root)
-		#print "id", id, " is Ok", id.IsOk()
-		while id.IsOk():
-			expanded = tree.IsExpanded(id)
-			#print "id expanded = ", expanded
-			l2 = [tree.IsSelected(id)]
-			
-			if expanded:
-				self._record_expansion(tree, id, l2)
-			#print "expansion at ", id , " is ",  l2
-			l.append(l2)
-			id, cookie = tree.GetNextChild(root,  cookie)
-		
-	def restore_expansion(self):
-		ix = gmPerson.gmCurrentPatient().get_identity()['description']
-		self.__last_pat_desc  = ix 
-
-		print "DEBUG restoring with ",ix 
-		if self._expansion_history.has_key(ix):
-			l = self._expansion_history[ix]
-		        self.__selected_node = None
-			self._restore_expand( self._tree, self._tree.GetRootItem(), l)
-
-		if self.__selected_node:		
-			self._tree.SelectItem( self.__selected_node)	
-
-	
-	
-	def _restore_expand(self, tree, root, l):
-		id, cookie = tree.GetFirstChild(root)
-		for x in l:
-			if not id.IsOk():
-				break
-			selected = x[0]
-			if selected:
-				self.__selected_node = id	
-			x = x[1:]
-			if x <>  []:
-				tree.Expand( id)
-				self._restore_expand( tree, id, x)
-			id, cookie = tree.GetNextChild(root, cookie)
-	
-			
-
-	
-
-		
+	#--------------------------------------------------------
+	# reget mixin API
+	#--------------------------------------------------------
+#	def _populate_with_data(self):
+#		"""Fills UI with data.
+#		"""
+#		if self.refresh_journal():
+#			return True
+#		return False
 #================================================================
 # MAIN
 #----------------------------------------------------------------
@@ -698,10 +541,6 @@ if __name__ == '__main__':
 		sys.exit("Cannot run without config file.")
 
 	try:
-		# make sure we have a db connection
-		gmPG.set_default_client_encoding('latin1')
-		pool = gmPG.ConnectionPool()
-		
 		# obtain patient
 		patient = gmPerson.ask_for_patient()
 		if patient is None:
@@ -727,16 +566,48 @@ if __name__ == '__main__':
 		_log.LogException("unhandled exception caught !", sys.exc_info(), 1)
 		# but re-raise them
 		raise
-	try:
-		pool.StopListeners()
-	except:
-		_log.LogException('unhandled exception caught', sys.exc_info(), verbose=1)
-		raise
 
 	_log.Log (gmLog.lInfo, "closing emr browser...")
 
 #================================================================
 # $Log: gmEMRBrowser.py,v $
+# Revision 1.66  2007/01/16 18:00:59  ncq
+# - cleanup
+# - explicitely sort episodes and encounters by when they were started
+#
+# Revision 1.65  2007/01/15 13:08:55  ncq
+# - remove explicit __relink_episode2issue as episode details editor now does it
+#
+# Revision 1.64  2007/01/13 22:26:55  ncq
+# - remove cruft
+# - mix expansion history into emr tree browser
+#
+# Revision 1.63  2007/01/06 23:41:40  ncq
+# - missing :
+#
+# Revision 1.62  2007/01/04 23:41:36  ncq
+# - use new episode edit area
+#
+# Revision 1.61  2006/12/25 22:50:50  ncq
+# - add editing of consultation details from EMR tree right-click popup menu
+#
+# Revision 1.60  2006/12/13 23:32:41  ncq
+# - emr journal on a diet
+#
+# Revision 1.59  2006/11/24 14:20:44  ncq
+# - used shiny new health issue edit area in issue context menu
+# - refresh tree after editing health issue
+#
+# Revision 1.58  2006/11/24 10:01:31  ncq
+# - gm_beep_statustext() -> gm_statustext()
+#
+# Revision 1.57  2006/11/05 16:02:00  ncq
+# - cleanup
+#
+# Revision 1.56  2006/10/09 12:22:27  ncq
+# - some cleanup
+# - adjust to changed signature of encounter.transfer_clinical_data()
+#
 # Revision 1.55  2006/06/26 13:03:22  ncq
 # - improve menu strings
 # - implement moving episodes among issues
