@@ -11,8 +11,8 @@ to anybody else.
 """
 # ========================================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/wxpython/gmGuiHelpers.py,v $
-# $Id: gmGuiHelpers.py,v 1.36.2.1 2006/09/02 17:59:35 ncq Exp $
-__version__ = "$Revision: 1.36.2.1 $"
+# $Id: gmGuiHelpers.py,v 1.48 2007/01/20 22:52:27 ncq Exp $
+__version__ = "$Revision: 1.48 $"
 __author__  = "K. Hilbert <Karsten.Hilbert@gmx.net>"
 __license__ = "GPL (details at http://www.gnu.org)"
 
@@ -23,41 +23,158 @@ if __name__ == '__main__':
 
 import wx
 
-from Gnumed.pycommon import gmLog, gmGuiBroker, gmPG
+from Gnumed.pycommon import gmLog, gmGuiBroker, gmPG2, gmLoginInfo
+from Gnumed.wxGladeWidgets import wxg3ButtonQuestionDlg
+
 _log = gmLog.gmDefLog
 _log.Log(gmLog.lData, __version__)
 
 _set_status_text = None
 # ========================================================================
-def gm_show_error(aMessage = None, aTitle = None, aLogLevel = None):
-	if aMessage is None:
-		aMessage = _('programmer forgot to specify error message')
+class c3ButtonQuestionDlg(wxg3ButtonQuestionDlg.wxg3ButtonQuestionDlg):
 
-	if aLogLevel is not None:
-		log_msg = string.replace(aMessage, '\015', ' ')
-		log_msg = string.replace(log_msg, '\012', ' ')
-		_log.Log(aLogLevel, log_msg)
+	def __init__(self, *args, **kwargs):
 
-	aMessage = str(aMessage) + _("\n\nPlease consult the error log for all the gory details !")
+		caption = kwargs['caption']
+		question = kwargs['question']
+		button_defs = kwargs['button_defs'][:3]
 
-	if aTitle is None:
-		aTitle = _('generic error message')
+		del kwargs['caption']
+		del kwargs['question']
+		del kwargs['button_defs']
 
-	print "-" * len(aTitle)
-	print aTitle
-	print "-" * len(aTitle)
-	print aMessage
+		wxg3ButtonQuestionDlg.wxg3ButtonQuestionDlg.__init__(self, *args, **kwargs)
 
-	dlg = wx.MessageDialog (
-		parent = None,
-		message = aMessage,
-		caption = aTitle,
-		style = wx.OK | wx.ICON_ERROR | wx.STAY_ON_TOP
-	)
-	dlg.ShowModal()
-	dlg.Destroy()
-	return True
-#-------------------------------------------------------------------------
+		self.SetTitle(title = caption)
+		self._LBL_question.SetLabel(label = question)
+
+		buttons = [self._BTN_1, self._BTN_2, self._BTN_3]
+		for idx in range(len(button_defs)):
+			buttons[idx].SetLabel(label = button_defs[idx]['label'])
+			buttons[idx].SetToolTipString(button_defs[idx]['tooltip'])
+			if button_defs[idx]['default']:
+				buttons[idx].SetDefault()
+
+		self.Fit()
+	#--------------------------------------------------------
+	# event handlers
+	#--------------------------------------------------------
+	def _on_BTN_1_pressed(self, evt):
+		if self.IsModal():
+			self.EndModal(wx.ID_YES)
+		else:
+			self.Close()
+	#--------------------------------------------------------
+	def _on_BTN_2_pressed(self, evt):
+		if self.IsModal():
+			self.EndModal(wx.ID_NO)
+		else:
+			self.Close()
+# ========================================================================
+class cTreeExpansionHistoryMixin:
+	"""TreeCtrl mixin class to record expansion history."""
+	def __init__(self):
+		if not isinstance(self, wx.TreeCtrl):
+			raise TypeError('[%s]: mixin can only be applied to wx.TreeCtrl, not [%s]' % (cTreeExpansionHistoryMixin, self.__class__.__name__))
+		self.expansion_state = {}
+	#--------------------------------------------------------
+	# public API
+	#--------------------------------------------------------
+	def snapshot_expansion(self):
+		self.__record_subtree_expansion(start_node_id = self.GetRootItem())
+	#--------------------------------------------------------
+	def restore_expansion(self):
+		if len(self.expansion_state) == 0:
+			return True
+		self.__restore_subtree_expansion(start_node_id = self.GetRootItem())
+	#--------------------------------------------------------
+	def print_expansion(self):
+		if len(self.expansion_state) == 0:
+			print "currently no expansion snapshot available"
+			return True
+		print "last snapshot of state of expansion"
+		print "-----------------------------------"
+		print "listing expanded nodes:"
+		for node_id in self.expansion_state.keys():
+			print "node ID:", node_id
+			print "  selected:", self.expansion_state[node_id]
+	#--------------------------------------------------------
+	# internal API
+	#--------------------------------------------------------
+	def __record_subtree_expansion(self, start_node_id=None):
+		"""This records node expansion states based on the item label.
+
+		A side effect of this is that identically named items can
+		become unduly synchronized in their expand state after a
+		snapshot/restore cycle.
+
+		Better choices might be
+
+			id(item.GetPyData()) or
+			item.GetPyData().get_tree_uid()
+
+		where get_tree_uid():
+
+			'[%s:%s]' % (self.__class__.__name__, id(self))
+
+		or some such. This would survive renaming of the item.
+
+		For database items it may be useful to include the
+		primary key which would - contrary to id() - survive
+		reloads from the database.
+		"""
+		# protect against empty tree where not even
+		# a root node exists
+		if not start_node_id.IsOk():
+			return True
+
+		if not self.IsExpanded(start_node_id):
+			return True
+
+		self.expansion_state[self.GetItemText(start_node_id)] = self.IsSelected(start_node_id)
+
+		child_id, cookie = self.GetFirstChild(start_node_id)
+		while child_id.IsOk():
+			self.__record_subtree_expansion(start_node_id = child_id)
+			child_id, cookie = self.GetNextChild(start_node_id, cookie)
+
+		return
+	#--------------------------------------------------------
+	def __restore_subtree_expansion(self, start_node_id=None):
+		start_node_label = self.GetItemText(start_node_id)
+		try:
+			node_selected = self.expansion_state[start_node_label]
+		except KeyError:
+			return
+
+		self.Expand(start_node_id)
+		if node_selected:
+			self.SelectItem(start_node_id)
+
+		child_id, cookie = self.GetFirstChild(start_node_id)
+		while child_id.IsOk():
+			self.__restore_subtree_expansion(start_node_id = child_id)
+			child_id, cookie = self.GetNextChild(start_node_id, cookie)
+
+		return
+# ========================================================================
+class cFileDropTarget(wx.FileDropTarget):
+	"""Generic file drop target class.
+
+	Protocol:
+		Widgets being declared file drop targets
+		must provide the method:
+
+			add_filename(filenames)
+	"""
+	#-----------------------------------------------
+	def __init__(self, target):
+		wx.FileDropTarget.__init__(self)
+		self.target = target
+	#-----------------------------------------------
+	def OnDropFiles(self, x, y, filenames):
+		self.target.add_filenames(filenames)
+# ========================================================================
 def gm_SingleChoiceDialog(aMessage = None, aTitle = None, aLogLevel = None, choices = None):
     if aMessage is None:
         aMessage = _('programmer forgot to specify info message')
@@ -85,6 +202,35 @@ def gm_SingleChoiceDialog(aMessage = None, aTitle = None, aLogLevel = None, choi
     else:
         return False
 #-------------------------------------------------------------------------
+def gm_show_error(aMessage = None, aTitle = None, aLogLevel = None):
+	if aMessage is None:
+		aMessage = _('programmer forgot to specify error message')
+
+	if aLogLevel is not None:
+		log_msg = string.replace(aMessage, '\015', ' ')
+		log_msg = string.replace(log_msg, '\012', ' ')
+		_log.Log(aLogLevel, log_msg)
+
+	aMessage += _("\n\nPlease consult the error log for all the gory details !")
+
+	if aTitle is None:
+		aTitle = _('generic error message')
+
+	print "-" * len(aTitle)
+	print aTitle
+	print "-" * len(aTitle)
+	print aMessage
+
+	dlg = wx.MessageDialog (
+		parent = None,
+		message = aMessage,
+		caption = aTitle,
+		style = wx.OK | wx.ICON_ERROR | wx.STAY_ON_TOP
+	)
+	dlg.ShowModal()
+	dlg.Destroy()
+	return True
+#-------------------------------------------------------------------------
 def gm_show_info(aMessage = None, aTitle = None, aLogLevel = None):
 	if aMessage is None:
 		aMessage = _('programmer forgot to specify info message')
@@ -129,32 +275,40 @@ def gm_show_warning(aMessage = None, aTitle = None, aLogLevel = None):
 	dlg.Destroy()
 	return True
 #-------------------------------------------------------------------------
-def gm_show_question(aMessage = 'programmer forgot to specify question', aTitle = 'generic user question dialog'):
-	dlg = wx.MessageDialog(
+def gm_show_question(aMessage = 'programmer forgot to specify question', aTitle = 'generic user question dialog', cancel_button=False):
+	if cancel_button:
+		style = wx.YES_NO | wx.CANCEL | wx.ICON_QUESTION | wx.STAY_ON_TOP
+	else:
+		style = wx.YES_NO | wx.ICON_QUESTION | wx.STAY_ON_TOP
+
+	dlg = wx.MessageDialog (
 		None,
 		aMessage,
 		aTitle,
-		wx.YES_NO | wx.ICON_QUESTION | wx.STAY_ON_TOP
+		style
 	)
 	btn_pressed = dlg.ShowModal()
 	dlg.Destroy()
 
 	if btn_pressed == wx.ID_YES:
 		return True
-	else:
+	elif btn_pressed == wx.ID_NO:
 		return False
+	else:
+		return None
 #-------------------------------------------------------------------------
 # FIXME: actually make this use a signal !
-def gm_beep_statustext(aMessage=None, aLogLevel=None):
+def gm_statustext(aMessage=None, aLogLevel=None, beep=True):
 	if aMessage is None:
 		aMessage = _('programmer forgot to specify alert message')
 
 	if aLogLevel is not None:
-		log_msg = string.replace(aMessage, '\015', ' ')
-		log_msg = string.replace(log_msg, '\012', ' ')
+		log_msg = aMessage.replace('\015', ' ')
+		log_msg = log_msg.replace('\012', ' ')
 		_log.Log(aLogLevel, log_msg)
 
-	wx.Bell()
+	if beep:
+		wx.Bell()
 
 	# only now and here can we assume that wxWindows
 	# is sufficiently initialized
@@ -189,11 +343,14 @@ Please enter the password for <gm-dbo>:""") % procedure,
 		return None
 
 	# 2) connect as gm-dbo
-	pool = gmPG.ConnectionPool()
-	conn = pool.get_connection_for_user(user='gm-dbo', password=pwd_gm_dbo, extra_verbose=False)
-	if conn is None:
+	login = gmPG2.get_default_login()
+	dsn = gmPG2.make_psycopg2_dsn(database=login.database, host=login.host, port=login.port, user='gm-dbo', password=pwd_gm_dbo)
+	try:
+		conn = gmPG2.get_connection(dsn=dsn, readonly=False, verbose=True)
+	except:
+		_log.LogException('cannot connect')
 		gmGuiHelpers.gm_show_error (
-			aMessage = _('Cannot connect as the GNUmed database user <gm-dbo>.'),
+			aMessage = _('Cannot connect as the GNUmed database owner <gm-dbo>.'),
 			aTitle = procedure,
 			aLogLevel = gmLog.lErr
 		)
@@ -273,7 +430,7 @@ class cTextWidgetValidator(wx.PyValidator):
 
 		if self.__non_empty and val.strip() == '':
 			print self.__msg
-#			gm_beep_statustext(self.__msg)
+#			gm_statustext(self.__msg)
 			ctrl.SetBackgroundColour('pink')
 			ctrl.SetFocus()
 			ctrl.Refresh()
@@ -282,7 +439,7 @@ class cTextWidgetValidator(wx.PyValidator):
 			for char in val:
 				if not char in string.digits:
 					print self.__msg
-#					gm_beep_statustext(self.__msg)
+#					gm_statustext(self.__msg)
 					ctrl.SetBackgroundColour('pink')
 					ctrl.SetFocus()
 					ctrl.Refresh()					
@@ -317,7 +474,7 @@ class cTextWidgetValidator(wx.PyValidator):
 		@param event - The event object containing context information
 		@type event - wx.Event
 		"""
-		key = event.KeyCode()
+		key = event.GetKeyCode()
 		if key < wx.WXK_SPACE or key == wx.WXK_DELETE or key > 255:
 			event.Skip()
 			return
@@ -332,27 +489,44 @@ class cTextWidgetValidator(wx.PyValidator):
 		# before it gets to the text control
 		return
 
-#============================================================
-class cReturnTraversalTextCtrl (wx.TextCtrl):
-	"""
-	Acts exactly like a plain TextCtrl except that RETURN also
-	calls wxWindow.Navigate ()
-
-	FIXME: detect 2.4 and then make self.Navigate a no-op
-	"""
-
-
-	def __init__ (self, *args):
-		wx.TextCtrl.__init__ (self, *args)
-		wx.EVT_TEXT_ENTER (self, self.GetId(), self.__on_enter)
-
-	def __on_enter (self, event):
-		self.Navigate ()
-	
 # ========================================================================
 # $Log: gmGuiHelpers.py,v $
-# Revision 1.36.2.1  2006/09/02 17:59:35  ncq
-# - make return on get_dbowner_connection() more consistent
+# Revision 1.48  2007/01/20 22:52:27  ncq
+# - .KeyCode -> GetKeyCode()
+#
+# Revision 1.47  2007/01/16 13:59:51  ncq
+# - protect against empty trees in expansion history mixin
+#
+# Revision 1.46  2007/01/15 13:04:25  ncq
+# - c3ButtonQuestionDlg
+# - remove cReturnTraversalTextCtrl
+#
+# Revision 1.45  2007/01/13 22:43:41  ncq
+# - remove str() raising Unicode exceptions
+#
+# Revision 1.44  2007/01/13 22:19:37  ncq
+# - cTreeExpansionHistoryMixin
+#
+# Revision 1.43  2007/01/12 13:09:46  ncq
+# - cFileDropTarget
+#
+# Revision 1.42  2006/12/15 15:24:06  ncq
+# - cleanup
+#
+# Revision 1.41  2006/11/24 09:53:24  ncq
+# - gm_beep_statustext() -> gm_statustext(beep=True)
+#
+# Revision 1.40  2006/11/05 14:18:57  ncq
+# - missing "style ="
+#
+# Revision 1.39  2006/10/24 13:23:31  ncq
+# - use gmPG2.get_default_login() in get_dbowner_connection()
+#
+# Revision 1.38  2006/10/08 11:03:09  ncq
+# - convert to gmPG2
+#
+# Revision 1.37  2006/09/03 11:29:30  ncq
+# - add cancel_button argument to show_question
 #
 # Revision 1.36  2006/08/01 22:03:49  ncq
 # - cleanup
