@@ -7,18 +7,14 @@ copyright: authors
 """
 #============================================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/wxpython/gmLoginDialog.py,v $
-# $Id: gmLoginDialog.py,v 1.69.2.1 2006/08/12 12:32:22 ncq Exp $
-__version__ = "$Revision: 1.69.2.1 $"
+# $Id: gmLoginDialog.py,v 1.73 2006/12/21 16:54:32 ncq Exp $
+__version__ = "$Revision: 1.73 $"
 __author__ = "H.Herb, H.Berger, R.Terry, K.Hilbert"
 __license__ = 'GPL (details at http://www.gnu.org)'
 
 import os.path, time, cPickle, zlib, types
 
-try:
-	import wxversion
-	import wx
-except ImportError:
-	from wxPython import wx
+import wx
 
 from Gnumed.pycommon import gmLoginInfo, gmGuiBroker, gmCfg, gmLog, gmI18N, gmNull
 from Gnumed.wxpython import gmGuiHelpers
@@ -81,7 +77,7 @@ class LoginPanel(wx.Panel):
 
 		bitmap = os.path.join (self.gb['resource dir'], 'bitmaps', 'gnumedlogo.png')
 		try:
-			wx.Image_AddHandler(wx.PNGHandler())
+#			wx.Image_AddHandler(wx.PNGHandler())
 			png = wx.Image(bitmap, wx.BITMAP_TYPE_PNG).ConvertToBitmap()
 			bmp = wx.StaticBitmap(self, -1, png, wx.Point(10, 10), wx.Size(png.GetWidth(), png.GetHeight()))
 			self.topsizer.Add(bmp, 0, wx.GROW|wx.ALIGN_CENTER_VERTICAL|wx.ALL, 10)
@@ -341,7 +337,7 @@ class LoginPanel(wx.Panel):
 
 	def GetLoginInfo(self):
 		"""convenience function for compatibility with gmLoginInfo.LoginInfo"""
-		if not self.cancelled:			
+		if not self.cancelled:
 			login = gmLoginInfo.LoginInfo(user=self.GetUser(), passwd=self.GetPassword(), host=self.GetHost())
 			login.SetDatabase(self.GetDatabase())
 			login.SetPort(self.GetPort())
@@ -472,8 +468,17 @@ if __name__ == '__main__':
 
 #############################################################################
 # $Log: gmLoginDialog.py,v $
-# Revision 1.69.2.1  2006/08/12 12:32:22  ncq
-# - handle wxPython (unicode) vs conf file (latin1, often) encoding conversion
+# Revision 1.73  2006/12/21 16:54:32  ncq
+# - inage handlers already inited
+#
+# Revision 1.72  2006/12/15 15:27:28  ncq
+# - cleanup
+#
+# Revision 1.71  2006/10/25 07:21:57  ncq
+# - no more gmPG
+#
+# Revision 1.70  2006/09/01 14:45:03  ncq
+# - assume conf file is latin1 ... FIX later !
 #
 # Revision 1.69  2006/07/30 17:50:03  ncq
 # - properly load bitmaps
