@@ -2,16 +2,15 @@
 """
 #================================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/wxpython/gmMedDocWidgets.py,v $
-# $Id: gmMedDocWidgets.py,v 1.90.2.4 2006/09/19 11:57:38 ncq Exp $
-__version__ = "$Revision: 1.90.2.4 $"
+# $Id: gmMedDocWidgets.py,v 1.110 2007/01/18 22:13:37 ncq Exp $
+__version__ = "$Revision: 1.110 $"
 __author__ = "Karsten Hilbert <Karsten.Hilbert@gmx.net>"
 
-import os.path, sys, re, time
+import os.path, sys, re as regex
 
-import wxversion
 import wx
 
-from Gnumed.pycommon import gmLog, gmI18N, gmCfg, gmPG, gmMimeLib, gmExceptions, gmMatchProvider, gmDispatcher, gmSignals, gmFuzzyTimestamp
+from Gnumed.pycommon import gmLog, gmI18N, gmCfg, gmPG2, gmMimeLib, gmExceptions, gmMatchProvider, gmDispatcher, gmSignals, gmFuzzyTimestamp
 from Gnumed.business import gmPerson, gmMedDoc
 from Gnumed.wxpython import gmGuiHelpers, gmRegetMixin, gmPhraseWheel, gmPlugin
 from Gnumed.wxGladeWidgets import wxgScanIdxPnl, wxgReviewDocPartDlg, wxgSelectablySortedDocTreePnl, wxgEditDocumentTypesPnl, wxgEditDocumentTypesDlg
@@ -19,6 +18,53 @@ from Gnumed.wxGladeWidgets import wxgScanIdxPnl, wxgReviewDocPartDlg, wxgSelecta
 _log = gmLog.gmDefLog
 _log.Log(gmLog.lInfo, __version__)
 
+#============================================================
+class cDocumentCommentPhraseWheel(gmPhraseWheel.cPhraseWheel):
+	"""Let user select a document comment from all existing comments."""
+	def __init__(self, *args, **kwargs):
+
+		context = {
+			u'ctxt_doc_type': {
+				u'where_part': u'and fk_type = %(pk_doc_type)s',
+				u'placeholder': u'pk_doc_type'
+			}
+		}
+
+		mp = gmMatchProvider.cMatchProvider_SQL2 (
+			queries = [u"""
+select *
+from (
+	select distinct on (comment) *
+	from (
+		-- keyed by doc type
+		select comment, comment as pk, 1 as rank
+		from blobs.doc_med
+		where
+			comment %(fragment_condition)s
+			%(ctxt_doc_type)s
+
+		union all
+
+		select comment, comment as pk, 2 as rank
+		from blobs.doc_med
+		where comment %(fragment_condition)s
+	) as q_union
+) as q_distinct
+order by rank, comment
+limit 25"""],
+			context = context
+		)
+		mp.setThresholds(3, 5, 7)
+		mp.unset_context(u'pk_doc_type')
+
+		kwargs['aMatchProvider'] = mp
+		kwargs['aDelay'] = 50
+		kwargs['selection_only'] = False
+		gmPhraseWheel.cPhraseWheel.__init__ (
+			self,
+			*args,
+			**kwargs
+		)
 #============================================================
 class cEditDocumentTypesDlg(wxgEditDocumentTypesDlg.wxgEditDocumentTypesDlg):
 	"""A dialog showing a cEditDocumentTypesPnl."""
@@ -40,91 +86,106 @@ class cEditDocumentTypesPnl(wxgEditDocumentTypesPnl.wxgEditDocumentTypesPnl):
 		self._LCTRL_doc_type.InsertColumn(0, _('type'))
 		self._LCTRL_doc_type.InsertColumn(1, _('description'))
 		self._LCTRL_doc_type.InsertColumn(2, _('user defined'))
+		self._LCTRL_doc_type.InsertColumn(3, _('in use'))
 	#--------------------------------------------------------
 	def repopulate_ui(self):
 
-		self.doc_types = gmMedDoc.get_document_types()
-		pos = len(self.doc_types) + 1
+		doc_types = gmMedDoc.get_document_types()
+		pos = len(doc_types) + 1
 		self._LCTRL_doc_type.DeleteAllItems()
 
-		for doc_type in self.doc_types:
+		for doc_type in doc_types:
 			row_num = self._LCTRL_doc_type.InsertStringItem(pos, label = doc_type['type'])
 			self._LCTRL_doc_type.SetStringItem(index = row_num, col = 1, label = doc_type['l10n_type'])
-			if doc_type['is_user']:
-				self._LCTRL_doc_type.SetStringItem(index = row_num, col = 2, label = 'X')
-			else:
-				self._LCTRL_doc_type.SetItemTextColour(row_num, col=wx.LIGHT_GREY)
+			if doc_type['is_user_defined']:
+				self._LCTRL_doc_type.SetStringItem(index = row_num, col = 2, label = ' X ')
+			if doc_type['is_in_use']:
+				self._LCTRL_doc_type.SetStringItem(index = row_num, col = 3, label = ' X ')
 
-		if len(self.doc_types) > 0:
+		if len(doc_types) > 0:
+			self._LCTRL_doc_type.set_data(data = doc_types)
 			self._LCTRL_doc_type.SetColumnWidth(col=0, width=wx.LIST_AUTOSIZE)
 			self._LCTRL_doc_type.SetColumnWidth(col=1, width=wx.LIST_AUTOSIZE)
 			self._LCTRL_doc_type.SetColumnWidth(col=2, width=wx.LIST_AUTOSIZE_USEHEADER)
+			self._LCTRL_doc_type.SetColumnWidth(col=3, width=wx.LIST_AUTOSIZE_USEHEADER)
 
-		self._BTN_rename.Enable(False)
-		self._BTN_add.Enable(False)
+		self._BTN_set_translation.Enable(False)
 		self._BTN_delete.Enable(False)
 
 		self._TCTRL_type.SetValue('')
-		self._ChBOX_is_user.SetValue(False)
-		self._TCTRL_description.SetValue('')
+		self._TCTRL_l10n_type.SetValue('')
 
 		self._LCTRL_doc_type.SetFocus()
 	#--------------------------------------------------------
 	# event handlers
 	#--------------------------------------------------------
 	def _on_list_item_selected(self, evt):
-		doc_type = self.doc_types[self._LCTRL_doc_type.GetFirstSelected()]
+		doc_type = self._LCTRL_doc_type.get_selected_item_data()
 
 		self._TCTRL_type.SetValue(doc_type['type'])
-		self._TCTRL_type.SetEditable(bool(doc_type['is_user']))
-		self._ChBOX_is_user.SetValue(bool(doc_type['is_user']))
-		self._TCTRL_description.SetValue(doc_type['l10n_type'])
+		self._TCTRL_l10n_type.SetValue(doc_type['l10n_type'])
 
-		self._BTN_rename.Enable(bool(doc_type['is_user']))
-		self._BTN_delete.Enable(bool(doc_type['is_user']))
-		self._BTN_add.Enable(True)
+		self._BTN_set_translation.Enable(True)
+		self._BTN_delete.Enable(not bool(doc_type['is_in_use']))
+		self._BTN_add.Enable(False)
 		return
 	#--------------------------------------------------------
-	def _on_rename_button_pressed(self, event):
-		doc_type = self.doc_types[self._LCTRL_doc_type.GetFirstSelected()]
-		doc_type['type'] = self._TCTRL_type.GetValue().strip()
-		doc_type.save_payload()			# FIXME: error handling ?
-		doc_type.set_translation(translation = self._TCTRL_description.GetValue().strip())
+	def _on_set_translation_button_pressed(self, event):
+		doc_type = self._LCTRL_doc_type.get_selected_item_data()
+		doc_type.set_translation(translation = self._TCTRL_l10n_type.GetValue().strip())
 
 		self.repopulate_ui()
 		return
 	#--------------------------------------------------------
 	def _on_delete_button_pressed(self, event):
-		doc_type = self.doc_types[self._LCTRL_doc_type.GetFirstSelected()]
-		gmMedDoc.delete_document_type(document_type = doc_type)			# FIXME: error handling
+		doc_type = self._LCTRL_doc_type.get_selected_item_data()
+		if doc_type['is_in_use']:
+			gmGuiHelpers.gm_show_info (
+				_(
+					'Cannot delete document type\n'
+					' [%s]\n'
+					'because it is currently in use.'
+				) % doc_type['l10n_type'],
+				_('deleting document type')
+			)
+			return
+
+		gmMedDoc.delete_document_type(document_type = doc_type)
 
 		self.repopulate_ui()
 		return
 	#--------------------------------------------------------
 	def _on_add_button_pressed(self, event):
-		if self._TCTRL_type.IsEditable() and self._TCTRL_type.IsModified():
-			doc_type = gmMedDoc.create_document_type(document_type = self._TCTRL_type.GetValue().strip())
-			doc_type.set_translation(translation = self._TCTRL_description.GetValue().strip())
-		else:
-			gmMedDoc.create_document_type(document_type = self._TCTRL_description.GetValue().strip())
+		desc = self._TCTRL_type.GetValue().strip()
+		if desc != '':
+			doc_type = gmMedDoc.create_document_type(document_type = desc)		# does not create dupes
+			l10n_desc = self._TCTRL_l10n_type.GetValue().strip()
+			if (l10n_desc != '') and (l10n_desc != doc_type['l10n_type']):
+				doc_type.set_translation(translation = l10n_desc)
 
 		self.repopulate_ui()
 		return
+	#--------------------------------------------------------
+	def _on_type_modified(self, event):
+		self._BTN_set_translation.Enable(False)
+		self._BTN_delete.Enable(False)
+		self._BTN_add.Enable(True)
+#		self._LCTRL_doc_type.deselect_selected_item()
+		return
 #============================================================
 class cDocumentTypeSelectionPhraseWheel(gmPhraseWheel.cPhraseWheel):
 	"""Let user select a document type."""
 	def __init__(self, *args, **kwargs):
 
 		mp = gmMatchProvider.cMatchProvider_SQL2 (
-			service = 'blobs',
 			queries = [
-"""select * from ((
+u"""select * from ((
 	select pk_doc_type, l10n_type, 1 as rank from blobs.v_doc_type where
-		is_user is True and
+		is_user_defined is True and
 		l10n_type %(fragment_condition)s
 ) union (
 	select pk_doc_type, l10n_type, 2 from blobs.v_doc_type where
-		is_user is False and
+		is_user_defined is False and
 		l10n_type %(fragment_condition)s
 )) as q1 order by q1.rank, q1.l10n_type
 """]
@@ -159,7 +220,11 @@ class cReviewDocPartDlg(wxgReviewDocPartDlg.wxgReviewDocPartDlg):
 		# associated episode (add " " to avoid popping up pick list)
 		self._PhWheel_episode.SetValue('%s ' % self.__part['episode'], self.__part['pk_episode'])
 		self._PhWheel_doc_type.SetValue(value = self.__part['l10n_type'], data = self.__part['pk_type'])
-		self._TCTRL_doc_comment.SetValue(self.__part['doc_comment'])
+		self._PhWheel_doc_type.add_callback_on_lose_focus(self._on_doc_type_loses_focus)
+
+		self._PRW_doc_comment.SetValue(self.__part['doc_comment'])
+		self._PRW_doc_comment.set_context(context = 'pk_doc_type', val = self.__part['pk_type'])
+
 		fts = gmFuzzyTimestamp.cFuzzyTimestamp(timestamp = self.__part['date_generated'])
 		self._PhWheel_doc_date.SetValue(fts.strftime('%Y-%m-%d'), fts)
 		if self.__part['ext_ref'] is not None:
@@ -191,7 +256,7 @@ class cReviewDocPartDlg(wxgReviewDocPartDlg.wxgReviewDocPartDlg):
 		if self.__part['reviewed_by_you']:
 			revs = self.__part.get_reviews()
 			for rev in revs:
-				if rev[5]:
+				if rev['is_your_review']:
 					self._ChBOX_abnormal.SetValue(bool(rev[2]))
 					self._ChBOX_relevant.SetValue(bool(rev[3]))
 					break
@@ -207,30 +272,30 @@ class cReviewDocPartDlg(wxgReviewDocPartDlg.wxgReviewDocPartDlg):
 		review_by_responsible_doc = None
 		reviews_by_others = []
 		for rev in revs:
-			if rev[4] and not rev[5]:
+			if rev['is_review_by_responsible_reviewer'] and not rev['is_your_review']:
 				review_by_responsible_doc = rev
-			if not (rev[4] or rev[5]):
+			if not (rev['is_review_by_responsible_reviewer'] or rev['is_your_review']):
 				reviews_by_others.append(rev)
 		# display them
 		if review_by_responsible_doc is not None:
 			row_num = self._LCTRL_existing_reviews.InsertStringItem(sys.maxint, label=review_by_responsible_doc[0])
 			self._LCTRL_existing_reviews.SetItemTextColour(row_num, col=wx.BLUE)
 			self._LCTRL_existing_reviews.SetStringItem(index = row_num, col=0, label=review_by_responsible_doc[0])
-			self._LCTRL_existing_reviews.SetStringItem(index = row_num, col=1, label=review_by_responsible_doc[1].Format('%Y-%m-%d %H:%M'))
-			if review_by_responsible_doc[2]:
-				self._LCTRL_existing_reviews.SetStringItem(index = row_num, col=2, label='X')
-			if review_by_responsible_doc[3]:
-				self._LCTRL_existing_reviews.SetStringItem(index = row_num, col=3, label='X')
+			self._LCTRL_existing_reviews.SetStringItem(index = row_num, col=1, label=review_by_responsible_doc[1].strftime('%Y-%m-%d %H:%M'))
+			if review_by_responsible_doc['is_technically_abnormal']:
+				self._LCTRL_existing_reviews.SetStringItem(index = row_num, col=2, label=u'X')
+			if review_by_responsible_doc['clinically_relevant']:
+				self._LCTRL_existing_reviews.SetStringItem(index = row_num, col=3, label=u'X')
 			self._LCTRL_existing_reviews.SetStringItem(index = row_num, col=4, label=review_by_responsible_doc[6])
 			row_num += 1
 		for rev in reviews_by_others:
 			row_num = self._LCTRL_existing_reviews.InsertStringItem(sys.maxint, label=rev[0])
 			self._LCTRL_existing_reviews.SetStringItem(index = row_num, col=0, label=rev[0])
-			self._LCTRL_existing_reviews.SetStringItem(index = row_num, col=1, label=rev[1].Format('%Y-%m-%d %H:%M'))
-			if rev[2]:
-				self._LCTRL_existing_reviews.SetStringItem(index = row_num, col=2, label='X')
-			if rev[3]:
-				self._LCTRL_existing_reviews.SetStringItem(index = row_num, col=3, label='X')
+			self._LCTRL_existing_reviews.SetStringItem(index = row_num, col=1, label=rev[1].strftime('%Y-%m-%d %H:%M'))
+			if rev['is_technically_abnormal']:
+				self._LCTRL_existing_reviews.SetStringItem(index = row_num, col=2, label=u'X')
+			if rev['clinically_relevant']:
+				self._LCTRL_existing_reviews.SetStringItem(index = row_num, col=3, label=u'X')
 			self._LCTRL_existing_reviews.SetStringItem(index = row_num, col=4, label=rev[6])
 		return True
 	#--------------------------------------------------------
@@ -252,27 +317,28 @@ class cReviewDocPartDlg(wxgReviewDocPartDlg.wxgReviewDocPartDlg):
 				_('editing document properties')
 			)
 			return False
-		if pk_episode != doc['pk_episode']:
-			# since the phrasewheel operates on the active
-			# patient all episodes really should belong
-			# to it so we don't check patient change
-			doc['pk_episode'] = pk_episode
 
 		doc_type = self._PhWheel_doc_type.GetData(can_create = True)
-		if doc_type is not None:
-			if doc_type != doc['pk_type']:
-				doc['pk_type'] = doc_type
-		else:
-			gmGuiHelpers.gm_beep_statustext(_('Cannot change document type to [%s].') % self._PhWheel_doc_type.GetValue().strip())
+		if doc_type is None:
+			gmGuiHelpers.gm_statustext(_('Cannot change document type to [%s].') % self._PhWheel_doc_type.GetValue().strip(), beep=True)
+			return False
 
-		if self._TCTRL_doc_comment.IsModified():
-			doc['comment'] = self._TCTRL_doc_comment.GetValue().strip()
+		# since the phrasewheel operates on the active
+		# patient all episodes really should belong
+		# to it so we don't check patient change
+		doc['pk_episode'] = pk_episode
+		doc['pk_type'] = doc_type
+		doc['comment'] = self._PRW_doc_comment.GetValue().strip()
+		doc['date'] = self._PhWheel_doc_date.GetData().get_pydt()
+		doc['ext_ref'] = self._TCTRL_reference.GetValue().strip()
 
-		if self._PhWheel_doc_date.IsModified():
-			doc['date'] = self._PhWheel_doc_date.GetData().timestamp
-
-		if self._TCTRL_reference.IsModified():
-			doc['ext_ref'] = self._TCTRL_reference.GetValue().strip()
+		success, data = doc.save_payload()
+		if not success:
+			gmGuiHelpers.gm_show_error (
+				_('Cannot link the document to episode\n\n [%s]') % epi_name,
+				_('editing document properties')
+			)
+			return False
 
 		# 2) handle review
 		if self._ChBOX_review.GetValue():
@@ -304,14 +370,6 @@ class cReviewDocPartDlg(wxgReviewDocPartDlg.wxgReviewDocPartDlg):
 			)
 			return False
 
-		success, data = doc.save_payload()
-		if not success:
-			gmGuiHelpers.gm_show_error (
-				_('Cannot link the document to episode\n\n [%s]') % epi_name,
-				_('editing document properties')
-			)
-			return False
-
 		return True
 	#--------------------------------------------------------
 	def _on_reviewed_box_checked(self, evt):
@@ -322,33 +380,60 @@ class cReviewDocPartDlg(wxgReviewDocPartDlg.wxgReviewDocPartDlg):
 		# NOTE: for now *always* apply to all pages so we can
 		# NOTE: offer review from document level as well
 		#self._ChBOX_sign_all_pages.Enable(enable = state)
+	#--------------------------------------------------------
+	def _on_doc_type_loses_focus(self):
+		pk_doc_type = self._PhWheel_doc_type.GetData()
+		if pk_doc_type is None:
+			self._PRW_doc_comment.unset_context(context = 'pk_doc_type')
+		else:
+			self._PRW_doc_comment.set_context(context = 'pk_doc_type', val = pk_doc_type)
+		return True
 #============================================================
+# FIXME: this must listen to patient change signals ...
 class cScanIdxDocsPnl(wxgScanIdxPnl.wxgScanIdxPnl, gmPlugin.cPatientChange_PluginMixin):
 	def __init__(self, *args, **kwds):
 		wxgScanIdxPnl.wxgScanIdxPnl.__init__(self, *args, **kwds)
 		gmPlugin.cPatientChange_PluginMixin.__init__(self)
 
-		mp = gmMatchProvider.cMatchProvider_SQL2 (
-			service = 'personalia',
-			queries = ["""
-select
-	pk_staff,
-	short_alias || ' (' || title || firstnames || ' ' || lastnames || ')',
-	1
-from dem.v_staff
-where
-	short_alias || ' ' || firstnames || ' ' || lastnames || ' ' || db_user %(fragment_condition)s"""]
-		)
-		mp.setThresholds(1, 2, 3)
-		self._PhWheel_reviewer.setMatchProvider(mp = mp)
+		self._PhWheel_reviewer.setMatchProvider(mp = gmPerson.cMatchProvider_Provider())
 
 		self.__init_ui_data()
+		self._PhWheel_doc_type.add_callback_on_lose_focus(self._on_doc_type_loses_focus)
+
+		# make me a file drop target
+		dt = gmGuiHelpers.cFileDropTarget(self)
+		self.SetDropTarget(dt)
 
 		# do not import globally since we might want to use
 		# this module without requiring any scanner to be available
 		from Gnumed.pycommon import gmScanBackend
 		self.scan_module = gmScanBackend
 	#--------------------------------------------------------
+	# file drop target API
+	#--------------------------------------------------------
+	def add_filenames(self, filenames):
+		pat = gmPerson.gmCurrentPatient()
+		if not pat.is_connected():
+			gmGuiHelpers.gm_statustext(_('Cannot accept new documents. No active patient.'))
+			return
+
+		# dive into folders dropped onto us and extract files (one level deep only)
+		real_filenames = []
+		for pathname in filenames:
+			try:
+				files = os.listdir(pathname)
+				gmGuiHelpers.gm_statustext(_('Extracting files from folder [%s] ...') % pathname)
+				for file in files:
+					fullname = os.path.join(pathname, file)
+					if not os.path.isfile(fullname):
+						continue
+					real_filenames.append(fullname)
+			except OSError:
+				real_filenames.append(pathname)
+
+		self.acquired_pages.extend(real_filenames)
+		self.__reload_LBOX_doc_pages()
+	#--------------------------------------------------------
 	def repopulate_ui(self):
 		pass
 	#--------------------------------------------------------
@@ -371,12 +456,12 @@ where
 		# FIXME: make this configurable: either now() or last_date()
 		fts = gmFuzzyTimestamp.cFuzzyTimestamp()
 		self._PhWheel_doc_date.SetValue(fts.strftime('%Y-%m-%d'), fts)
-		self._TBOX_doc_comment.SetValue('')
+		self._PRW_doc_comment.SetValue('')
 		# FIXME: should be set to patient's primary doc
 		self._PhWheel_reviewer.selection_only = True
 		me = gmPerson.gmCurrentProvider()
 		self._PhWheel_reviewer.SetValue (
-			value = '%s (%s%s %s)' % (me['short_alias'], me['title'], me['firstnames'], me['lastnames']),
+			value = u'%s (%s%s %s)' % (me['short_alias'], me['title'], me['firstnames'], me['lastnames']),
 			data = me['pk_staff']
 		)
 		# -----------------------------
@@ -418,7 +503,7 @@ where
 			)
 			return False
 
-		if self._TBOX_doc_comment.GetValue().strip() == '':
+		if self._PRW_doc_comment.GetValue().strip() == '':
 			gmGuiHelpers.gm_show_error (
 				aMessage = _('No document comment supplied. Add a comment for this document.'),
 				aTitle = title
@@ -447,7 +532,7 @@ where
 		devices = self.scan_module.get_devices()
 
 		if devices is False:
-			gmGuiHelpers.gm_beep_statustext (
+			gmGuiHelpers.gm_statustext (
 				_('There is no scanner support installed on this machine.'),
 				gmLog.lWarn
 			)
@@ -458,7 +543,7 @@ where
 			chosen_device = None
 		else:
 			if len(devices) == 0:
-				gmGuiHelpers.gm_beep_statustext (
+				gmGuiHelpers.gm_statustext (
 					_('Cannot find an active scanner.'),
 					gmLog.lWarn
 				)
@@ -476,21 +561,31 @@ where
 				return None
 			chosen_device = devices[device_idx][0]
 
-		# FIXME: load directory from backend config
-		fname = self.scan_module.acquire_page_into_file (
+		tmpdir = os.path.expanduser(os.path.join('~', 'gnumed', 'tmp'))
+		if not os.path.isdir(tmpdir):
+			try:
+				os.makedirs(tmpdir)
+			except:
+				tmpdir = None
+		fnames = self.scan_module.acquire_pages_into_files (
 			device = chosen_device,
 			delay = 5,
+			tmpdir = tmpdir,
 			calling_window = self
 		)
-		if fname is False:
+		if fnames is False:
 			gmGuiHelpers.gm_show_error (
 				aMessage = _('Page could not be acquired from source.'),
 				aTitle = _('acquiring page')
 			)
 			return None
-		self.acquired_pages.append(fname)
+		if len(fnames) == 0:		# no pages scanned
+			return True
+		# FIXME: configure whether to use XSane or sane directly
+		self.acquired_pages.extend(fnames)
 		# update list of pages in GUI
 		self.__reload_LBOX_doc_pages()
+		return True
 	#--------------------------------------------------------
 	def _load_btn_pressed(self, evt):
 		# patient file chooser
@@ -523,7 +618,7 @@ where
 		page_fname = self._LBOX_doc_pages.GetClientData(page_idx)
 		(result, msg) = gmMimeLib.call_viewer_on_file(page_fname)
 		if not result:
-			gmGuiHelpers.gm_show_error (
+			gmGuiHelpers.gm_show_warning (
 				aMessage = _('Cannot display document part:\n%s') % msg,
 				aTitle = _('displaying page')
 			)
@@ -615,13 +710,13 @@ from your computer.""") % page_fname,
 
 		# update business object with metadata
 		# - date of generation
-		new_doc['date'] = self._PhWheel_doc_date.GetData().timestamp
+		new_doc['date'] = self._PhWheel_doc_date.GetData().get_pydt()
 		# - external reference
 		ref = gmMedDoc.get_ext_ref()
 		if ref is not None:
 			new_doc['ext_ref'] = ref
 		# - comment
-		comment = self._TBOX_doc_comment.GetLineText(0).strip()
+		comment = self._PRW_doc_comment.GetLineText(0).strip()
 		if comment != '':
 			new_doc['comment'] = comment
 		# - save it
@@ -688,7 +783,7 @@ off this message in the GNUmed configuration.""") % ref
 
 		# prepare for next document
 		self.__init_ui_data()
-		gmGuiHelpers.gm_beep_statustext(_('Successfully saved new document.'))
+		gmGuiHelpers.gm_statustext(_('Successfully saved new document.'))
 		return True
 	#--------------------------------------------------------
 	def _startover_btn_pressed(self, evt):
@@ -697,6 +792,14 @@ off this message in the GNUmed configuration.""") % ref
 	def _reviewed_box_checked(self, evt):
 		self._ChBOX_abnormal.Enable(enable = self._ChBOX_reviewed.GetValue())
 		self._ChBOX_relevant.Enable(enable = self._ChBOX_reviewed.GetValue())
+	#--------------------------------------------------------
+	def _on_doc_type_loses_focus(self):
+		pk_doc_type = self._PhWheel_doc_type.GetData()
+		if pk_doc_type is None:
+			self._PRW_doc_comment.unset_context(context = 'pk_doc_type')
+		else:
+			self._PRW_doc_comment.set_context(context = 'pk_doc_type', val = pk_doc_type)
+		return True
 #============================================================
 class cSelectablySortedDocTreePnl(wxgSelectablySortedDocTreePnl.wxgSelectablySortedDocTreePnl, gmRegetMixin.cRegetOnPaintMixin):
 	"""A document tree that can be sorted."""
@@ -775,7 +878,7 @@ class cDocTree(wx.TreeCtrl):
 	#--------------------------------------------------------
 	def refresh(self):
 		if not self.__pat.is_connected():
-			gmGuiHelpers.gm_beep_statustext (
+			gmGuiHelpers.gm_statustext (
 				_('Cannot load documents. No active patient.'),
 				gmLog.lErr
 			)
@@ -863,7 +966,7 @@ class cDocTree(wx.TreeCtrl):
 
 			label = _('%s%7s %s: %s (%s page(s), %s)') % (
 				review,
-				doc['date'].Format('%m/%Y'),
+				doc['date'].strftime('%m/%Y'),
 				doc['l10n_type'][:26],
 				cmt,
 				page_num,
@@ -945,9 +1048,8 @@ class cDocTree(wx.TreeCtrl):
 		# doc node
 		if isinstance(item1, gmMedDoc.cMedDoc):
 
-			# FIXME: should be "date" but have to wait until fuzzy date problem solved
-			#date_field = 'date'
-			date_field = 'modified_when'
+			date_field = 'date'
+			#date_field = 'modified_when'
 
 			if self.__sort_mode == 'age':
 				# reverse sort by date
@@ -1064,27 +1166,37 @@ class cDocTree(wx.TreeCtrl):
 	# internal API
 	#--------------------------------------------------------
 	def __handle_doc_context(self):
-		# build menu
+
+		menu = wx.Menu(title = _('document menu'))
+
+		# edit metadata
+		ID = wx.NewId()
+		menu.AppendItem(wx.MenuItem(menu, ID, _('Review/Edit properties')))
+		wx.EVT_MENU(menu, ID, self.__review_curr_part)
+
+		# export pages
+		ID = wx.NewId()
+		menu.AppendItem(wx.MenuItem(menu, ID, _('Export to disk')))
+		wx.EVT_MENU(menu, ID, self.__export_doc_to_disk)
+
+		# FIXME: add item "reorder pages"
+
+		# show descriptions
 		descriptions = self.__curr_node_data.get_descriptions()
 		desc_menu = wx.Menu()
 		for desc in descriptions:
 			d_id = wx.NewId()
 			# contract string
-			tmp = re.split('\r\n+|\r+|\n+|\s+|\t+', desc)
+			tmp = regex.split('\r\n+|\r+|\n+|\s+|\t+', desc)
 			tmp = ' '.join(tmp)
 			# but only use first 30 characters
 			tmp = "%s ..." % tmp[:30]
 			desc_menu.AppendItem(wx.MenuItem(desc_menu, d_id, tmp))
 			# connect handler
 			wx.EVT_MENU(desc_menu, d_id, self.__show_description)
-		ID_load_submenu = wx.NewId()
-		menu = wx.Menu(title = _('document menu'))
-		# FIXME: add item "reorder pages"
-		menu.AppendMenu(ID_load_submenu, _('descriptions ...'), desc_menu)
-		# edit metadata
-		ID = wx.NewId()
-		menu.AppendItem(wx.MenuItem(menu, ID, _('Review/Edit properties')))
-		wx.EVT_MENU(menu, ID, self.__review_curr_part)
+
+		menu.AppendMenu(wx.NewId(), _('descriptions ...'), desc_menu)
+
 		# show menu
 		self.PopupMenu(menu, wx.DefaultPosition)
 		menu.Destroy()
@@ -1113,35 +1225,40 @@ class cDocTree(wx.TreeCtrl):
 		# sanity check
 		if part['size'] == 0:
 			_log.Log(gmLog.lErr, 'cannot display part [%s] - 0 bytes' % part['pk_obj'])
-			gmGuiHelpers.gm_show_error(
+			gmGuiHelpers.gm_show_error (
 				aMessage = _('Document part does not seem to exist in database !'),
 				aTitle = _('showing document')
 			)
 			return None
 
 		# get export directory for temporary files
-		def_tmp_dir = os.path.join('~', 'gnumed', 'tmp')
+		def_tmp_dir = os.path.expanduser(os.path.join('~', 'gnumed', 'tmp'))
+		if not os.path.isdir(def_tmp_dir):
+			try:
+				os.makedirs(def_tmp_dir)
+			except:
+				def_tmp_dir = None
 		cfg = gmCfg.cCfgSQL()
-		tmp_dir = cfg.get_by_workplace (
+		tmp_dir = cfg.get2 (
 			option = "horstspace.tmp_dir",
 			workplace = gmPerson.gmCurrentProvider().get_workplace(),
+			bias = 'workplace',
 			default = def_tmp_dir
 		)
 		exp_base = os.path.abspath(os.path.expanduser(os.path.join(tmp_dir, 'docs')))
-		if not os.path.exists(exp_base):
-			_log.Log(gmLog.lErr, "The directory [%s] does not exist ! Falling back to default temporary directory." % exp_base) # which is None == tempfile.tempdir == use system defaults
+		if not os.path.isdir(exp_base):
+			_log.Log(gmLog.lWarn, "The directory [%s] does not exist ! Falling back to default temporary directory." % exp_base) # which is None == tempfile.tempdir == use system defaults
 			exp_base = None
 		else:
 			_log.Log(gmLog.lData, "working into directory [%s]" % exp_base)
 
 		# determine database export chunk size
-		chunksize = int(cfg.get_by_workplace (
+		chunksize = int(cfg.get2 (
 			option = "horstspace.blob_export_chunk_size",
 			workplace = gmPerson.gmCurrentProvider().get_workplace(),
+			bias = 'workplace',
 			default = 1 * 1024 * 1024		# 1 MB
 		))
-		if chunksize is None:
-			chunksize = 1 * 1024 * 1024		# 1 MB
 
 		# retrieve doc part
 		fname = part.export_to_file(aTempDir = exp_base, aChunkSize = chunksize)
@@ -1154,7 +1271,13 @@ class cDocTree(wx.TreeCtrl):
 			return None
 
 		# display it
-		(result, msg) = gmMimeLib.call_viewer_on_file(fname)
+		block_during_view = bool( cfg.get2 (
+			option = 'horstspace.document_viewer.block_during_view',
+			workplace = gmPerson.gmCurrentProvider().get_workplace(),
+			bias = 'user',
+			default = None
+		))
+		(result, msg) = gmMimeLib.call_viewer_on_file(fname, block=block_during_view)
 		if not result:
 			gmGuiHelpers.gm_show_error (
 				aMessage = _('Cannot display document part:\n%s') % msg,
@@ -1198,6 +1321,43 @@ class cDocTree(wx.TreeCtrl):
 	#--------------------------------------------------------
 	def __show_description(self, evt):
 		print "showing description"
+	#--------------------------------------------------------
+	# document level context menu handlers
+	#--------------------------------------------------------
+	def __export_doc_to_disk(self, evt):
+		"""Export document into directory.
+
+		- one file per object
+		- into subdirectory named after patient
+		"""
+		pat = gmPerson.gmCurrentPatient()
+		ident = pat.get_identity()
+		# FIXME: make configurable
+		dname = '%s_%s-%s' % (ident['lastnames'], ident['firstnames'], ident['dob'].strftime('%Y-%m-%d'))
+		# FIXME: make configurable
+		def_dir = os.path.abspath(os.path.expanduser(os.path.join('~', 'gnumed', 'export', 'docs', dname)))
+		if not os.access(def_dir, os.F_OK):
+			os.makedirs(def_dir)
+		
+		dlg = wx.DirDialog (
+			parent = self,
+			message = _('Save document into directory ...'),
+			defaultPath = def_dir,
+			style = wx.DD_DEFAULT_STYLE
+		)
+		if dlg.ShowModal() != wx.ID_OK:
+			dlg.Destroy()
+			return True
+		dirname = dlg.GetPath()
+		dlg.Destroy()
+
+		wx.BeginBusyCursor()
+		fnames = self.__curr_node_data.export_parts_to_files(export_dir=dirname)
+		wx.EndBusyCursor()
+
+		gmGuiHelpers.gm_statustext(_('Successfully exported %s pages into the directory [%s].') % (len(fnames), dirname), beep=False)
+
+		return True
 #============================================================
 # main
 #------------------------------------------------------------
@@ -1209,17 +1369,72 @@ if __name__ == '__main__':
 
 #============================================================
 # $Log: gmMedDocWidgets.py,v $
-# Revision 1.90.2.4  2006/09/19 11:57:38  ncq
+# Revision 1.110  2007/01/18 22:13:37  ncq
+# - tell user when we expand a folder to extract files
+#
+# Revision 1.109  2007/01/17 14:01:56  ncq
+# - when folder dropped onto scanidxpnl extract files one level into it
+#
+# Revision 1.108  2007/01/12 13:10:11  ncq
+# - use cFileDropTarget in ScanIdxPnl
+#
+# Revision 1.107  2007/01/10 23:01:07  ncq
+# - properly update document/object metadata
+#
+# Revision 1.106  2007/01/07 23:08:52  ncq
+# - improve cDocumentCommentPhraseWheel query and link it to the doc_type field
+# - add "export to disk" to doc tree context menu
+#
+# Revision 1.105  2007/01/06 23:42:35  ncq
+# - cDocumentCommentPhraseWeel and adjust to wxGlade based uses thereof
+#
+# Revision 1.104  2006/12/27 16:45:42  ncq
+# - adjust to acquire_pages_into_files() returning a list
+#
+# Revision 1.103  2006/12/21 10:55:09  ncq
+# - fix inverted is_in_use logic on enabling delete button
+#
+# Revision 1.102  2006/12/13 22:32:17  ncq
+# - need to explicitely check for "is_user_defined is True/False"
+#
+# Revision 1.101  2006/12/13 20:55:22  ncq
+# - is_user -> is_user_defined
+#
+# Revision 1.100  2006/12/11 21:40:12  ncq
+# - support in_use in doc type list ctrl
+# - slight improvement of doc type edit dialog logic
+#
+# Revision 1.99  2006/11/24 10:01:31  ncq
+# - gm_beep_statustext() -> gm_statustext()
+#
+# Revision 1.98  2006/11/06 10:01:17  ncq
+# - handle _on_description_modified() in edit-doc-types
+#
+# Revision 1.97  2006/10/31 17:22:49  ncq
+# - unicode()ify queries
+# - cleanup
+# - PgResult is now dict, so use it instead of index
+# - add missing os.path.expanduser()
+#
+# Revision 1.96  2006/10/25 07:46:44  ncq
+# - Format() -> strftime() since datetime.datetime does not have .Format()
+#
+# Revision 1.95  2006/10/24 13:26:11  ncq
+# - no more gmPG.
+# - use cMatchProvider_Provider() in scan/index panel
+#
+# Revision 1.94  2006/10/08 11:05:32  ncq
+# - properly use db cfg
+#
+# Revision 1.93  2006/09/19 12:00:42  ncq
 # - clear scan/idx panel on patient change
 #
-# Revision 1.90.2.3  2006/08/28 21:29:28  ncq
-# - properly detect failure of acquire_page_from_file()
+# Revision 1.92  2006/09/12 17:27:35  ncq
+# - support horstspace.document_viewer.block_during_view
 #
-# Revision 1.90.2.2  2006/08/28 20:49:20  ncq
-# - let scan backend choose it's own filename
-#
-# Revision 1.90.2.1  2006/08/14 18:39:26  ncq
-# - fix TWAIN failure on pressing scan button
+# Revision 1.91  2006/09/01 15:03:26  ncq
+# - improve scanner device choice handling
+# - better tmp dir handling on document import/export
 #
 # Revision 1.90  2006/07/24 20:51:26  ncq
 # - get_by_user() -> get2()
