@@ -10,19 +10,19 @@ generator.
 """
 #============================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/wxpython/gmPatSearchWidgets.py,v $
-# $Id: gmPatSearchWidgets.py,v 1.41.2.1 2006/09/13 07:57:55 ncq Exp $
-__version__ = "$Revision: 1.41.2.1 $"
+# $Id: gmPatSearchWidgets.py,v 1.59 2007/01/20 22:52:27 ncq Exp $
+__version__ = "$Revision: 1.59 $"
 __author__ = "K.Hilbert <Karsten.Hilbert@gmx.net>"
 __license__ = 'GPL (for details see http://www.gnu.org/)'
 
-import sys, os.path, time, string, re
+import sys, os.path, time, string, re, glob
 
 import wx
 
-from Gnumed.pycommon import gmLog, gmDispatcher, gmSignals, gmPG, gmI18N, gmCfg
+from Gnumed.pycommon import gmLog, gmDispatcher, gmSignals, gmPG2, gmI18N, gmCfg, gmTools
 from Gnumed.business import gmPerson, gmKVK
 from Gnumed.wxpython import gmGuiHelpers, gmDemographicsWidgets
-from Gnumed.wxGladeWidgets import wxgSelectPersonFromListPnl, wxgSelectPersonFromListDlg, wxgSelectPersonDTOFromListDlg
+from Gnumed.wxGladeWidgets import wxgSelectPersonFromListDlg, wxgSelectPersonDTOFromListDlg
 
 _log = gmLog.gmDefLog
 _log.Log(gmLog.lInfo, __version__)
@@ -37,17 +37,6 @@ class cSelectPersonFromListDlg(wxgSelectPersonFromListDlg.wxgSelectPersonFromLis
 
 	def __init__(self, *args, **kwargs):
 		wxgSelectPersonFromListDlg.wxgSelectPersonFromListDlg.__init__(self, *args, **kwargs)
-	#--------------------------------------------------------
-	def set_persons(self, persons=None):
-		self._PNL_select_person.set_persons(persons=persons)
-	#--------------------------------------------------------
-	def get_selected_person(self):
-		return self._PNL_select_person.get_selected_person()
-#============================================================
-class cSelectPersonFromListPnl(wxgSelectPersonFromListPnl.wxgSelectPersonFromListPnl):
-
-	def __init__(self, *args, **kwargs):
-		wxgSelectPersonFromListPnl.wxgSelectPersonFromListPnl.__init__(self, *args, **kwargs)
 
 		self.__cols = [
 			_('Title'),
@@ -55,18 +44,15 @@ class cSelectPersonFromListPnl(wxgSelectPersonFromListPnl.wxgSelectPersonFromLis
 			_('Firstname'),
 			_('Nickname'),
 			_('DOB'),
-			_('Gender')
+			_('Gender'),
+			_('last visit'),
+			_('found via')
 		]
 		self.__init_ui()
 	#--------------------------------------------------------
 	def __init_ui(self):
 		for col in range(len(self.__cols)):
 			self._LCTRL_persons.InsertColumn(col, self.__cols[col])
-
-#		msg = _('Please select a patient from the list below.')
-#		self._lbl_message.SetLabel(msg)
-	#--------------------------------------------------------
-	# public API
 	#--------------------------------------------------------
 	def set_persons(self, persons=None):
 		self._LCTRL_persons.DeleteAllItems()
@@ -78,21 +64,21 @@ class cSelectPersonFromListPnl(wxgSelectPersonFromListPnl.wxgSelectPersonFromLis
 		for person in persons:
 			ident = person.get_identity()
 
-			if ident['title'] is None:
-				row_num = self._LCTRL_persons.InsertStringItem(pos, label = '')
-			else:
-				row_num = self._LCTRL_persons.InsertStringItem(pos, label = ident['title'])
+			row_num = self._LCTRL_persons.InsertStringItem(pos, label = gmTools.coalesce(ident['title'], ''))
 			self._LCTRL_persons.SetStringItem(index = row_num, col = 1, label = ident['lastnames'])
 			self._LCTRL_persons.SetStringItem(index = row_num, col = 2, label = ident['firstnames'])
-			if ident['preferred'] is None:
-				self._LCTRL_persons.SetStringItem(index = row_num, col = 3, label = '')
+			self._LCTRL_persons.SetStringItem(index = row_num, col = 3, label = gmTools.coalesce(ident['preferred'], ''))
+			self._LCTRL_persons.SetStringItem(index = row_num, col = 4, label = ident['dob'].strftime('%x'))
+			self._LCTRL_persons.SetStringItem(index = row_num, col = 5, label = gmTools.coalesce(ident['l10n_gender'], '?'))
+			pat = gmPerson.cPatient(identity=ident)
+			enc = pat.get_last_encounter()
+			if enc is None:
+				label = u''
 			else:
-				self._LCTRL_persons.SetStringItem(index = row_num, col = 3, label = ident['preferred'])
-			self._LCTRL_persons.SetStringItem(index = row_num, col = 4, label = ident['dob'].Format('%x'))
-			if ident['l10n_gender'] is None:
-				self._LCTRL_persons.SetStringItem(index = row_num, col = 5, label = '?')
-			else:
-				self._LCTRL_persons.SetStringItem(index = row_num, col = 5, label = ident['l10n_gender'])
+				label = u'%s (%s)' % (enc['started'].strftime('%x'), enc['l10n_type'])
+			self._LCTRL_persons.SetStringItem(index = row_num, col = 6, label = label)
+			try: self._LCTRL_persons.SetStringItem(index = row_num, col = 7, label = ident['match_type'])
+			except: pass
 
 		for col in range(len(self.__cols)):
 			self._LCTRL_persons.SetColumnWidth(col=col, width=wx.LIST_AUTOSIZE)
@@ -113,24 +99,15 @@ class cSelectPersonFromListPnl(wxgSelectPersonFromListPnl.wxgSelectPersonFromLis
 	#--------------------------------------------------------
 	def _on_list_item_activated(self, evt):
 		self._BTN_select.Enable(True)
-		# FIXME: somehow invoke SELECT button
-		print "need code to invoke SELECT button"
+		if self.IsModal():
+			self.EndModal(wx.ID_OK)
+		else:
+			self.Close()
 #============================================================
 class cSelectPersonDTOFromListDlg(wxgSelectPersonDTOFromListDlg.wxgSelectPersonDTOFromListDlg):
 
 	def __init__(self, *args, **kwargs):
 		wxgSelectPersonDTOFromListDlg.wxgSelectPersonDTOFromListDlg.__init__(self, *args, **kwargs)
-	#--------------------------------------------------------
-	def set_dtos(self, dtos=None):
-		return self._PNL_select_person.set_dtos(dtos=dtos)
-	#--------------------------------------------------------
-	def get_selected_dto(self):
-		return self._PNL_select_person.get_selected_dto()
-#============================================================
-class cSelectPersonDTOFromListPnl(wxgSelectPersonFromListPnl.wxgSelectPersonFromListPnl):
-
-	def __init__(self, *args, **kwargs):
-		wxgSelectPersonFromListPnl.wxgSelectPersonFromListPnl.__init__(self, *args, **kwargs)
 
 		self.__cols = [
 			_('Source'),
@@ -139,15 +116,12 @@ class cSelectPersonDTOFromListPnl(wxgSelectPersonFromListPnl.wxgSelectPersonFrom
 			_('DOB'),
 			_('Gender')
 		]
-
 		self.__init_ui()
 	#--------------------------------------------------------
 	def __init_ui(self):
 		for col in range(len(self.__cols)):
 			self._LCTRL_persons.InsertColumn(col, self.__cols[col])
 	#--------------------------------------------------------
-	# public API
-	#--------------------------------------------------------
 	def set_dtos(self, dtos=None):
 		self._LCTRL_persons.DeleteAllItems()
 
@@ -160,9 +134,8 @@ class cSelectPersonDTOFromListPnl(wxgSelectPersonFromListPnl.wxgSelectPersonFrom
 			dto = rec['dto']
 			self._LCTRL_persons.SetStringItem(index = row_num, col = 1, label = dto.lastnames)
 			self._LCTRL_persons.SetStringItem(index = row_num, col = 2, label = dto.firstnames)
-			self._LCTRL_persons.SetStringItem(index = row_num, col = 3, label = dto.dob.Format('%x'))
-			if dto.gender is not None:
-				self._LCTRL_persons.SetStringItem(index = row_num, col = 4, label = dto.gender)
+			self._LCTRL_persons.SetStringItem(index = row_num, col = 3, label = dto.dob.strftime('%x'))
+			self._LCTRL_persons.SetStringItem(index = row_num, col = 4, label = gmTools.coalesce(dto.gender, ''))
 
 		for col in range(len(self.__cols)):
 			self._LCTRL_persons.SetColumnWidth(col=col, width=wx.LIST_AUTOSIZE)
@@ -182,44 +155,59 @@ class cSelectPersonDTOFromListPnl(wxgSelectPersonFromListPnl.wxgSelectPersonFrom
 		return
 	#--------------------------------------------------------
 	def _on_list_item_activated(self, evt):
-		self._BTN_select.Enable(False)
-		# FIXME: somehow invoke SELECT button
-		print "need code to invoke SELECT button"
+		self._BTN_select.Enable(True)
+		if self.IsModal():
+			self.EndModal(wx.ID_OK)
+		else:
+			self.Close()
 #============================================================
 def load_persons_from_xdt():
 
-	# FIXME: potentially return several patients
+	bdt_files = []
 
+	# some can be auto-detected
+	# MCS/Isynet: $DRIVE:\Winacs\TEMP\BDTxx.tmp where xx is the workplace
+	candidates = []
+	drives = 'cdefghijklmnopqrstuvwxyz'
+	for drive in drives:
+		candidate = drive + ':\Winacs\TEMP\BDT*.tmp'
+		candidates.extend(glob.glob(candidate))
+	for candidate in candidates:
+		path, filename = os.path.split(candidate)
+		# FIXME: add encoding !
+		bdt_files.append({'file': candidate, 'source': 'MCS/Isynet %s' % filename[-6:-4]})
+
+	# some need to be configured
 	xdt_profiles = _cfg.get('workplace', 'XDT profiles')
 	if xdt_profiles is None:
 		return []
 
-	bdt_files = []
 	for profile in xdt_profiles:
 		name = _cfg.get('XDT profile %s' % profile, 'filename')
 		if name is None:
-			_log.Log(gmLog.lWarn, 'XDT profile [%s] does not define a file name' % profile)
+			_log.Log(gmLog.lErr, 'XDT profile [%s] does not define a <filename>' % profile)
 			continue
 		encoding = _cfg.get('XDT profile %s' % profile, 'encoding')
 		if encoding is None:
-			gmGuiHelpers.gm_show_error (
-				_(	'Cannot access BDT file\n\n'
-					' [%s]\n\n'
-					'to import patient.\n\n'
-					'The profile [%s] does not specify an encoding.'
-				) % (name, profile),
-				_('Activating xDT patient')
-			)
-			continue
+			_log.Log(gmLog.lWarn, 'xDT source profile [%s] does not specify an <encoding> for BDT file [%s]' % (profile, name))
 		source = _cfg.get('XDT profile %s' % profile, 'source')
 		if source is None:
 			source = _('unknown')
-		bdt_files.append({'file': name, 'source': source, 'encoding': encoding})
+		dob_format = _cfg.get('XDT profile %s' % profile, 'DOB format')
+		if dob_format is None:
+			_log.Log(gmLog.lErr, 'XDT profile [%s] does not define a date of birth format in <DOB format>' % profile)
+			continue
+		bdt_files.append({'file': name, 'source': source, 'encoding': encoding, 'dob_format': dob_format})
 
 	dtos = []
 	for bdt_file in bdt_files:
 		try:
-			dto = gmPerson.get_person_from_xdt(filename = bdt_file['file'], encoding = bdt_file['encoding'])
+			# FIXME: potentially return several patients per file
+			dto = gmPerson.get_person_from_xdt (
+				filename = bdt_file['file'],
+				encoding = bdt_file['encoding'],
+				dob_format = bdt_file['dob_format']
+			)
 
 		except IOError:
 			gmGuiHelpers.gm_show_info (
@@ -231,7 +219,7 @@ def load_persons_from_xdt():
 				) % bdt_file,
 				_('Activating xDT patient')
 			)
-			_log.LogException('cannot access xDT file [%s]' % bdt_file)
+			_log.LogException('cannot access xDT file [%s]' % bdt_file['file'])
 			continue
 
 		except ValueError:
@@ -242,7 +230,19 @@ def load_persons_from_xdt():
 				) % bdt_file,
 				_('Activating xDT patient')
 			)
-			_log.LogException('cannot read patient from xDT file [%s]' % bdt_file)
+			_log.LogException('cannot read patient from xDT file [%s]' % bdt_file['file'])
+			continue
+
+		except:
+			gmGuiHelpers.gm_show_error (
+				_(
+				'Cannot load patient from BDT file\n\n'
+				' [%s]'
+				) % bdt_file,
+				_('Activating xDT patient'),
+				aLogLevel = None
+			)
+			_log.LogException('cannot read patient from xDT file [%s]' % bdt_file['file'])
 			continue
 
 		dtos.append({'dto': dto, 'source': bdt_file['source']})
@@ -259,8 +259,10 @@ def load_patient_from_external_sources(parent=None):
 	# more types: KVK files, other interfaces, ...
 
 	if len(dtos) == 0:
+		gmGuiHelpers.gm_statustext(_('No patients found in external sources.'))
 		return True
 	if len(dtos) == 1:
+		gmGuiHelpers.gm_statustext(_('Only one patient found in external sources. Using that patient.'))
 		dto = dtos[0]['dto']
 	if len(dtos) > 1:
 		if parent is None:
@@ -271,19 +273,13 @@ def load_patient_from_external_sources(parent=None):
 		if result == wx.ID_CANCEL:
 			return True
 		dto = dlg.get_selected_dto()['dto']
+		dlg.Destroy()
 
-	# FIXME: config: delete DTO source after selection
+	# FIXME: config: delete DTO source after selection, eg KVK
 
 	# search
 	searcher = gmPerson.cPatientSearcher_SQL()
-	persons = searcher.get_persons (
-		search_dict = {
-			'firstnames': dto.firstnames,
-			'lastnames': dto.lastnames,
-			'gender': dto.gender,
-			'dob': dto.dob
-		}
-	)
+	persons = searcher.get_persons (dto = dto)
 
 	if len(persons) == 0:
 		ident = gmPerson.create_identity (
@@ -297,7 +293,7 @@ def load_patient_from_external_sources(parent=None):
 				_(
 				'Cannot create new patient:\n\n'
 				' [%s %s (%s), %s]'
-				) % (dto.firstnames, dto.lastnames, dto.gender, dto.dob.Format('%x')),
+				) % (dto.firstnames, dto.lastnames, dto.gender, dto.dob.strftime('%x')),
 				_('Activating xDT patient')
 			)
 			return False
@@ -315,6 +311,7 @@ def load_patient_from_external_sources(parent=None):
 		if result == wx.ID_CANCEL:
 			return True
 		person = dlg.get_selected_person()
+		dlg.Destroy()
 
 	if not gmPerson.set_active_patient(patient = person.get_identity()):
 		gmGuiHelpers.gm_show_info (
@@ -322,201 +319,12 @@ def load_patient_from_external_sources(parent=None):
 			'Cannot activate patient:\n\n'
 			'%s %s (%s)\n'
 			'%s'
-			) % (dto.firstnames, dto.lastnames, dto.gender, dto.dob.Format('%x')),
+			) % (dto.firstnames, dto.lastnames, dto.gender, dto.dob.strftime('%x')),
 			_('Activating xDT patient')
 		)
 		return False
 
 	return True
-
-#============================================================
-# country-specific functions
-#------------------------------------------------------------
-# FIXME: this belongs elsewhere !!!!
-
-def pat_expand_default(curs = None, ID_list = None):
-	if ID_list is None:
-		return ([], [])
-
-	if curs is None:
-		return ([], [])
-
-	pat_data = []
-
-	# FIXME: add more data here
-	# - last visit
-	# - appointment
-	# - current waiting time
-	# - presence
-	# - KVK indicator
-	# - been here this Quartal
-	# ...
-	# Note: this query must ALWAYS return the ID in field 0
-	cmd = """
-		SELECT pk_identity, lastnames, firstnames, to_char(dob, 'DD.MM.YYYY')
-		FROM v_basic_person
-		WHERE pk_identity in (%s)
-		""" % ','.join(map(lambda x: str(x), ID_list))
-	pat_data = gmPG.run_ro_query(curs, cmd)
-	if pat_data is None:
-		_log.Log(gmLog.lErr, 'cannot fetch extended patient data')
-
-	return pat_data, col_order
-#------------------------------------------------------------
-patient_expander = {
-	'default': pat_expand_default,
-	'de': pat_expand_default
-}
-#============================================================
-class cPatientPickList(wx.Dialog):
-	def __init__(
-		self,
-		parent,
-		id = -1,
-		title = None,
-		pos = (-1, -1),
-		size = (600, 400),
-	):
-
-		if title is None:
-			title = _('please select a patient')
-
-		# this works (as suggested by Robin Dunn) but is quite ugly IMO
-		prnt = wx.GetTopLevelParent(parent)
-		wx.Dialog.__init__(
-			self,
-			prnt,
-			id,
-			title,
-			pos,
-			size,
-			style = wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER | wx.STAY_ON_TOP
-		)
-
-		self.__register_events()
-
-		self.__do_layout()
-		self.__items = []
-	#--------------------------------------------------------
-	# external API
-	#--------------------------------------------------------
-	def SetItems(self, items = None, col_order = None):
-		if items is None:
-			items = []
-		if col_order is None:
-			col_order = []
-		# TODO: make selectable by 0-9
-		self.__items = items
-		# set col_order
-		if not col_order:
-			col_order = [
-				{'label': _('Surname'), 'field name': 'lastnames'},
-				{'label': _('First name'), 'field name': 'firstnames'},
-				{'label': _('nick'), 'field name': 'preferred'},
-				{'label': _('DOB'), 'field name': 'dob'}
-			]
-
-		# 1) set up column headers
-		self.__listctrl.ClearAll()
-		for order_idx in range(len(col_order)):
-			self.__listctrl.InsertColumn(order_idx, col_order[order_idx]['label'])
-
-		# 2) add items
-		for row_idx in range(len(self.__items)):
-			row = self.__items[row_idx]
-			# first column
-			self.__listctrl.InsertStringItem(row_idx, str(row[col_order[0]['field name']]))
-			# subsequent columns
-			for order_idx in range(1, len(col_order)):
-				self.__listctrl.SetStringItem(row_idx, order_idx, str(row[col_order[order_idx]['field name']]))
-
-		# adjust column width
-		for order_idx in range(len(col_order)):
-			self.__listctrl.SetColumnWidth(col=order_idx, width=wx.LIST_AUTOSIZE)
-
-		# FIXME: and make ourselves just big enough
-		self.sizer_main.Fit(self)
-		self.Fit()
-	#--------------------------------------------------------
-	# internal API
-	#--------------------------------------------------------
-	def __register_events(self):
-		wx.EVT_LIST_ITEM_ACTIVATED(self, ID_PatPickList, self._on_item_activated)
-		wx.EVT_BUTTON(self, wx.ID_CANCEL, self._on_cancel)
-	#--------------------------------------------------------
-	def __do_layout(self):
-		self.__listctrl = wx.ListCtrl(
-			parent = self,
-			id = ID_PatPickList,
-			size = (600,200),
-			style = wx.LC_SINGLE_SEL | wx.VSCROLL | wx.SUNKEN_BORDER| wx.LC_REPORT | wx.LC_VRULES| wx.LC_HRULES
-		)
-		#-----------------------------------------------------------------------------------------------------------
-		# make horizontal sizer and put <Add><Ok><Cancel> into it
-		#-----------------------------------------------------------------------------------------------------------
-		sizer_buttons = wx.BoxSizer(wx.HORIZONTAL)				#bottom sizer to hold buttons
-		# Ok Button = load patient
-		btnOK = wx.Button (
-			self,
-			wx.ID_OK,
-			_("&Activate"),
-			wx.DefaultPosition,
-			wx.DefaultSize,
-			0
-		)
-		# allow add new patient
-		btnAddNew = wx.Button (
-			self,
-			ID_BTN_AddNew,
-			_("Add as &New"),
-			wx.DefaultPosition,
-			wx.DefaultSize,
-			0
-		)
-		# cancel pick list
-		btnCancel = wx.Button (
-			self,
-			wx.ID_CANCEL,
-			_("&Cancel"),
-			wx.DefaultPosition,
-			wx.DefaultSize,
-			0
-		)
-		spacer = wx.BoxSizer(wx.HORIZONTAL)
-		sizer_buttons.Add(spacer, 20, 1, wx.EXPAND)
-		sizer_buttons.Add(btnAddNew, 0, wx.EXPAND | wx.TOP | wx.BOTTOM, 5)
-		sizer_buttons.Add(btnOK, 0, wx.EXPAND | wx.ALL , 5)
-		sizer_buttons.Add(btnCancel, 0, wx.EXPAND | wx.ALL, 5)
-		#---------------------------------------------------------------------------------------
-		# vertical box sizer will stack vertically
-		#  - list control
-		#  - row of buttons
-		#----------------------------------------------------------------------------------------
-		self.sizer_main = wx.BoxSizer(wx.VERTICAL)
-		self.sizer_main.Add(self.__listctrl, 1, wx.EXPAND | wx.ALL, 10)
-		self.sizer_main.AddSizer(sizer_buttons, 0, wx.EXPAND | wx.LEFT | wx.RIGHT, 10)
-		#-----------------------------------
-		# now set the main sizer
-		#-----------------------------------
-		self.SetAutoLayout(True)
-		self.SetSizer(self.sizer_main)
-		self.sizer_main.Fit(self)
-		self.sizer_main.SetSizeHints(self)
-		self.__listctrl.SetFocus()					# won't work on Windoze without this
-	#--------------------------------------------------------
-	# event handlers
-	#--------------------------------------------------------
-	def _on_item_activated(self, evt):
-		# item dict must always contain ID to be used for selection later on at index 0
-		self.selected_item = self.__items[evt.m_itemIndex]
-		try:
-			self.EndModal(1)
-		except KeyError:
-			_log.LogException('item [%s] has faulty structure' % item, sys.exc_info())
-			self.EndModal(-1)
-	#--------------------------------------------------------
-	def _on_cancel(self, evt):
-		self.EndModal(-1)
 #============================================================
 class cPatientSelector(wx.TextCtrl):
 	"""Widget for smart search for patients."""
@@ -558,23 +366,6 @@ class cPatientSelector(wx.TextCtrl):
 		# FIXME: set query generator
 		self.__pat_searcher = gmPerson.cPatientSearcher_SQL()
 
-		# - retriever
-		try:
-			self.__pat_expander = patient_expander[gmI18N.system_locale_level['full']]
-		except KeyError:
-			try:
-				self.__pat_expander = patient_expander[gmI18N.system_locale_level['country']]
-			except KeyError:
-				try:
-					self.__pat_expander = patient_expander[gmI18N.system_locale_level['language']]
-				except KeyError:
-					self.__pat_expander = patient_expander['default']
-
-		# get connection
-		backend = gmPG.ConnectionPool()
-		self.__conn = backend.GetConnection('personalia')
-		# FIXME: error handling
-
 		self.__prev_search_term = None
 		self.__prev_idents = []
 		self.__pat_picklist_col_defs = []
@@ -725,6 +516,7 @@ class cPatientSelector(wx.TextCtrl):
 			persons = []
 			for ident in self.__prev_idents:
 				persons.append(gmPerson.cPerson(identity=ident))
+
 			dlg = cSelectPersonFromListDlg(parent=wx.GetTopLevelParent(self), id=-1)
 			dlg.set_persons(persons=persons)
 			result = dlg.ShowModal()
@@ -732,7 +524,9 @@ class cPatientSelector(wx.TextCtrl):
 				wx.BeginBusyCursor()
 				person = dlg.get_selected_person()
 				self.SetActivePatient(person)
-				wx.EndBusyCursor()
+			dlg.Destroy()
+
+			wx.EndBusyCursor()
 			return True
 
 		# previous search fragment
@@ -779,10 +573,15 @@ class cPatientSelector(wx.TextCtrl):
 	#--------------------------------------------------------
 	def _on_enter(self, evt):
 
-		curr_search_term = self.GetValue()
-		if curr_search_term.strip() == '':
+		curr_search_term = self.GetValue().strip()
+		if curr_search_term == '':
 			return None
 
+		if self.curr_pat.is_connected():
+			ident = self.curr_pat.get_identity()
+			if curr_search_term == ident['description']:
+				return None
+
 		wx.BeginBusyCursor()
 
 		if self.__always_dismiss_after_search:
@@ -801,7 +600,9 @@ class cPatientSelector(wx.TextCtrl):
 		if persons is None:
 			wx.EndBusyCursor()
 			gmGuiHelpers.gm_show_info (
-				_('Error searching for matching patients.\n\nSearch term: "%s"' % curr_search_term),
+				_('Error searching for matching patients.\n\n'
+				  'Search term: "%s"'
+				) % curr_search_term,
 				_('selecting patient')
 			)
 			return None
@@ -811,7 +612,8 @@ class cPatientSelector(wx.TextCtrl):
 		if len(persons) == 0:
 			wx.EndBusyCursor()
 			gmGuiHelpers.gm_show_info (
-				_('Cannot find any matching patients.\n\n'
+				_('Cannot find any matching patients.\n'
+				  'You may want to try a shorter search term.\n\n'
 				  'Search term: "%s"\n\n'
 				  'You will be taken to the "New Patient" wizard now.'
 				) % curr_search_term,
@@ -833,10 +635,12 @@ class cPatientSelector(wx.TextCtrl):
 		wx.EndBusyCursor()
 		result = dlg.ShowModal()
 		if result == wx.ID_CANCEL:
+			dlg.Destroy()
 			return None
 
 		wx.BeginBusyCursor()
 		person = dlg.get_selected_person()
+		dlg.Destroy()
 		self.SetActivePatient(person)
 		wx.EndBusyCursor()
 
@@ -851,7 +655,6 @@ if __name__ == "__main__":
 	gmI18N.install_domain()
 
 	app = wx.PyWidgetTester(size = (200, 40))
-#	app.SetWidget(cPatientSelector, -1)
 	app.SetWidget(cSelectPersonFromListDlg, -1)
 	app.MainLoop()
 
@@ -968,8 +771,64 @@ if __name__ == "__main__":
 
 #============================================================
 # $Log: gmPatSearchWidgets.py,v $
-# Revision 1.41.2.1  2006/09/13 07:57:55  ncq
-# - handle source file encoding in xDT sources
+# Revision 1.59  2007/01/20 22:52:27  ncq
+# - .KeyCode -> GetKeyCode()
+#
+# Revision 1.58  2007/01/18 22:07:52  ncq
+# - (Get)KeyCode() -> KeyCode so 2.8 can do
+#
+# Revision 1.57  2007/01/10 23:04:12  ncq
+# - support explicit DOB format for xDT files
+#
+# Revision 1.56  2006/12/13 14:57:16  ncq
+# - inform about no patients found in external sources
+#
+# Revision 1.55  2006/11/24 14:23:19  ncq
+# - self.Close() does not need wx.ID_*
+#
+# Revision 1.54  2006/11/24 09:56:03  ncq
+# - improved message when error searching patient
+#
+# Revision 1.53  2006/11/20 19:11:04  ncq
+# - improved message when no matching patient found
+#
+# Revision 1.52  2006/11/20 17:05:55  ncq
+# - do not search if supposed search term matches 'description' of current patient
+#
+# Revision 1.51  2006/11/01 12:54:40  ncq
+# - there may not be a previous encounter so don't try to
+#   format it's start date if so
+#
+# Revision 1.50  2006/10/31 12:43:09  ncq
+# - out with the crap
+# - no more patient expanders
+#
+# Revision 1.49  2006/10/30 16:46:52  ncq
+# - missing encoding in xDT source defs does not *have* to be
+#   an error as the file itself may contain the encoding itself
+#
+# Revision 1.48  2006/10/28 14:57:17  ncq
+# - use cPatient.get_last_encounter()
+#
+# Revision 1.47  2006/10/28 12:34:53  ncq
+# - make person and dto selector dialogs handle functionality themselves
+# - remove person selector panel class
+# - act on ENTER/double-click in person/dto select list
+#
+# Revision 1.46  2006/10/25 07:46:44  ncq
+# - Format() -> strftime() since datetime.datetime does not have .Format()
+#
+# Revision 1.45  2006/10/24 13:26:43  ncq
+# - switch to gmPG2
+#
+# Revision 1.44  2006/09/13 07:55:11  ncq
+# - handle encoding in xDT patient sources
+#
+# Revision 1.43  2006/09/06 07:22:34  ncq
+# - add missing import for glob module
+#
+# Revision 1.42  2006/09/01 14:46:30  ncq
+# - add (untested) MCS/Isynet external patient source
 #
 # Revision 1.41  2006/08/09 15:00:47  ncq
 # - better search widget tooltip
