@@ -10,26 +10,28 @@ This is based on seminal work by Ian Haywood <ihaywood@gnu.org>
 
 ############################################################################
 # $Source: /sources/gnumed/gnumed/gnumed/client/wxpython/gmPhraseWheel.py,v $
-# $Id: gmPhraseWheel.py,v 1.76 2006/07/19 20:29:50 ncq Exp $
-__version__ = "$Revision: 1.76 $"
+# $Id: gmPhraseWheel.py,v 1.91 2007/01/20 22:52:27 ncq Exp $
+__version__ = "$Revision: 1.91 $"
 __author__  = "K.Hilbert <Karsten.Hilbert@gmx.net>, I.Haywood, S.J.Tan <sjtan@bigpond.com>"
 
 import string, types, time, sys, re
 
-import wxversion
 import wx
 import wx.lib.mixins.listctrl as listmixins
 
+# GNUmed specific
+if __name__ == '__main__':
+	sys.path.insert(0, '../../')
 from Gnumed.wxpython import gmTimer, gmGuiHelpers
-from Gnumed.pycommon import gmLog, gmExceptions, gmPG, gmMatchProvider, gmGuiBroker, gmNull
+from Gnumed.pycommon import gmLog, gmExceptions, gmPG2, gmMatchProvider, gmGuiBroker, gmNull
 
 _log = gmLog.gmDefLog
 if __name__ == "__main__":
 	_log.SetAllLogLevels(gmLog.lData)
 
 _log.Log(gmLog.lInfo, __version__)
-
 #============================================================
+# FIXME: merge with gmListWidgets
 class cPhraseWheelListCtrl(wx.ListCtrl, listmixins.ListCtrlAutoWidthMixin):
 	def __init__(self, *args, **kwargs):
 		try:
@@ -47,8 +49,11 @@ class cPhraseWheelListCtrl(wx.ListCtrl, listmixins.ListCtrlAutoWidthMixin):
 	#--------------------------------------------------------
 	def GetSelectedItemData(self):
 		return self.__data[self.GetFirstSelected()]['data']
+	#--------------------------------------------------------
+	def get_selected_item_label(self):
+		return self.__data[self.GetFirstSelected()]['label']
 #============================================================
-class cPhraseWheel (wx.TextCtrl):
+class cPhraseWheel(wx.TextCtrl):
 	"""Widget for smart guessing of user fields, after Richard Terry's interface."""
 
 	default_phrase_separators = re.compile('[;/|]+')
@@ -62,9 +67,7 @@ class cPhraseWheel (wx.TextCtrl):
 		**kwargs):
 
 		self.__matcher = aMatchProvider
-		self.__real_matcher = None
 		self.__currMatches = []
-		self._input_was_selected = False
 
 		self.phrase_separators = cPhraseWheel.default_phrase_separators
 		self.allow_multiple_phrases()
@@ -73,20 +76,29 @@ class cPhraseWheel (wx.TextCtrl):
 		self.left_part = ''
 		self.right_part = ''
 		self.data = None
-		
+
 		self.selection_only = selection_only
+		self.snap_to_first_match = (False or self.selection_only)
 		self._has_focus = False
 		self._screenheight = wx.SystemSettings.GetMetric(wx.SYS_SCREEN_Y)
 
 		self._on_selection_callbacks = []
 		self._on_lose_focus_callbacks = []
 		self._on_set_focus_callbacks = []
-		self.notified_listeners = False
+		self._on_modified_callbacks = []
 
 		wx.TextCtrl.__init__ (self, parent, id, **kwargs)
 
 		# multiple matches dropdown list
-		self.__dropdown = wx.PopupWindow(parent)
+		try:
+			self.__dropdown = wx.PopupWindow(parent)
+			add_picklist_to_sizer = False
+		except NotImplementedError:
+			# on MacOSX wx.PopupWindow does not exist
+			self.__dropdown = wx.Window(parent=parent, style = wx.SIMPLE_BORDER)
+			szr_scroll = wx.BoxSizer(wx.VERTICAL)
+			self.__dropdown.SetSizer(szr_scroll)
+			add_picklist_to_sizer = True
 
 		# FIXME: support optional headers
 #		if kwargs['show_list_headers']:
@@ -99,6 +111,9 @@ class cPhraseWheel (wx.TextCtrl):
 		)
 		self._picklist.InsertColumn(0, '')
 
+		if add_picklist_to_sizer:
+			szr_scroll.Add(self._picklist, 1, wx.EXPAND)
+
 		self.__dropdown.Hide()
 
 		# set event handlers
@@ -113,15 +128,19 @@ class cPhraseWheel (wx.TextCtrl):
 				callback = self._on_timer_fired,
 				delay = aDelay
 			)
+			# initially stopped
+			self.__timer.Stop()
 	#--------------------------------------------------------
 	# external API
 	#--------------------------------------------------------
+	def set_snap_to_first_match(self, state = False):
+		self.snap_to_first_match = (state or self.selection_only)
+	#--------------------------------------------------------
 	def allow_multiple_phrases(self, state = True):
 		self.__handle_multiple_phrases = state
 	#--------------------------------------------------------
-	def setMatchProvider (self, mp):
+	def setMatchProvider (self, mp=None):
 		self.__matcher = mp
-		self.__real_matcher = None
 	#--------------------------------------------------------
 	def add_callback_on_selection(self, callback=None):
 		"""Add a callback for a listener.
@@ -133,22 +152,28 @@ class cPhraseWheel (wx.TextCtrl):
 		user changes a previously selected value.
 		"""
 		if not callable(callback):
-			_log.Log(gmLog.lWarn, 'ignoring callback [%s], it is not callable' % callback)
-			return False
+			raise ValueError('[add_callback_on_selection]: ignoring callback [%s], it is not callable' % callback)
+
 		self._on_selection_callbacks.append(callback)
 	#---------------------------------------------------------
 	def add_callback_on_set_focus(self, callback=None):
 		if not callable(callback):
-			_log.Log(gmLog.lWarn, 'ignoring callback [%s] - not callable' % callback)
-			return False
+			raise ValueError('[add_callback_on_set_focus]: ignoring callback [%s] - not callable' % callback)
+
 		self._on_set_focus_callbacks.append(callback)
 	#---------------------------------------------------------
 	def add_callback_on_lose_focus(self, callback=None):
 		if not callable(callback):
-			_log.Log(gmLog.lWarn, 'ignoring callback [%s] - not callable' % callback)
-			return False
+			raise ValueError('[add_callback_on_lose_focus]: ignoring callback [%s] - not callable' % callback)
+
 		self._on_lose_focus_callbacks.append(callback)
 	#---------------------------------------------------------
+	def add_callback_on_modified(self, callback=None):
+		if not callable(callback):
+			raise ValueError('[add_callback_on_modified]: ignoring callback [%s] - not callable' % callback)
+
+		self._on_modified_callbacks.append(callback)
+	#---------------------------------------------------------
 	def SetData(self, data=None):
 		"""Set the data and thereby set the value, too.
 
@@ -159,16 +184,30 @@ class cPhraseWheel (wx.TextCtrl):
 		The whole thing will only work if data is found
 		in the match space anyways.
 		"""
-		matched, matches = self.__matcher.getMatches('*')
-		if not matched and self.selection_only:
-			return False
-		if len(matches) == 0:
-			return False
+		if self.__matcher is None:
+			matched, matches = (False, [])
+		else:
+			matched, matches = self.__matcher.getMatches('*')
+
+		if self.selection_only:
+			if not matched or (len(matches) == 0):
+				self.SetBackgroundColour('pink')
+				return False
+
 		for match in matches:
 			if match['data'] == data:
-				self.SetValue(value = match['label'], data = data)
+				self.SetBackgroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOW))
+				wx.TextCtrl.SetValue(self, match['label'])
+				self.data = data
 				return True
-		return False
+
+		# no match found ...
+		if self.selection_only:
+			self.SetBackgroundColour('pink')
+			return False
+
+		self.SetBackgroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOW))
+		return True
 	#---------------------------------------------------------
 	def GetData (self):
 		"""
@@ -179,37 +218,42 @@ class cPhraseWheel (wx.TextCtrl):
 	def SetValue (self, value, data=None):
 
 		wx.TextCtrl.SetValue(self, value)
-		self._input_was_selected = False
+		self.data = data
+		self.SetBackgroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOW))
 
-		# set data item if available
-		if data is not None:
-			self.data = data
-			self._input_was_selected = True
+		# if data already available
+		if self.data is not None:
 			return True
 
-		# or try to find one from matches
-		stat, matches = self.__matcher.getMatches(aFragment = value)
+		# or try to find data from matches
+		if self.__matcher is None:
+			stat, matches = (False, [])
+		else:
+			stat, matches = self.__matcher.getMatches(aFragment = value)
+
 		for match in matches:
 			if match['label'] == value:
 				self.data = match['data']
-				self._input_was_selected = True
 				return True
 
 		# not found
-		self.data = None
 		if self.selection_only:
+			self.SetBackgroundColour('pink')
 			return False
+
 		return True
 	#--------------------------------------------------------
-	def set_context (self, context, val):
-		if self.__real_matcher:
-			# forget any caching, as it's now invalid
-			self.__matcher = self.__real_matcher
-			self.__real_matcher = None
-		if self.__matcher:
-			self.__matcher.set_context (context, val)
+	def set_context (self, context=None, val=None):
+		if self.__matcher is not None:
+			self.__matcher.set_context(context=context, val=val)
 		else:
-			_log.Log(gmLog.lErr, "aMatchProvider must be set to set context")
+			_log.Log(gmLog.lWarn, "aMatchProvider must be set to set context")
+	#---------------------------------------------------------
+	def unset_context(self, context=None):
+		if self.__matcher is not None:
+			self.__matcher.unset_context(context=context)
+		else:
+			_log.Log(gmLog.lWarn, "aMatchProvider must be set to unset context")
 	#---------------------------------------------------------
 	def _updateMatches(self, val=None):
 		"""Get the matches for the currently typed input fragment."""
@@ -242,7 +286,7 @@ class cPhraseWheel (wx.TextCtrl):
 			self.input2match = val
 
 		# get all currently matching items
-		if self.__matcher:
+		if self.__matcher is not None:
 			matched, self.__currMatches = self.__matcher.getMatches(self.input2match)
 			self._picklist.SetItems(self.__currMatches)
 		else:
@@ -255,7 +299,7 @@ class cPhraseWheel (wx.TextCtrl):
 
 		# this helps if the current input was already selected from the
 		# list but still is the substring of another pick list item
-		if self._input_was_selected:
+		if self.data is not None:
 			return 1
 
 		if not self._has_focus:
@@ -267,7 +311,6 @@ class cPhraseWheel (wx.TextCtrl):
 		# if only one match and text == match
 		if len(self.__currMatches) == 1:
 			if self.__currMatches[0]['label'] == self.input2match:
-				self._input_was_selected = 1
 				self.data = self.__currMatches[0]['data']
 				return 1
 
@@ -315,23 +358,27 @@ class cPhraseWheel (wx.TextCtrl):
 	#--------------------------------------------------------
 	def _on_list_item_selected(self, *args, **kwargs):
 		"""Gets called when user selected a list item."""
+
 		self._hide_dropdown()
+		self.SetBackgroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOW))
+
+		self.data = self._picklist.GetSelectedItemData()	# just so that _calc_display_string could use it
 
 		# update our display
 		if self.__handle_multiple_phrases:
-			wx.TextCtrl.SetValue (self, '%s%s%s' % (self.left_part, self._calc_display_string(), self.right_part))
+			wx.TextCtrl.SetValue (self, u'%s%s%s' % (self.left_part, self._calc_display_string(), self.right_part))
 		else:
 			wx.TextCtrl.SetValue (self, self._calc_display_string())
-		self.Navigate()
-		self.MarkDirty()
 
 		self.data = self._picklist.GetSelectedItemData()
-		self._input_was_selected = True
+		self.MarkDirty()
 
 		# and tell the listeners about the user's selection
 		for call_listener in self._on_selection_callbacks:
 			call_listener(self.data)
-		self.notified_listeners = True
+
+		self.Navigate()
+		return
 	#--------------------------------------------------------
 	# individual key handlers
 	#--------------------------------------------------------
@@ -407,14 +454,15 @@ class cPhraseWheel (wx.TextCtrl):
 		return
 	#--------------------------------------------------------
 	def _on_text_update (self, event):
-		"""Internal handler for wx.EVT_TEXT (called when text has changed)"""
+		"""Internal handler for wx.EVT_TEXT.
 
-		self._input_was_selected = False
+		Called when text was changed by user or SetValue().
+		"""
 		self.data = None
 
 		# if empty string then kill list dropdown window
 		# we also don't need a timer event then
-		if self.GetValue().strip() == 0:
+		if self.GetValue().strip() == u'':
 			self._hide_dropdown()
 			self.__timer.Stop()
 		else:
@@ -422,12 +470,12 @@ class cPhraseWheel (wx.TextCtrl):
 			# milliseconds needed for Windows bug
 			self.__timer.Start(oneShot = True)
 
-		if self.notified_listeners:
-			# Aargh! we told the listeners that we selected <foo>
-			# but now the user is typing again !
-			for notify_listener in self._on_selection_callbacks:
-				notify_listener(None)
-			self.notified_listeners = False
+		# notify interested parties
+		for callback in self._on_modified_callbacks:
+			if not callback():
+				print "[%s:_on_text_update]: %s returned False" % (self.__class__.__name__, str(callback))
+
+		return
 	#--------------------------------------------------------
 	def _on_timer_fired(self, cookie):
 		"""Callback for delayed match retrieval timer.
@@ -454,21 +502,19 @@ class cPhraseWheel (wx.TextCtrl):
 
 		# notify interested parties
 		for callback in self._on_set_focus_callbacks:
-			try:
-				if not callback():
-					print "[%s:_on_set_focus]: %s returned False" % (self.__class__.__name__, str(callback))
-			except:
-				_log.LogException("[%s:_on_set_focus]: error calling %s" % (self.__class__.__name__, str(callback)), sys.exc_info())
+			if not callback():
+				print "[%s:_on_set_focus]: %s returned False" % (self.__class__.__name__, str(callback))
 
-		# if empty set to first "match"
-		if self.GetValue().strip() == '':
-			# programmers better make sure the turnaround time is limited
-			self._updateMatches()
-			if len(self.__currMatches) > 0:
-				wx.TextCtrl.SetValue(self, self.__currMatches[0]['label'])
-				self.data = self.__currMatches[0]['data']
-				self._input_was_selected = True
-				self.MarkDirty()
+		if self.snap_to_first_match:
+			if self.GetValue().strip() == u'':
+				# programmers better make sure the turnaround time is limited
+				self._updateMatches(val='*')
+				if len(self.__currMatches) > 0:
+					wx.TextCtrl.SetValue(self, self.__currMatches[0]['label'])
+					self.data = self.__currMatches[0]['data']
+					self.MarkDirty()
+
+		self.__timer.Start(oneShot = True)
 
 		return True
 	#--------------------------------------------------------
@@ -481,100 +527,191 @@ class cPhraseWheel (wx.TextCtrl):
 		self.__timer.Stop()
 		self._hide_dropdown()
 
+		self.SetSelection(1,1)
+
 		# can/must we auto-set the value from the match list ?
-		if (self.selection_only) and (not self._input_was_selected) and (self.GetValue().strip() != ''):
-			self._updateMatches()
-			no_matches = len(self.__currMatches)
-			if no_matches == 1:
-				wx.TextCtrl.SetValue(self, self.__currMatches[0]['label'])
-				self.data = self.__currMatches[0]['data']
-				self._input_was_selected = True
-				self.MarkDirty()
-			elif no_matches > 1:
-				gmGuiHelpers.gm_beep_statustext(_('Cannot auto-select from list. There are several matches for the input.'))
-				return True
-			else:
-				gmGuiHelpers.gm_beep_statustext(_('There are no matches for this input.'))
-				self.Clear()
-				return True
+		if self.snap_to_first_match:
+			if self.data is None:
+				val = self.GetValue().strip()
+				if val != u'':
+					self._updateMatches()
+					no_matches = len(self.__currMatches)
+					if no_matches == 1:
+						wx.TextCtrl.SetValue(self, self.__currMatches[0]['label'])
+						self.data = self.__currMatches[0]['data']
+						self.MarkDirty()
+					elif no_matches > 1:
+						gmGuiHelpers.gm_statustext(_('Cannot auto-select from list. There are several matches for the input [%s].') % val)
+						if self.selection_only:
+							self.SetBackgroundColour('pink')
+						return True
+					else:
+						gmGuiHelpers.gm_statustext(_('There are no matches for this input.'))
+						if self.selection_only:
+							self.SetBackgroundColour('pink')
+							self.Clear()
+						return True
 
 		# notify interested parties
 		for callback in self._on_lose_focus_callbacks:
-			try:
-				if not callback():
-					print "[%s:_on_lose_focus]: %s returned False" % (self.__class__.__name__, str(callback))
-			except:
-				_log.LogException("[%s:_on_lose_focus]: error calling %s" % (self.__class__.__name__, str(callback)), sys.exc_info())
+			if not callback():
+				print "[%s:_on_lose_focus]: %s returned False" % (self.__class__.__name__, str(callback))
+
 		return True
 #--------------------------------------------------------
 # MAIN
 #--------------------------------------------------------
 if __name__ == '__main__':
 	from Gnumed.pycommon import gmI18N
-	#----------------------------------------------------
-	def clicked (data):
-		print "Selected :%s" % data
-	#----------------------------------------------------
-	class TestApp (wx.App):
-		def OnInit (self):
+	gmI18N.activate_locale()
+	gmI18N.install_domain(text_domain='gnumed')
 
-			frame = wx.Frame (
-				None,
-				-4,
-				"phrase wheel test for GNUmed",
-				size=wx.Size(300, 350),
-				style=wx.DEFAULT_FRAME_STYLE|wx.NO_FULL_REPAINT_ON_RESIZE
-			)
-
-			items = [	{'data':1, 'label':"Bloggs", 	'weight':5},
-						{'data':2, 'label':"Baker",  	'weight':4},
-						{'data':3, 'label':"Jones",  	'weight':3},
-						{'data':4, 'label':"Judson", 	'weight':2},
-						{'data':5, 'label':"Jacobs", 	'weight':1},
-						{'data':6, 'label':"Judson-Jacobs",'weight':5}
-					]
-			mp1 = gmMatchProvider.cMatchProvider_FixedList(items)
-			# do NOT treat "-" as a word separator here as there are names like "asa-sismussen"
-			mp1.setWordSeparators(separators = '[ \t=+&:@]+')
-			ww1 = cPhraseWheel(
-				parent = frame,
-				id = -1,
-#				pos = (50, 50),
-#				size = (180, 30),
-				aMatchProvider = mp1
-			)
-
-			print "Do you want to test the database connected phrase wheel ?"
-			yes_no = raw_input('y/n: ')
-			if yes_no == 'y':
-				src = {
-					'service': 'default',
-					'table': 'gmpw_sql_test',
-					'column': 'phrase',
-					'limit': 25
-				}
-				score = {
-					'service': 'default',
-					'table': 'score_gmpw_sql_test',
-					'column': 'fk_gmpw_sql_test'
-				}
-				mp2 = gmMatchProvider.cMatchProvider_SQL([src], score)
-				ww2 = cPhraseWheel(
-					parent = frame,
-					id = -1,
-					pos = (50, 250),
-					size = (180, 30),
-					aMatchProvider = mp2
-				)
-
-			frame.Show (1)
-			return 1
+	prw = None
 	#--------------------------------------------------------
-	app = TestApp ()
-	app.MainLoop ()
+	def display_values_set_focus(*args, **kwargs):
+		print "got focus:"
+		print "value:", prw.GetValue()
+		print "data :", prw.GetData()
+		return True
+	#--------------------------------------------------------
+	def display_values_lose_focus(*args, **kwargs):
+		print "lost focus:"
+		print "value:", prw.GetValue()
+		print "data :", prw.GetData()
+		return True
+	#--------------------------------------------------------
+	def display_values_modified(*args, **kwargs):
+		print "modified:"
+		print "value:", prw.GetValue()
+		print "data :", prw.GetData()
+		return True
+	#--------------------------------------------------------
+	def display_values_selected(*args, **kwargs):
+		print "selected:"
+		print "value:", prw.GetValue()
+		print "data :", prw.GetData()
+		return True
+	#--------------------------------------------------------
+	def test_prw_fixed_list():
+		app = wx.PyWidgetTester(size = (200, 50))
+
+		items = [	{'data':1, 'label':"Bloggs", 	'weight':5},
+					{'data':2, 'label':"Baker",  	'weight':4},
+					{'data':3, 'label':"Jones",  	'weight':3},
+					{'data':4, 'label':"Judson", 	'weight':2},
+					{'data':5, 'label':"Jacobs", 	'weight':1},
+					{'data':6, 'label':"Judson-Jacobs",'weight':5}
+				]
+
+		mp = gmMatchProvider.cMatchProvider_FixedList(items)
+		# do NOT treat "-" as a word separator here as there are names like "asa-sismussen"
+		mp.setWordSeparators(separators = '[ \t=+&:@]+')
+		global prw
+		prw = cPhraseWheel(
+			parent = app.frame,
+			id = -1,
+			aMatchProvider = mp
+		)
+		prw.add_callback_on_set_focus(callback=display_values_set_focus)
+		prw.add_callback_on_modified(callback=display_values_modified)
+		prw.add_callback_on_lose_focus(callback=display_values_lose_focus)
+		prw.add_callback_on_selection(callback=display_values_selected)
+
+		app.frame.Show(True)
+		app.MainLoop()
+
+		return True
+	#--------------------------------------------------------
+	def test_prw_sql2():
+		print "Do you want to test the database connected phrase wheel ?"
+		yes_no = raw_input('y/n: ')
+		if yes_no != 'y':
+			return True
+
+		gmPG2.get_connection()
+		# FIXME: add callbacks
+		# FIXME: add context
+		query = u'select code, name from dem.country where _(name) %(fragment_condition)s'
+		mp = gmMatchProvider.cMatchProvider_SQL2(queries = [query])
+		app = wx.PyWidgetTester(size = (200, 50))
+		global prw
+		prw = cPhraseWheel (
+			parent = app.frame,
+			id = -1,
+			aMatchProvider = mp
+		)
+
+		app.frame.Show(True)
+		app.MainLoop()
+
+		return True
+	#--------------------------------------------------------
+	try:
+		test_prw_fixed_list()
+		test_prw_sql2()
+	except StandardError:
+		_log.LogException("unhandled exception caught !", sys.exc_info(), 1)
+		raise
 
 #==================================================
 # $Log: gmPhraseWheel.py,v $
+# Revision 1.91  2007/01/20 22:52:27  ncq
+# - .KeyCode -> GetKeyCode()
+#
+# Revision 1.90  2007/01/18 22:07:52  ncq
+# - (Get)KeyCode() -> KeyCode so 2.8 can do
+#
+# Revision 1.89  2007/01/06 23:44:19  ncq
+# - explicitely unset selection on lose focus
+#
+# Revision 1.88  2006/11/28 20:51:13  ncq
+# - a missing self
+# - remove some prints
+#
+# Revision 1.87  2006/11/27 23:08:36  ncq
+# - add snap_to_first_match
+# - add on_modified callbacks
+# - set background in lose_focus in some cases
+# - improve test suite
+#
+# Revision 1.86  2006/11/27 12:42:31  ncq
+# - somewhat improved dropdown picklist on Mac, not properly positioned yet
+#
+# Revision 1.85  2006/11/26 21:42:47  ncq
+# - don't use wx.ScrolledWindow or we suffer double-scrollers
+#
+# Revision 1.84  2006/11/26 20:58:20  ncq
+# - try working around lacking wx.PopupWindow
+#
+# Revision 1.83  2006/11/26 14:51:19  ncq
+# - cleanup/improve test suite so we can get MacOSX nailed (down)
+#
+# Revision 1.82  2006/11/26 14:09:59  ncq
+# - fix sys.path when running standalone for test suite
+# - fix test suite
+#
+# Revision 1.81  2006/11/24 09:58:39  ncq
+# - cleanup
+# - make it really work when matcher is None
+#
+# Revision 1.80  2006/11/19 11:16:02  ncq
+# - remove self._input_was_selected
+#
+# Revision 1.79  2006/11/06 12:54:00  ncq
+# - we don't actually need self._input_was_selected thanks to self.data
+#
+# Revision 1.78  2006/11/05 16:10:11  ncq
+# - cleanup
+# - now really handle context
+# - add unset_context()
+# - stop timer in __init__()
+# - start timer in _on_set_focus()
+# - some u''-ification
+#
+# Revision 1.77  2006/10/25 07:24:51  ncq
+# - gmPG -> gmPG2
+# - match provider _SQL deprecated
+#
 # Revision 1.76  2006/07/19 20:29:50  ncq
 # - import cleanup
 #
@@ -870,9 +1007,6 @@ if __name__ == '__main__':
 # - faster scrolling
 # - wxEditableListBox ?
 
-# - press down only once to get into list
-# - moving between list members is too slow
-
 # - if non-learning (i.e. fast select only): autocomplete with match
 #   and move cursor to end of match
 #-----------------------------------------------------------------------------------------------
