@@ -2,19 +2,15 @@
 """
 #================================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/wxpython/gmProviderInboxWidgets.py,v $
-# $Id: gmProviderInboxWidgets.py,v 1.9 2006/05/28 16:19:54 ncq Exp $
-__version__ = "$Revision: 1.9 $"
+# $Id: gmProviderInboxWidgets.py,v 1.12 2007/01/04 22:52:34 ncq Exp $
+__version__ = "$Revision: 1.12 $"
 __author__ = "Karsten Hilbert <Karsten.Hilbert@gmx.net>"
 
 #import os.path, sys, re, time
 
-try:
-	import wxversion
-	import wx
-except ImportError:
-	from wxPython import wx
+import wx
 
-from Gnumed.pycommon import gmLog, gmI18N, gmDispatcher, gmSignals
+from Gnumed.pycommon import gmLog, gmI18N, gmDispatcher, gmSignals, gmTools
 from Gnumed.business import gmProviderInbox, gmPerson
 from Gnumed.wxpython import gmGuiHelpers
 from Gnumed.wxGladeWidgets import wxgProviderInboxPnl
@@ -50,7 +46,7 @@ class cProviderInboxPnl(wxgProviderInboxPnl.wxgProviderInboxPnl):
 		Welcome %(title)s %(lname)s !
 
 	Below find the new messages in your Inbox.
-""") % {'title': _me['title'], 'lname': _me['lastnames']}
+""") % {'title': gmTools.coalesce(_me['title'], gmPerson.map_gender2salutation(_me['gender'])), 'lname': _me['lastnames']}
 
 		self._msg_welcome.SetLabel(msg)
 	#--------------------------------------------------------
@@ -129,7 +125,7 @@ Leaving message in inbox.""") % handler_key,
 	def _on_delete_focussed_msg(self, evt):
 		inbox = gmProviderInbox.cProviderInbox()
 		if not inbox.delete_message(self.__focussed_msg[8]):
-			gmGuiHelpers.gm_beep_statustext (
+			gmGuiHelpers.gm_statustext (
 				_('Cannot remove message from Inbox.')
 			)
 			return False
@@ -151,6 +147,15 @@ Leaving message in inbox.""") % handler_key,
 		return True
 #============================================================
 # $Log: gmProviderInboxWidgets.py,v $
+# Revision 1.12  2007/01/04 22:52:34  ncq
+# - show proper salutation for people without title
+#
+# Revision 1.11  2006/12/17 20:46:24  ncq
+# - cleanup
+#
+# Revision 1.10  2006/11/24 10:01:31  ncq
+# - gm_beep_statustext() -> gm_statustext()
+#
 # Revision 1.9  2006/05/28 16:19:54  ncq
 # - repopulate_ui() needed for receive_focus() from plugin base class
 #
