@@ -1,11 +1,9 @@
-"""GnuMed SOAP related widgets.
-
-The code in here is independant of gmPG.
+"""GNUmed SOAP related widgets.
 """
 #============================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/wxpython/gmSOAPWidgets.py,v $
-# $Id: gmSOAPWidgets.py,v 1.74.2.1 2006/09/01 09:25:13 ncq Exp $
-__version__ = "$Revision: 1.74.2.1 $"
+# $Id: gmSOAPWidgets.py,v 1.88 2007/01/15 20:22:46 ncq Exp $
+__version__ = "$Revision: 1.88 $"
 __author__ = "Carlos Moro <cfmoro1976@yahoo.es>, K.Hilbert <Karsten.Hilbert@gmx.net>"
 __license__ = "GPL"
 
@@ -13,15 +11,11 @@ __license__ = "GPL"
 import types
 
 # 3rd party
-try:
-	import wxversion
-	import wx
-except ImportError:
-	from wxPython import wx
+import wx
 
-# GnuMed
-from Gnumed.pycommon import gmDispatcher, gmSignals, gmI18N, gmLog, gmExceptions, gmMatchProvider
-from Gnumed.wxpython import gmResizingWidgets, gmPhraseWheel, gmEMRStructWidgets, gmGuiHelpers, gmRegetMixin, gmVaccWidgets, gmEditArea
+# GNUmed
+from Gnumed.pycommon import gmDispatcher, gmSignals, gmI18N, gmLog, gmExceptions, gmMatchProvider, gmTools
+from Gnumed.wxpython import gmResizingWidgets, gmPhraseWheel, gmEMRStructWidgets, gmGuiHelpers, gmRegetMixin, gmEditArea
 from Gnumed.business import gmPerson, gmEMRStructItems, gmSOAPimporter
 
 _log = gmLog.gmDefLog
@@ -131,23 +125,46 @@ class cProgressNoteInputNotebook(wx.Notebook, gmRegetMixin.cRegetOnPaintMixin):
 	#--------------------------------------------------------
 	# public API
 	#--------------------------------------------------------
-	def add_editor(self, problem=None):
+	def add_editor(self, problem=None, allow_same_problem=False):
 		"""Add a progress note editor page."""
+
 		label = _('new episode')
+
 		if problem is not None:
 			emr = self.__pat.get_emr()
 			if isinstance(problem, gmEMRStructItems.cEpisode):
 				problem = emr.episode2problem(episode = problem)
 			elif isinstance(problem, gmEMRStructItems.cHealthIssue):
 				problem = emr.health_issue2problem(issue = problem)
-			if isinstance(problem, gmEMRStructItems.cProblem):
-				label = problem['problem']
-			else:
-				_log.Log(gmLog.lErr, 'cannot open progress note editor for [%s] (TypeError)' % str(problem))
-				return False
+			if not isinstance(problem, gmEMRStructItems.cProblem):
+				raise TypeError('cannot open progress note editor for [%s]' % str(problem))
+
+			label = problem['problem']
 			# FIXME: configure length
 			if len(label) > 23:
 				label = label[:20] + '...'
+
+		if not allow_same_problem:
+			# check for dupes
+			for page_idx in range(self.GetPageCount()):
+				page = self.GetPage(page_idx)
+				existing_problem = page.get_problem()
+				# unassociated
+				if existing_problem is None:
+					if problem is None:
+						self.SetSelection(page_idx)
+						return True
+					continue
+				# episodes
+				if problem['type'] == 'episode':
+					if problem['pk_episode'] == existing_problem['pk_episode']:
+						self.SetSelection(page_idx)
+						return True
+				if problem['type'] == 'issue':
+					if problem['pk_health_issue'] == existing_problem['pk_health_issue']:
+						self.SetSelection(page_idx)
+						return True
+
 		new_page = cResizingSoapPanel(parent = self, problem = problem)
 		return self.AddPage (
 			page = new_page,
@@ -337,20 +354,23 @@ class cNotebookedProgressNoteInputPanel(wx.Panel, gmRegetMixin.cRegetOnPaintMixi
 				issue = emr.problem2issue(problem)
 				last_encounter = emr.get_last_encounter(issue_id = issue['pk'])
 				if last_encounter is None:
-					last = issue['modified_when'].Format('%m/%Y')
+					last = issue['modified_when'].strftime('%m/%Y')
 				else:
-					last = last_encounter['last_affirmed'].Format('%m/%Y')
+					last = last_encounter['last_affirmed'].strftime('%m/%Y')
+				label = u'%s: %s "%s"' % (last, problem['l10n_type'], problem['problem'])
 			elif problem['type'] == 'episode':
 				epi = emr.problem2episode(problem)
 				last_encounter = emr.get_last_encounter(episode_id = epi['pk_episode'])
 				if last_encounter is None:
-					last = epi['episode_modified_when'].Format('%m/%Y')
+					last = epi['episode_modified_when'].strftime('%m/%Y')
 				else:
-					last = last_encounter['last_affirmed'].Format('%m/%Y')
-			item = _('%s "%s" (%s)') % (last, problem['problem'], problem['l10n_type'])
-			self.__LST_problems.Append(item, problem)
+					last = last_encounter['last_affirmed'].strftime('%m/%Y')
+				label = u'%s: %s "%s"%s' % (last, problem['l10n_type'], problem['problem'], gmTools.coalesce(epi['health_issue'], '', ' (%s)'))
+			self.__LST_problems.Append(label, problem)
 		splitter_width = self.__splitter.GetSizeTuple()[0]
 		self.__splitter.SetSashPosition((splitter_width / 2), True)
+		self.Refresh()
+		#self.Update()
 		return True
 	#--------------------------------------------------------
 	# event handling
@@ -409,40 +429,17 @@ class cNotebookedProgressNoteInputPanel(wx.Panel, gmRegetMixin.cRegetOnPaintMixi
 		"""
 		problem_idx = self.__LST_problems.GetSelection()
 		problem = self.__LST_problems.GetClientData(problem_idx)
-		emr = self.__pat.get_emr()
 
-		title = _('opening progress note editor')
-		msg = _('Cannot open progress note editor for\n\n'
-				'[%s].\n\n') % problem['problem']
-
-		if problem['type'] == 'issue':
-			# health issue selected: user wants to start new episode
-			if self.__soap_notebook.add_editor(problem = problem):
-				# FIXME: ask whether closing "previous" episodes for issue
-				return True
-			gmGuiHelpers.gm_show_error(aMessage = msg, aTitle = title)
-			return False
-
-		# must be episode, then
-		# editor for it already there ?
-		for page_idx in range(self.__soap_notebook.GetPageCount()):
-			page = self.__soap_notebook.GetPage(page_idx)
-			pnl_problem = page.get_problem()
-			# skip unassociated
-			if pnl_problem is None:
-				continue
-			# skip issues
-			if pnl_problem['type'] != 'episode':
-				continue
-			if pnl_problem['pk_episode'] == problem['pk_episode']:
-				# yes, so raise that editor (returns idx of old page)
-				self.__soap_notebook.SetSelection(page_idx)
-				return True
-
-		# no, add editor for episode
 		if self.__soap_notebook.add_editor(problem = problem):
 			return True
-		gmGuiHelpers.gm_show_error(aMessage = msg, aTitle = title)
+
+		gmGuiHelpers.gm_show_error (
+			aMessage = _(
+				'Cannot open progress note editor for\n\n'
+				'[%s].\n\n'
+			) % problem['problem'],
+			aTitle = _('opening progress note editor')
+		)
 		return False
 	#--------------------------------------------------------
 	def __on_save(self, event):
@@ -451,8 +448,11 @@ class cNotebookedProgressNoteInputPanel(wx.Panel, gmRegetMixin.cRegetOnPaintMixi
 		page_idx = self.__soap_notebook.GetSelection()
 		soap_nb_page = self.__soap_notebook.GetPage(page_idx)
 		if not soap_nb_page.save():
+			gmGuiHelpers.gm_statustext(_('Problem saving progress note: duplicate information ?'))
 			return False
 		self.__soap_notebook.DeletePage(page_idx)
+		# always keep one unassociated editor open
+		self.__soap_notebook.add_editor()
 		self.__refresh_problem_list()
 		return True
 	#--------------------------------------------------------
@@ -607,7 +607,6 @@ class cResizingSoapWin (gmResizingWidgets.cResizingWindow):
 		aoe = ''
 		rfe = ''
 		has_rfe = False
-		has_plan = False
 		soap_lines_contents = self.GetValue()
 		for line_content in soap_lines_contents.values():
 			if line_content.text.strip() == '':
@@ -618,82 +617,32 @@ class cResizingSoapWin (gmResizingWidgets.cResizingWindow):
 				gmSOAPimporter.soap_bundle_TEXT_KEY: line_content.text.rstrip()
 			})
 			if line_content.data.is_rfe:
-				rfe += line_content.text.rstrip()
 				has_rfe = True
-			if line_content.data.soap_cat == 'p':
-				has_plan = True
+				rfe += line_content.text.rstrip()
 			if line_content.data.soap_cat == 'a':
 				aoe += line_content.text.rstrip()
-		if not (has_rfe and has_plan):
-			msg = _('Progress note must have Reason for Encounter and Plan.')
-			gmGuiHelpers.gm_show_info(msg, _('saving progress note'), gmLog.lErr)
-			return False
 
-		# work out episode name
 		emr = self.__pat.get_emr()
-		episode = None
-		problem = self.__problem
+
 		# - new episode, must get name from narrative (or user)
-		if (problem is None) or (problem['type'] == 'issue'):
+		if (self.__problem is None) or (self.__problem['type'] == 'issue'):
+			# work out episode name
 			if len(aoe) != 0:
 				epi_name = aoe
 			else:
 				epi_name = rfe
-			if problem is None:
-				episode = emr.add_episode(episode_name = epi_name[:45], is_open = True)		# FIXME: un-hardcode length ?
-			else:
-				issue = emr.problem2issue(problem)
-				# FIXME: make ttl configurable
-				ttl = 90	# 90 days, 3 months
-				all_closed = issue.close_expired_episode(ttl=ttl)
 
-				if all_closed:
-					gmGuiHelpers.gm_beep_statustext(_('Closed episodes older than %s days on health issue [%s]') % (ttl, issue['description']))
-					# either error or non-expired open episode exists
-					# FIXME: all_closed not working
+			# new unassociated episode
+			new_episode = emr.add_episode(episode_name = epi_name[:45], pk_health_issue = None, is_open = True)
 
-				open_epis = emr.get_episodes(open_status = True)
-				open_epis = filter(lambda epi: epi['pk_health_issue'] == issue['pk'], open_epis)
-				if len(open_epis) > 1:
-						_log.Log(gmLog.lErr, 'there is more than one open episode for health issue [%s]' % str(issue))
-						for e in open_epis:
-							_log.Log(gmLog.lData, str(e))
+			if self.__problem is not None:
+				issue = emr.problem2issue(self.__problem)
+				if not gmEMRStructWidgets.move_episode_to_issue(episode = new_episode, target_issue = issue, save_to_backend = True):
+					print "error moving episode to issue"
 
-				if len(open_epis) == 1:
-						# need to ask user what to do
-						# - close old open episode and continue with new one ?
-						# - use old open episode as is ?
-						# - use old open episode but rename ?
-						#xxxxxxxxxxxxxxxxxxxx
-						
-						print "FIXME  len(open_epis) == 1"
-						d = wx.MessageDialog (
-							self,
-							_("There is an episode still open for this issue. Close the old episode (yes)\nRelink under old episode(no)\nThink about it (cancel)"),
-							caption = "WARNING !",
-							style = wx.YES| wx.NO | wx.CANCEL
-						)
-						answ = d.ShowModal()
-						if answ == wx.ID_YES:
-							open_epis[0]['episode_open'] = False
-							open_epis[0].save_payload()
-						elif answ == wx.ID_NO:
-							episode = open_epis[0]
-						else:
-							return False
-						
-				if episode is None:
-					# error, close all and hope things work out ...
-					issue.close_expired_episode(ttl=-1)
-					episode = emr.add_episode(episode_name = epi_name[:45], pk_health_issue = problem['pk_health_issue'], is_open = True)
-
-			if episode is None:
-				msg = _('Cannot create episode [%s] to save progress note under.' % epi_name)
-				gmGuiHelpers.gm_show_error(msg, _('saving progress note'), gmLog.lErr)
-				return False
-			epi_id = episode['pk_episode']
+			epi_id = new_episode['pk_episode']
 		else:
-			epi_id = problem['pk_episode']
+			epi_id = self.__problem['pk_episode']
 
 		# set up clinical context in progress note
 		encounter = emr.get_active_encounter()
@@ -959,8 +908,8 @@ if __name__ == "__main__":
 	import sys
 	_log = gmLog.gmDefLog
 	_log.SetAllLogLevels(gmLog.lData)
-	from Gnumed.pycommon import gmPG
-	gmPG.set_default_client_encoding('latin1')
+	from Gnumed.pycommon import gmPG2
+	gmPG2.set_default_client_encoding('UNICODE')
 	#--------------------------------------------------------
 	def get_narrative(pk_encounter=None, pk_health_issue = None, default_labels=None):
 		"""
@@ -1115,8 +1064,49 @@ if __name__ == "__main__":
 
 #============================================================
 # $Log: gmSOAPWidgets.py,v $
-# Revision 1.74.2.1  2006/09/01 09:25:13  ncq
-# - it is issue.close_expired_episode() not plural
+# Revision 1.88  2007/01/15 20:22:46  ncq
+# - move_episode_to_issue() is in gmEMRStructWidgets
+#
+# Revision 1.87  2007/01/15 13:05:38  ncq
+# - use move_episode_to_issue()
+#
+# Revision 1.86  2006/12/15 15:28:37  ncq
+# - signal problem saving progress note
+#
+# Revision 1.85  2006/11/28 20:53:41  ncq
+# - a missing Refresh()
+#
+# Revision 1.84  2006/11/26 17:13:17  ncq
+# - properly check for duplicate episode editors in add_editor when problem is None
+#
+# Revision 1.83  2006/11/24 10:01:31  ncq
+# - gm_beep_statustext() -> gm_statustext()
+#
+# Revision 1.82  2006/11/20 18:23:53  ncq
+# - smarten up add_editor() with allow_same_problem
+# - after save() open new unassociated editor if none there and refresh problem list
+#
+# Revision 1.81  2006/11/20 16:04:45  ncq
+# - improve problem list problem labels (show associated issue for episodes)
+#
+# Revision 1.80  2006/10/31 13:32:58  ncq
+# - cleanup
+# - require only rfe
+#
+# Revision 1.79  2006/10/25 07:46:44  ncq
+# - Format() -> strftime() since datetime.datetime does not have .Format()
+#
+# Revision 1.78  2006/10/25 07:25:38  ncq
+# - drop minimum soap entry as requested by user
+#
+# Revision 1.77  2006/10/23 13:23:04  ncq
+# - we don't need vacc widgets currently
+#
+# Revision 1.76  2006/10/08 11:08:42  ncq
+# - move to gmPG2 and only when testing
+#
+# Revision 1.75  2006/09/01 14:47:02  ncq
+# - fix typo
 #
 # Revision 1.74  2006/07/19 20:29:50  ncq
 # - import cleanup
