@@ -7,18 +7,14 @@ to anybody else.
 """
 #=========================================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/wxpython/gmStaffWidgets.py,v $
-# $Id: gmStaffWidgets.py,v 1.7.2.1 2006/09/02 20:54:59 ncq Exp $
-__version__ = "$Revision: 1.7.2.1 $"
+# $Id: gmStaffWidgets.py,v 1.13 2006/12/31 16:25:43 ncq Exp $
+__version__ = "$Revision: 1.13 $"
 __author__  = "K. Hilbert <Karsten.Hilbert@gmx.net>"
 __license__ = "GPL (details at http://www.gnu.org)"
 
-try:
-	import wxversion
-	import wx
-except ImportError:
-	from wxPython import wx
+import wx
 
-from Gnumed.pycommon import gmLog, gmPG
+from Gnumed.pycommon import gmLog, gmPG2, gmTools
 from Gnumed.business import gmPerson
 from Gnumed.wxpython import gmGuiHelpers
 from Gnumed.wxGladeWidgets import wxgAddPatientAsStaffDlg, wxgEditStaffListDlg
@@ -53,11 +49,9 @@ class cEditStaffListDlg(wxgEditStaffListDlg.wxgEditStaffListDlg):
 			row_num = self._LCTRL_staff.InsertStringItem(pos, label=staff['short_alias'])
 			self._LCTRL_staff.SetStringItem(index = row_num, col = 1, label = staff['db_user'])
 			self._LCTRL_staff.SetStringItem(index = row_num, col = 2, label = staff['role'])
-			self._LCTRL_staff.SetStringItem(index = row_num, col = 3, label = '%s %s, %s' % (staff['title'], staff['lastnames'], staff['firstnames']))
-			if staff['comment'] is None:
-				self._LCTRL_staff.SetStringItem(index = row_num, col = 4, label = '')
-			else:
-				self._LCTRL_staff.SetStringItem(index = row_num, col = 4, label = staff['comment'])
+			title = gmTools.coalesce(staff['title'], '')
+			self._LCTRL_staff.SetStringItem(index = row_num, col = 3, label = '%s %s, %s' % (title, staff['lastnames'], staff['firstnames']))
+			self._LCTRL_staff.SetStringItem(index = row_num, col = 4, label = gmTools.coalesce(staff['comment'], ''))
 			self._LCTRL_staff.SetStringItem(index = row_num, col = 5, label = '%s / %s' % (lbl_active[bool(staff['is_active'])], lbl_login[bool(staff['can_login'])]))
 			# color
 			if staff['is_active'] and staff['can_login']:
@@ -102,10 +96,7 @@ class cEditStaffListDlg(wxgEditStaffListDlg.wxgEditStaffListDlg):
 		self._TCTRL_name.SetValue('%s.%s %s' % (staff['title'], staff['firstnames'], staff['lastnames']))
 		self._TCTRL_alias.SetValue(staff['short_alias'])
 		self._TCTRL_account.SetValue(staff['db_user'])
-		if staff['comment'] is None:
-			self._TCTRL_comment.SetValue('')
-		else:
-			self._TCTRL_comment.SetValue(staff['comment'])
+		self._TCTRL_comment.SetValue(gmTools.coalesce(staff['comment'], ''))
 	#--------------------------------------------------------
 	def _on_listitem_deselected(self, evt):
 		self._btn_save.Enable(False)
@@ -125,27 +116,18 @@ class cEditStaffListDlg(wxgEditStaffListDlg.wxgEditStaffListDlg):
 		if conn is None:
 			return False
 
-		# 1) inactivate staff entry
+		# 1) activate staff entry
 		staff = gmPerson.cStaff(aPK_obj=pk_staff)
 		staff['is_active'] = True
 		staff.save_payload(conn=conn)				# FIXME: error handling
 
 		# 2) enable database account login
-		queries = [('select gm_create_user(%s, %s)', [staff['db_user'], 'flying wombat'])]
-		success, data = gmPG.run_commit2 (
+		rowx, idx = gmPG2.run_rw_queries (
 			link_obj = conn,
-			queries = queries,
+			queries = [{'cmd': u'select gm_create_user(%s, %s)', 'args': [staff['db_user'], 'flying wombat']}],
 			end_tx = True
 		)
 		conn.close()
-		if not success:
-			gmGuiHelpers.gm_show_error (
-				aMessage = _('Failed to activate GNUmed database user.'),
-				aTitle = _('Activating GNUmed staff member'),
-				aLogLevel = gmLog.lErr
-			)
-			return False
-
 		self.__init_ui_data()
 		return True
 	#--------------------------------------------------------
@@ -162,21 +144,12 @@ class cEditStaffListDlg(wxgEditStaffListDlg.wxgEditStaffListDlg):
 		staff.save_payload(conn=conn)				# FIXME: error handling
 
 		# 2) disable database account login
-		queries = [('select gm_disable_user(%s)', [staff['db_user']])]
-		success, data = gmPG.run_commit2 (
+		rows, idx = gmPG2.run_rw_queries (
 			link_obj = conn,
-			queries = queries,
+			queries = [{'cmd': u'select gm_disable_user(%s)', 'args': [staff['db_user']]}],
 			end_tx = True
 		)
 		conn.close()
-		if not success:
-			gmGuiHelpers.gm_show_error (
-				aMessage = _('Failed to disable GNUmed database user.'),
-				aTitle = _('Disabling GNUmed staff member'),
-				aLogLevel = gmLog.lErr
-			)
-			return False
-
 		self.__init_ui_data()
 		return True
 	#--------------------------------------------------------
@@ -220,7 +193,7 @@ class cAddPatientAsStaffDlg(wxgAddPatientAsStaffDlg.wxgAddPatientAsStaffDlg):
 		name = ident.get_active_name()
 		txt = _("""
   %s "%s" %s
-  born: %s""") % (name['first'], name['preferred'], name['last'], ident['dob'].Format(_('%Y-%m-%d')))
+  born: %s""") % (name['first'], name['preferred'], name['last'], ident['dob'].strftime('%Y-%m-%d'))
 		self._TXT_person.SetValue(txt)
 		txt = name['first'][:2] + name['last'][:2]
 		self._TXT_short_alias.SetValue(txt)
@@ -251,28 +224,36 @@ class cAddPatientAsStaffDlg(wxgAddPatientAsStaffDlg.wxgAddPatientAsStaffDlg):
 		pat = gmPerson.gmCurrentPatient()
 		queries = [
 			# database account
-			('select gm_create_user(%s, %s)', [self._TXT_account.GetValue(), self._TXT_password.GetValue()]),
+			{'cmd': u'select gm_create_user(%s, %s)', 'args': [self._TXT_account.GetValue(), self._TXT_password.GetValue()]},
 			# staff entry
-			('insert into dem.staff (fk_identity, fk_role, db_user, short_alias) values (%s, (select pk from dem.staff_role where name=\'doctor\'), %s, %s)', [pat.getID(), self._TXT_account.GetValue(), self._TXT_short_alias.GetValue()])
+			{'cmd': u"insert into dem.staff (fk_identity, fk_role, db_user, short_alias) values (%s, (select pk from dem.staff_role where name='doctor'), %s, %s)", 'args': [pat.getID(), self._TXT_account.GetValue(), self._TXT_short_alias.GetValue()]}
 		]
-		success, data = gmPG.run_commit2 (
-			link_obj = conn,
-			queries = queries,
-			end_tx = True
-		)
-		if not success:
-			gmGuiHelpers.gm_show_error (
-				aMessage = _('Failed to create new GNUmed database user.\n\nTherefore cannot add new staff member.'),
-				aTitle = _('Adding GNUmed staff member'),
-				aLogLevel = gmLog.lErr
-			)
-			return False
-		self.Close()
+		rows, idx = gmPG2.run_rw_queries(link_obj = conn, queries = queries, end_tx = True)
+		if self.IsModal():
+			self.EndModal(wx.ID_OK)
+		else:
+			self.Close()
 #==========================================================================
 # $Log: gmStaffWidgets.py,v $
-# Revision 1.7.2.1  2006/09/02 20:54:59  ncq
-# - be careful about staff['comment']
-# - use gmGuiHelpers.get_dbowner_connection()
+# Revision 1.13  2006/12/31 16:25:43  ncq
+# - strftime() does not take unicode
+#
+# Revision 1.12  2006/11/24 14:23:41  ncq
+# - EndModal() needs wx.ID_*
+#
+# Revision 1.11  2006/10/31 13:30:27  ncq
+# - use gmPG2
+#
+# Revision 1.10  2006/10/25 07:46:44  ncq
+# - Format() -> strftime() since datetime.datetime does not have .Format()
+#
+# Revision 1.9  2006/10/25 07:21:57  ncq
+# - no more gmPG
+#
+# Revision 1.8  2006/09/03 11:32:10  ncq
+# - clean up wx import
+# - use gmTools.coalesce()
+# - use gmGuiHelpers.get_dbowner_connection instead of local crap copy
 #
 # Revision 1.7  2006/06/17 16:45:19  ncq
 # - only insert column labels once
